unit AnnualData;
// 130903 RJC
// 130722 RJC Remove memo fields a/c lock-up of program & computer;
//            It is assumed some errant values cause the fault.
// 130414 RJC SQL statement corrected to that defined by Robyn Douglas.
//            INSERT..VALUES (..params..)..DataSource-control doesn't work
//            so each param set under manual control.

interface

uses
  dbtables, SysUtils, Controls, Dialogs, db;
  
procedure ImportDataWithRefund(const nBatchID: integer;const sPeriod: string;
  const dStartDate, dEndDate: TDateTime; const sBatchTypeName: string;
  const nBatchTypeID: integer);

implementation

uses
  DataMod, Main, GenFns, AnnualCharges, MyMessage;

function UpdateFaultsTable(const nBatchID: integer;const sPeriod: string): boolean;
var
  qFQuetzal : TQuery;
  dsFQuetzal: TDataSource;
  qFault : TQuery;
  sBatchID : string;
  sBillingPeriod: string;
  sYear    : string;
  sMonth   : string;
  sMinDate : string;
  sMaxDate : string;
  MessageForm : TMessagesForm;
begin
  sBatchID := IntToStr(nBatchID);
  sBillingPeriod := sPeriod;
  sYear  := Copy(sBillingPeriod,1,4);
  sMonth := Copy(sBillingPeriod,6,2);
  sMinDate := sYear+'.'+sMonth+'.01';       // First day of the month.
  if (sMonth = '12') then begin
    sMonth := '01';                         // January...
    sYear  := IntToStr(StrToInt(sYear)+1);  // ...next Year.
  end
  else begin
    sMonth := Right('0'+IntToStr(StrToInt(sMonth)+1),2);
  end;
  sMaxDate := sYear+'.'+sMonth+'.01';       // First day of Next month.

  MessageForm := TMessagesForm.Create(MainForm);
  MessageForm.Caption := 'Fault Report for '+sBillingPeriod;
  MessageForm.Msg.Caption := 'Reading details. Please wait ...';
  MessageForm.Show;
  MainForm.ProcessWinMessages;
  qFQuetzal  := TQuery.Create(MainForm);
  // Remove any data that conflicts with the new information.
  with qFQuetzal do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('DELETE FROM tFaultDetails');
    SQL.Add('WHERE BatchID = '+sBatchID);
    SQL.Add('AND BillingPeriod LIKE '''+sBillingPeriod+'''');
    ExecSQL;
    SQL.Clear;
    // Setup the query with all of the data to be copied to PreProcessor.
    // NB A separate program updates Availability, AvailabilityTargetMet.
    DatabaseName := 'dbQuetzal';
    SQL.Add('Select Call.CallNumber, Call.LastName, Call.FirstName,');
    SQL.Add(' Call.Ref2, Call.CallSubject1, Call.CallSubject2, Call.CallSubject3,');
    SQL.Add(' Call.Logon, Call.ConfigId, Call.ConfigCallPrompt, Call.SupportGroup,');
    SQL.Add(' Call.ResolutionCode1, Call.ResolutionCode2, Call.ResolutionCode3,');
    SQL.Add(' Call.LogDatTim, Call.AllDatTim, Call.LastDatTim, Call.SLT1,');
    SQL.Add(' Call.SLTUnique1, Call.LogonCode,');
    SQL.Add(' DATEDIFF(mi, Call.AllDatTim, Call.LastDatTim) AS QuetzalDownTime,');
  //  SQL.Add(' DATEDIFF(mi, Call.AllDatTim, Call.LastDatTim) AS ReportedDowntime, '+
  //    'Call_Comments.Comments AS DowntimeComments,'); // 130722 RJC Remove memos.
    SQL.Add(' DATEDIFF(mi, Call.AllDatTim, Call.LastDatTim) AS ReportedDowntime,');
    SQL.Add(' Call.OrgLvlCode AS ShipTo,');
    SQL.Add(' vwMetroWANEvent49.EventCode as EventCode, Call.IthelpUser, '+
              'vwMetroWANEvent49.EventDateTime,');
  //  SQL.Add(' (1) AS IsCurrentEvent, Call.comments,');// 130722 RJC Remove memos.
  //  SQL.Add(' (1) AS IsCurrentEvent,');
//    SQL.Add(' SLT.TarFixWks, SLT.TarFixHrs, SLT.TarFixMins, SLT.TarResWks, SLT.TarResHrs,');
//    SQL.Add(' SLT.TarResMins, vwMetroWANEvent49.Information AS EventName, '+
    SQL.Add(' vwMetroWANEvent49.Information AS EventName, '+
              'Configuration.Ref2 AS ServiceID,');
//    SQL.Add(' (CASE WHEN Call.SLTUnique1 IN (153, 202, 204, 294) THEN ''Station''');
//    SQL.Add('       WHEN Call.SLTUnique1 IN (154, 155, 156, 203, 293) THEN ''Network''');
//    SQL.Add('       WHEN Call.SLT1 LIKE ''%Station'' THEN ''Station''');
//    SQL.Add('       WHEN Call.SLT1 LIKE ''%Other'' THEN ''Network''');
    SQL.Add(' (CASE WHEN Call.SLTUnique1 IN (153, 157, 294) THEN ''Station''');
    SQL.Add('       WHEN Call.SLTUnique1 IN (154, 155, 156, 159, 293) THEN ''Network''');
    SQL.Add('       WHEN Call.SLT1 LIKE ''%Zone Station%'' THEN ''Station''');
    SQL.Add('       WHEN Call.SLT1 LIKE ''%Zone Other%'' THEN ''Network''');
    SQL.Add('       WHEN Call.SLT1 LIKE ''%Zone'' THEN ''Network''');
    SQL.Add('       ELSE ''Unknown'' END) AS AvailabilityType');
{
    20130414 RJC.
    This section is constant for all transactions and updated using Params so
    there is no need to set values in the data resultset.
    SQL.Add('       ELSE ''Unknown'' END) AS AvailabilityType,');
    SQL.Add(' (CASE WHEN Call.SLT1 LIKE ''INNER ZONE STATION'' THEN 99.95');
    SQL.Add('       WHEN Call.SLT1 LIKE ''INNER ZONE OTHER'' THEN 99');
    SQL.Add('       WHEN Call.SLT1 LIKE ''MIDDLE ZONE STATION'' THEN 99.95');
    SQL.Add('       WHEN Call.SLT1 LIKE ''MIDDLE ZONE OTHER'' THEN 99');
    SQL.Add('       WHEN Call.SLT1 LIKE ''OUTER ZONE STATION'' THEN 99');
    SQL.Add('       WHEN Call.SLT1 LIKE ''DATACENTRE LINK'' THEN 99.95');
    SQL.Add('       ELSE 100 END) AS Availability,');
    SQL.Add(' ('+sBatchID+') AS BatchID, '+
      '('''+sBillingPeriod+''') AS BillingPeriod, (0) AS PPTaken, '+
      '(1) AS AvailabilityTargetMet');
}
    SQL.Add('FROM Call');
    SQL.Add(' INNER JOIN vwMetroWANEvent49 ON '+
      'Call.CallNumber = vwMetroWANEvent49.CallNumber');
    SQL.Add(' LEFT OUTER JOIN Configuration '+
      'ON Call.ConfigCode = Configuration.ConfigCode');
    SQL.Add(' LEFT OUTER JOIN Config_Classification ON '+
      'Configuration.ConfigClassificationUnique = '+
      'Config_Classification.ConfigClassificationUnique');
//    SQL.Add(' LEFT OUTER JOIN SLT ON Call.SLTUnique1 = SLT.SLTUnique');
  // 130722 RJC Remove memos:
  //    SQL.Add(' LEFT OUTER JOIN Call_Comments ON Call.CallNumber = Call_Comments.CallNumber');
    SQL.Add('WHERE  (Call.CallStatus > 4) AND ');
    SQL.Add('   ((Config_Classification.ClassName LIKE ''KAMCO'') OR '+
                       ' (Call.CallSubject2 LIKE ''KAMCO%'')) AND ');
    SQL.Add('   (Configuration.Ref2 IS NOT NULL) AND ');           // ServiceID.
    SQL.Add('   (NOT (Call.CallSubject1 LIKE ''c-%'')) AND '+
                    '(NOT (Call.CallSubject1 = ''SIMCARD'')) AND '+
                    '(NOT (Call.CallSubject2 = ''NETWORK AUTOMATION''))');
    SQL.Add('   AND ((Call.LogDatTim >= CONVERT(DATETIME, '''+sMinDate+''', 102)) AND'+
                   ' (Call.LogDatTim <  CONVERT(DATETIME, '''+sMaxDate+''', 102)))');
    SQL.Add('ORDER BY Call.CallNumber');
    Open;
  end;
  if qFQuetzal.Eof then begin
    MessageDlg('No entries found in Quetzal for faults.', mtInformation, [mbOk], 0);
    Result := False;
  end
  else begin
{// temp start
FileQuery(qFQuetzal, 'qFQuetzal');
// temp end.   }
    MessageForm.Msg.Caption := 'Saving details. Please wait ...';
    MainForm.ProcessWinMessages;
    dsFQuetzal := TDataSource.Create(MainForm);
    dsFQuetzal.DataSet := qFQuetzal;
    // Establish the Insert statement with the data to be saved -
    // this is done like a master-slave relationship.
    qFault     := TQuery.Create(MainForm);
    with qFault do begin
      DatabaseName := 'dbPPdata';
      SQL.Add('Insert into tFaultDetails');
      SQL.Add(' (CallNumber, FirstName, LastName,');
      SQL.Add('  Ref2, CallSubject1, CallSubject2, CallSubject3,');
      SQL.Add('  Logon, ConfigId, ConfigCallPrompt, SupportGroup,');
      SQL.Add('  ResolutionCode1, ResolutionCode2, ResolutionCode3,');
      SQL.Add('  LogDatTim, AllDatTim, LastDatTim, SLT1,');
      SQL.Add('  SLTUnique, LogonCode,');
  // 130722 RJC Remove memos:
  //    SQL.Add('  ReportedDowntime, DowntimeComments,');
      SQL.Add('  ReportedDowntime,');
      SQL.Add('  ShipTo,');
      SQL.Add('  EventCode, IthelpUser, EventDateTime,');
  // 130722 RJC Remove memos:
  //    SQL.Add('  IsCurrentEvent, comments, TarFixWks, TarFixHrs,');
      SQL.Add('  IsCurrentEvent, TarFixWks, TarFixHrs,');
      SQL.Add('  TarFixMins, TarResWks, TarResHrs,');
      SQL.Add('  TarResMins, EventName, ServiceID,');
      SQL.Add('  QuetzalDownTime, AvailabilityType, Availability,');
      SQL.Add('  BatchID, BillingPeriod, PPTaken, AvailabilityTargetMet)');
      SQL.Add('VALUES (:CallNumber, :FirstName, :LastName,');
      SQL.Add('  :Ref2, :CallSubject1, :CallSubject2, :CallSubject3,');
      SQL.Add('  :Logon, :ConfigId, :ConfigCallPrompt, :SupportGroup,');
      SQL.Add('  :ResolutionCode1, :ResolutionCode2, :ResolutionCode3,');
      SQL.Add('  :LogDatTim, :AllDatTim, :LastDatTim, :SLT1,');
      SQL.Add('  :SLTUnique, :LogonCode,');
  //      SQL.Add('  :ReportedDowntime, :DowntimeComments,'); // 130722 RJC Remove memos.
      SQL.Add('  :ReportedDowntime,');
      SQL.Add('  :ShipTo,');
      SQL.Add('  :EventCode, :IthelpUser, :EventDateTime,');
  //      SQL.Add('  :IsCurrentEvent, :Comments, :TarFixWks, :TarFixHrs,');
                                      // 130722 RJC Remove memos.
      SQL.Add('  :IsCurrentEvent, :TarFixWks, :TarFixHrs,');
      SQL.Add('  :TarFixMins, :TarResWks, :TarResHrs,');
      SQL.Add('  :TarResMins, :EventName, :ServiceID,');
      SQL.Add('  :QuetzalDownTime, :AvailabilityType, :Availability,');
      SQL.Add('  :BatchID, :BillingPeriod, :PPTaken, :AvailabilityTargetMet)');
    end;  // qFault.
    // Set constant values once before saving all.
    qFault.ParamByName('BatchID').AsInteger := StrToInt(sBatchID);
    qFault.ParamByName('BillingPeriod').AsString := sBillingPeriod;
    qFault.ParamByName('PPTaken').AsBoolean := FALSE;                // initial.
    qFault.ParamByName('IsCurrentEvent').AsInteger := 1;             // default.
    qFault.ParamByName('Availability').AsFloat := 100.0;             // default.
    qFault.ParamByName('AvailabilityTargetMet').AsBoolean := TRUE;   // default.
    qFault.ParamByName('TarFixWks').AsFloat := 0;
    qFault.ParamByName('TarFixHrs').AsFloat := 0;
    qFault.ParamByName('TarFixMins').AsFloat := 0;
    qFault.ParamByName('TarResWks').AsFloat := 0;
    qFault.ParamByName('TarResHrs').AsFloat := 0;
    qFault.ParamByName('TarResMins').AsFloat := 0;
    // Now read each record of data and save all to tDataCollection.
    while not qFQuetzal.Eof do begin
      with qFault do begin
        ParamByName('CallNumber').AsString :=
              CleanString(qFQuetzal.FieldByName('CallNumber').AsString);
        ParamByName('FirstName').AsString := 
              CleanString(qFQuetzal.FieldByName('FirstName').AsString);
        ParamByName('LastName').AsString := 
              CleanString(qFQuetzal.FieldByName('LastName').AsString);
        ParamByName('Ref2').AsString := 
              CleanString(qFQuetzal.FieldByName('Ref2').AsString);
        ParamByName('CallSubject1').AsString :=
              CleanString(qFQuetzal.FieldByName('CallSubject1').AsString);
        ParamByName('CallSubject2').AsString := 
              CleanString(qFQuetzal.FieldByName('CallSubject2').AsString);
        ParamByName('CallSubject3').AsString := 
              CleanString(qFQuetzal.FieldByName('CallSubject3').AsString);
        ParamByName('Logon').AsString :=              // Save max 32 chars.
              Copy(Trim(
              CleanString(qFQuetzal.FieldByName('Logon').AsString)
              ),1,32);
        ParamByName('ConfigId').AsString :=
              CleanString(qFQuetzal.FieldByName('ConfigId').AsString);
        ParamByName('ConfigCallPrompt').AsString :=   // Save max 15 chars.
              Copy(Trim(
              CleanString(qFQuetzal.FieldByName('ConfigCallPrompt').AsString)
              ),1,15);
        ParamByName('SupportGroup').AsString :=
              CleanString(qFQuetzal.FieldByName('SupportGroup').AsString);
        ParamByName('ResolutionCode1').AsString :=
              CleanString(qFQuetzal.FieldByName('ResolutionCode1').AsString);
        ParamByName('ResolutionCode2').AsString :=
              CleanString(qFQuetzal.FieldByName('ResolutionCode2').AsString);
        ParamByName('ResolutionCode3').AsString := 
              CleanString(qFQuetzal.FieldByName('ResolutionCode3').AsString);
        ParamByName('LogDatTim').AsDateTime :=
              qFQuetzal.FieldByName('LogDatTim').AsDateTime;
        ParamByName('AllDatTim').AsDateTime :=
              qFQuetzal.FieldByName('AllDatTim').AsDateTime;
        ParamByName('LastDatTim').AsDateTime :=
              qFQuetzal.FieldByName('LastDatTim').AsDateTime;
        ParamByName('SLT1').AsString := 
              CleanString(qFQuetzal.FieldByName('SLT1').AsString);
        ParamByName('SLTUnique').AsFloat :=
              qFQuetzal.FieldByName('SLTUnique1').AsFloat;
        ParamByName('LogonCode').AsFloat :=
              qFQuetzal.FieldByName('LogonCode').AsFloat;
        ParamByName('ReportedDowntime').AsInteger :=
              qFQuetzal.FieldByName('ReportedDowntime').AsInteger;
    // 130722 RJC Remove memos:
    //    ParamByName('DowntimeComments').AsMemo := qFQuetzal.FieldByName('ReportedDowntime').AsString;
        ParamByName('ShipTo').AsString :=
              CleanString(qFQuetzal.FieldByName('ShipTo').AsString);
        ParamByName('EventCode').AsFloat :=
              qFQuetzal.FieldByName('EventCode').AsFloat;
        ParamByName('IthelpUser').AsString := 
              CleanString(qFQuetzal.FieldByName('IthelpUser').AsString);
        ParamByName('EventDateTime').AsDateTime :=
              qFQuetzal.FieldByName('EventDateTime').AsDateTime;
//        ParamByName('IsCurrentEvent').AsFloat :=
//              qFQuetzal.FieldByName('IsCurrentEvent').AsFloat;
//
    // 130722 RJC Remove memos:
    //    ParamByName('Comments').AsMemo := qFQuetzal.FieldByName('Comments').AsString;
// Following set to 0 as links to SLT1 table are unreliable & RLD recalculates.        
//        if VarIsNull(qFQuetzal['TarFixWks']) then
//          ParamByName('TarFixWks').AsFloat := 0
//        else
//          ParamByName('TarFixWks').AsFloat :=
//                qFQuetzal.FieldByName('TarFixWks').AsFloat;
//        if VarIsNull(qFQuetzal['TarFixHrs']) then
//          ParamByName('TarFixHrs').AsFloat := 0
//        else
//          ParamByName('TarFixHrs').AsFloat :=
//                qFQuetzal.FieldByName('TarFixHrs').AsFloat;
//        if VarIsNull(qFQuetzal['TarFixMins']) then
//          ParamByName('TarFixMins').AsFloat := 0
//        else
//          ParamByName('TarFixMins').AsFloat :=
//                qFQuetzal.FieldByName('TarFixMins').AsFloat;
//        if VarIsNull(qFQuetzal['TarResWks']) then
//          ParamByName('TarResWks').AsFloat := 0
//        else
//          ParamByName('TarResWks').AsFloat :=
//                qFQuetzal.FieldByName('TarResWks').AsFloat;
//        if VarIsNull(qFQuetzal['TarResHrs']) then
//          ParamByName('TarResHrs').AsFloat := 0
//        else
//          ParamByName('TarResHrs').AsFloat :=
//                qFQuetzal.FieldByName('TarResHrs').AsFloat;
//        if VarIsNull(qFQuetzal['TarResMins']) then
//          ParamByName('TarResMins').AsFloat := 0
//        else
//          ParamByName('TarResMins').AsFloat :=
//                FQuetzal.FieldByName('TarResMins').AsFloat;
        ParamByName('EventName').AsString := 
              CleanString(qFQuetzal.FieldByName('EventName').AsString);
        ParamByName('ServiceID').AsString := 
              CleanString(qFQuetzal.FieldByName('ServiceID').AsString);
        ParamByName('QuetzalDownTime').AsInteger :=
              qFQuetzal.FieldByName('QuetzalDownTime').AsInteger;
        ParamByName('AvailabilityType').AsString := 
              CleanString(qFQuetzal.FieldByName('AvailabilityType').AsString);
//        ParamByName('Availability').AsFloat :=
//              qFQuetzal.FieldByName('Availability').AsFloat;

        Prepare;
        ExecSQL;
        qFQuetzal.Next;
      end;  // qFault.
    end;  // qFQuetzal.Eof.

    // Finally, fill tFaultDetails.ServiceClient with tBatchType.Description.
    with qFault do begin
      SQL.Clear;
      SQL.Add('UPDATE tFaultDetails');
      SQL.Add('SET ServiceClient = BT.Description');
      SQL.Add('FROM (SELECT tBatchType.Description');
      SQL.Add('FROM tBatchType');
      SQL.Add('INNER JOIN tBatch ON tBatchType.ID = tBatch.BatchTypeID');
      SQL.Add('WHERE tBatch.ID = '+sBatchID+') AS BT');
      SQL.Add('WHERE tFaultDetails.BatchID = '+sBatchID);
      ExecSQL;
    end;
    // Finished - clean up.
    qFault.Close;
    qFault.Free;
    dsFQuetzal.Free;
    Result := True;
  end;
  // Finished - clean up.
  qFQuetzal.Close;
  qFQuetzal.Free;
  MessageForm.Hide;
  MainForm.ProcessWinMessages;
  MessageForm.Close;
  MessageForm.Free;
end;  // UpdateFaultsTable.

procedure ImportDataWithRefund(const nBatchID: integer;const sPeriod: string;
  const dStartDate, dEndDate: TDateTime; const sBatchTypeName: string;
  const nBatchTypeID: integer);
var
  qry : TQuery;
  sBatchID : string;
  lContinue : boolean;
begin
  qry := TQuery.Create(MainForm);
  qry.DatabaseName := 'dbPPdata';
  sBatchID := IntToStr(nBatchID);
  // Check if tFaultDetails populated.
  with qry do begin
    Close;
    SQL.Clear;
    SQL.Add('SELECT ID');
    SQL.Add('FROM tFaultDetails');
    SQL.Add('WHERE BatchID = '+sBatchID);
    SQL.Add('AND BillingPeriod LIKE '''+sPeriod+'''');
    SQL.Add('AND PPTaken = 1');
    Open;
    lContinue := False;
    if Eof then begin
      // Not processed - see if read.
      Close;
      SQL.Strings[4] := 'AND (PPTaken IS NULL OR PPTaken = 0)';
      Open;
      if Eof then begin
        // Not read, or no faults registered this period.
        if (MessageDlg('No faults imported. '+
          'Do you want to import the Fault Details?',
            mtConfirmation, [mbYes, mbNo], 0) = mrYes) then begin
          UpdateFaultsTable(nBatchID, sPeriod);
          lContinue := True;
        end
        else begin
          MessageDlg('Faults must be imported before '+
            'proceeding, so import abandoned. Try again later.',
            mtWarning, [mbOk], 0);
        end;  // MessageDlg.
      end
      else begin
        // Faults registered this period, are they to be reloaded?
        if (MessageDlg('Do you want to Reload the Fault Details '+
          'from Quetzal?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then begin
          UpdateFaultsTable(nBatchID, sPeriod);
        end;
        lContinue := True;
      end;
    end
    else begin
      MessageDlg('Faults have already been processed. Import abandoned.',
        mtWarning, [mbOk], 0);
    end;
    Close;
  end;  // qry.
  if lContinue then begin
    // First, determine and save the annual charges.
    AnnualChargesInvoice( nBatchID, dStartDate,
      dEndDate, sBatchTypeName, nBatchTypeID);
    with qry do begin
      // Next, assume all services are 100% available - to be qualified soon.
      Close;
      SQL.Clear;
      SQL.Add('UPDATE tDataCollection');
      SQL.Add('SET Var02 = ''100''');
      SQL.Add('WHERE tDataCollection.BatchID = '+sBatchID);
      ExecSQL;
      // Now, add availability into Var02.
      SQL.Clear;
      SQL.Add('UPDATE tDataCollection');
      SQL.Add('SET Var02 = UPD.Var02');
//      SQL.Add('FROM (SELECT DC.ID, CAST(fd.Availability AS varchar) AS Var02');
//      SQL.Add('	FROM tFaultDetails fd');
//      SQL.Add('	INNER JOIN tDataCollection dc ON ((fd.ServiceID = dc.ServiceID)');
//      SQL.Add('	 AND (fd.BatchID = dc.BatchID))');
      SQL.Add('FROM (SELECT TmpDC.ID, TmpDC.Var02');
      SQL.Add('	FROM (SELECT TOP 10000 DC.ID, CAST(fd.Availability AS varchar) AS Var02, '+
                     'fd.ServiceID AS fdServiceID');
      SQL.Add('  FROM tDataCollection dc');
      SQL.Add('  LEFT JOIN tFaultDetails fd ON ((dc.ServiceID = fd.ServiceID) '+
                      'AND (dc.BatchID = fd.BatchID))');
      SQL.Add('  WHERE dc.BatchID = '+sBatchID);
      SQL.Add('  ORDER BY fd.ServiceID) AS TmpDC');
      SQL.Add(' WHERE TmpDC.fdServiceID IS NOT NULL) AS UPD');
{
      SQL.Add('	FROM tDataCollection dc');
      SQL.Add('	LEFT JOIN tFaultDetails fd ON ((dc.ServiceID = fd.ServiceID)');
      SQL.Add('	 AND (dc.BatchID = fd.BatchID))');
      SQL.Add(' WHERE dc.BatchID = '+sBatchID+'
      SQL.Add('  AND ) AS UPD');
}
      SQL.Add('WHERE tDataCollection.ID = UPD.ID');
      ExecSQL;
      // And now we can add any refunds a/c poor performance.
      SQL.Clear;
      SQL.Add('INSERT INTO tDataCollection');
      SQL.Add('(SequenceNo, ServiceID, ServiceType, TxnDate,');
      SQL.Add(' TxnTime, TransactionTypeID, GSTFlag,');
      SQL.Add(' AmountExGST, GSTAmount, AmountIncGST,');
      SQL.Add(' FromDate, ToDate, BatchID, Var01, Var02)');
      SQL.Add('SELECT DC.SequenceNo, DC.ServiceID, DC.ServiceType,');
      SQL.Add(' DC.TxnDate, DC.TxnTime, DC.TransactionTypeID,');
      SQL.Add(' DC.GSTFlag, (DC.AmountExGST * -1) AS "AmountExGST",');
      SQL.Add(' (DC.GSTAmount * -1) AS "GSTAmount",');
      SQL.Add(' (DC.AmountIncGST * -1) AS "AmountIncGST",');
      SQL.Add(' DC.FromDate, DC.ToDate, DC.BatchID,');
      SQL.Add(' (''Refund'') AS "Var01",');
      SQL.Add(' CAST(FD.Availability AS varchar) AS "Var02"');
      SQL.Add('FROM tDataCollection DC');
      SQL.Add('LEFT JOIN tFaultDetails FD ON ((DC.ServiceID = FD.ServiceID)');
      SQL.Add(' AND (DC.BatchID = FD.BatchID))');
      SQL.Add('WHERE DC.BatchID = '+sBatchID);
      SQL.Add('AND ((FD.AvailabilityTargetMet = NULL) OR '+
        '(FD.AvailabilityTargetMet = 0))');
      ExecSQL;
      // Now update the values stored in the tBatch table.
      // Define the TotalRevenue invoiced (including rebates).
      SQL.Clear;
      SQL.Add('UPDATE tBatch');
      SQL.Add('SET TotalRevenue = BatchRate.TotalRevenue');
      SQL.Add('FROM (SELECT Sum(tDataCollection.AmountExGST) '+
        'AS "TotalRevenue"');
      SQL.Add('FROM tDataCollection');
      SQL.Add('WHERE tDataCollection.BatchID = '+ sBatchID +') AS BatchRate');
      SQL.Add('WHERE tBatch.ID = '+ sBatchID);
      ExecSQL;
      // Define the number of rebates included.
      SQL.Clear;
      SQL.Add('UPDATE tBatch');
      SQL.Add('SET InvoiceNo = BatchRate.InvoiceNo');
      SQL.Add('FROM (SELECT COUNT(*) AS "InvoiceNo"');
      SQL.Add('FROM tDataCollection');
      SQL.Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      SQL.Add('AND tDataCollection.AmountExGST < 0) AS BatchRate');
      SQL.Add('WHERE tBatch.ID = '+ sBatchID);
      ExecSQL;
      // Define the total value of the rebates.
      SQL.Clear;
      SQL.Add('UPDATE tBatch');
      SQL.Add('SET TotalCredits = BatchRate.TotalCredits');
      SQL.Add('FROM (SELECT Sum(tDataCollection.AmountExGST) AS "TotalCredits"');
      SQL.Add('FROM tDataCollection');
      SQL.Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      SQL.Add('AND tDataCollection.AmountExGST < 0) AS BatchRate');
      SQL.Add('WHERE tBatch.ID = '+ sBatchID);
      ExecSQL;
      MAINFORM.PopulateBatchScreen;
    end;  // qry.
  end;  // lContinue.
end;  // ImportDataWithRefund.

end.
