unit CurrentRentCalls;

interface

uses
  classes, dbtables, SysUtils;

procedure UpdateCurrentRentCalls(const nBatch: integer);

implementation

uses
  DataMod, Main;

procedure UpdateCurrentRentCalls(const nBatch: integer);
// Update the tServiceID fields:
//        PreviousRent, PreviousCalls, CurrentRent, CurrentCalls.
// Called when tTransaction transactions are created from tDataCollection data.
var
  qCurrentVals : TQuery;
  slQry : TStringList;
  sBatch : string;
begin
  qCurrentVals := TQuery.Create(MainForm);
  qCurrentVals.DatabaseName := 'dbPPdata';
  slQry := TStringList.Create;
  sBatch := IntToStr(nBatch);
  // First, make sure that there are not any Null amounts to kill the resuls.
  with slQry do begin
    Add('UPDATE tTransaction');
    Add('SET tTransaction.AmountExGST = 0');
    Add('WHERE tTransaction.BatchID = '+ sBatch);
    Add('AND tTransaction.AmountExGST IS NULL');
  end;  // slQry.
  qCurrentVals.SQL := slQry;
  qCurrentVals.ExecSQL;
  // Copy current rent & calls to previous rent & calls in tServiceID.
  with slQry do begin
    Clear;
    Add('UPDATE tServiceID');
    Add('SET tServiceID.PreviousRent = tServiceID.CurrentRent,');
    Add(' tServiceID.PreviousCalls = tServiceID.CurrentCalls');
    Add('WHERE tServiceID.ID IN');
    Add(' (SELECT DISTINCT tTransaction.ServiceID');
    Add(' FROM tTransaction');
    Add(' WHERE tTransaction.BatchID = '+ sBatch);
    Add(' AND tTransaction.RevenueFlag <> 1)');
  end;  // slQry.
  qCurrentVals.SQL := slQry;
  qCurrentVals.ExecSQL;
  // Save new values to current rent & calls.
  with slQry do begin
    Clear;
    Add('UPDATE tServiceID');
    Add('SET tServiceID.CurrentRent = TT.RentAmt,');
    Add(' tServiceID.CurrentCalls = TT.CallAmt');
    Add('FROM (SELECT tTransaction.ServiceID, tTransaction.ServiceTypeID,');
    Add(' Sum(CASE tTransactionGroup.Description WHEN ''RENT'' '+
      'THEN tTransaction.AmountExGST ELSE 0 END) AS RentAmt,');
    Add(' Sum(CASE tTransactionGroup.Description WHEN ''CALL'' '+
      'THEN tTransaction.AmountExGST ELSE 0 END) AS CallAmt');
    Add('FROM ((tTransaction');
    Add('LEFT JOIN tServiceID ON tTransaction.ServiceID = tServiceID.ID)');
    Add('LEFT JOIN tTransactionType '+
      'ON tTransaction.TransactionTypeID = tTransactionType.ID)');
    Add('LEFT JOIN tTransactionGroup '+
      'ON tTransactionType.TransactionGroupID = tTransactionGroup.ID');
    Add('WHERE tTransaction.BatchID = '+ sBatch);
    Add('AND tTransaction.RevenueFlag <> 1');
    Add('AND tServiceID.ServiceTypeID = tTransaction.ServiceTypeID');
    Add('GROUP BY tTransaction.ServiceID, tTransaction.ServiceTypeID) AS TT');
    Add('WHERE tServiceID.ID = TT.ServiceID');
    Add('AND tServiceID.ServiceTypeID = TT.ServiceTypeID');
  end;  // slQry.
  qCurrentVals.SQL := slQry;
  qCurrentVals.ExecSQL;
  qCurrentVals.Free;
  slQry.Free;
end;  // UpdateCurrentRentCalls.

end.
