unit Filter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, ToolEdit, Db, DBTables, DBCtrls;

type
  TFilterForm = class(TForm)
    edtServiceID: TEdit;
    lblServiceID: TLabel;
    lblBatchID: TLabel;
    edtBatchID: TEdit;
    lblBillingPeriod: TLabel;
    edtBillingPeriod: TEdit;
    gbDates: TGroupBox;
    cbDate: TCheckBox;
    edtFromDate: TDateEdit;
    lblFromDate: TLabel;
    edtToDate: TDateEdit;
    lblToDate: TLabel;
    gbTransaction: TGroupBox;
    gbService: TGroupBox;
    rbExpense: TRadioButton;
    rbRevenue: TRadioButton;
    btnOkay: TButton;
    btnCancel: TButton;
    rbServiceCode: TRadioButton;
    rbServiceDescription: TRadioButton;
    edtServiceDescription: TEdit;
    edtServDesc: TEdit;
    edtSupplier: TEdit;
    lblServDesc: TLabel;
    lblSupplier: TLabel;
    qServ: TQuery;
    dsServ: TDataSource;
    cbServCode: TComboBox;
    procedure cbDateClick(Sender: TObject);
    procedure rbServiceCodeClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnOkayClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbServCodeChange(Sender: TObject);
  private
    { Private declarations }
  protected
    procedure SetBatchID(const Value: string);
    procedure SetBillingPeriod(const Value: string);
  published
    property BatchID: string write SetBatchID;
    property BillingPeriod: string write SetBillingPeriod;
  public
    { Public declarations }
  end;

var
  FilterForm: TFilterForm;

implementation

uses
  DataMod, Main;

{$R *.DFM}

var
  slQuery  : TStringList;
//  slQuery1 : TStringList;
//  slQuery2 : TStringList;
  slCondition : TStringList;
  slServTypeID : TStringList;
  slServDesc : TStringList;
  slSupplier : TStringList;

procedure TFilterForm.FormCreate(Sender: TObject);
begin
  slCondition := TStringList.Create;
  slServTypeID := TStringList.Create;
  slServDesc := TStringList.Create;
  slSupplier := TStringList.Create;
{
  slQuery  := TStringList.Create;
  slQuery1 := TStringList.Create;
  slQuery2 := TStringList.Create;
  slQuery1.Add('SELECT *');                                                // 0.
  slQuery1.Add('FROM (SELECT FT1.*, tCustomer.ShipTo, tServiceID.ServiceID, '+
    'tServiceType.Code, tServiceType.Description AS "ServiceType",');      // 1.
  slQuery1.Add(' SUBSTRING(tTransactionGroup.Description,1,10) '+
    'AS "TransactionGroup", tTransactionType.Description '+
    'AS "TransactionType"');                                               // 2.
  slQuery1.Add('FROM (SELECT tFinalisedTransaction.ServiceID AS "SID", '+
    'tFinalisedTransaction.BatchID, '+
    'SUBSTRING(tBatch.BillingPeriod,1,7) AS "BillingPeriod",');            // 3.
  slQuery1.Add(' tFinalisedTransaction.TransactionTypeID,');               // 4.
  slQuery1.Add(' CAST(tFinalisedTransaction.RevenueFlag AS int) '+
    'AS "RevenueFlag", tFinalisedTransaction.CustomerID, '+
    'tFinalisedTransaction.ServiceTypeID,');                               // 5.
  slQuery1.Add(' Sum(tFinalisedTransaction.AmountExGST) AS "AmountExGST",');//6.
  slQuery1.Add(' Sum(tFinalisedTransaction.GSTAmount) AS "GSTAmount", '+
    'Sum(tFinalisedTransaction.AmountIncGST) AS "AmountIncGST"');          // 7.
  slQuery1.Add('FROM tFinalisedTransaction');                              // 8.
  slQuery1.Add('LEFT JOIN tServiceID '+
    'ON tFinalisedTransaction.ServiceID = tServiceID.ID');                 // 9.
  slQuery1.Add('LEFT JOIN tBatch '+
    'ON tFinalisedTransaction.BatchID = tBatch.ID');                       //10.

  slQuery.Text := slQuery1.Text;

  slQuery2.Add('GROUP BY tBatch.BillingPeriod, tFinalisedTransaction.BatchID, '+
    'tFinalisedTransaction.ServiceID, '+
    'tFinalisedTransaction.TransactionTypeID,');                           //11.
  slQuery2.Add(' CAST(tFinalisedTransaction.RevenueFlag AS int), '+
    'tFinalisedTransaction.CustomerID, '+
    'tFinalisedTransaction.ServiceTypeID) AS FT1');                        //12.
  slQuery2.Add('INNER JOIN tCustomer ON FT1.CustomerID = tCustomer.ID');   //13.
  slQuery2.Add('INNER JOIN tServiceID ON FT1.SID = tServiceID.ID');        //14.
  slQuery2.Add('LEFT OUTER JOIN tServiceType '+
    'ON FT1.ServiceTypeID = tServiceType.ID');                             //15.
  slQuery2.Add('LEFT JOIN (tTransactionType INNER JOIN tTransactionGroup '+
    'ON tTransactionType.TransactionGroupID = tTransactionGroup.ID)');     //16.
  slQuery2.Add(' ON FT1.TransactionTypeID = tTransactionType.ID');  //17.
  slQuery.Text := slQuery.Text + slQuery2.Text;
}
  qServ.Open;  // Linked display of service types.
  while not qServ.Eof do begin
    cbServCode.Items.Add(qServ.FieldByName('Code').AsString);
    slServTypeID.Add(qServ.FieldByName('id').AsString);
    slServDesc.Add(qServ.FieldByName('Description').AsString);
    slSupplier.Add(qServ.FieldByName('SupplierName').AsString);
    qServ.Next;
  end;
  qServ.Close;
  cbServCode.ItemIndex := 0;
  edtServDesc.Text := slServDesc[0];
  edtSupplier.Text := slSupplier[0];
end;  // FormCreate.

procedure TFilterForm.SetBillingPeriod(const Value: string);
begin
  edtBillingPeriod.Text := Value;
end;  // SetBillingPeriod.

procedure TFilterForm.SetBatchID(const Value: string);
begin
  edtBatchID.Text := Value;
end;  // SetBatchID.

procedure TFilterForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  qServ.Close;
  slCondition.Free;
  slQuery.Free;
end;  // FormClose.

procedure TFilterForm.cbDateClick(Sender: TObject);
begin
  lblFromDate.Enabled := cbDate.Checked;
  edtFromDate.Enabled := cbDate.Checked;
  lblToDate.Enabled := cbDate.Checked;
  edtToDate.Enabled := cbDate.Checked;
  if cbDate.Checked then edtFromDate.SetFocus;
end;  // cbDateClick.

procedure TFilterForm.rbServiceCodeClick(Sender: TObject);
begin
  cbServCode.Enabled := rbServiceCode.Checked;
  lblServDesc.Enabled := rbServiceCode.Checked;
  lblSupplier.Enabled := rbServiceCode.Checked;
  edtServiceDescription.Enabled := rbServiceDescription.Checked;
  if rbServiceCode.Checked then cbServCode.SetFocus
  else                          edtServiceDescription.SetFocus;
end;  // rbServiceCodeClick.

procedure TFilterForm.btnCancelClick(Sender: TObject);
begin
  Hide;
end;// btnCancelClick.

procedure TFilterForm.btnOkayClick(Sender: TObject);
//var
//  sOrder : string;
//  SubString : string;
//  WhereStarted : boolean;
//  iCount : integer;
//  slTemp : TStringList;
begin
  // Update the SQL statement.
{
  iCount := (slQuery.Count-1);
  if (Pos('ORDER BY ',UpperCase(slQuery.Strings[iCount])) > 0) then begin
    sOrder := slQuery.Strings[iCount];
  end
  else begin
    sOrder := 'ORDER BY ServiceID, BillingPeriod, BatchID';
  end;
  slQuery.Text := slQuery1.Text;
  // Establish selected filter conditions.
  WhereStarted := False;
  slCondition.Clear;
  SubString := Trim(edtServiceID.Text);
  if (Length(SubString) > 0) then begin
    slCondition.Add('WHERE tServiceID.ServiceID LIKE '''+ SubString +'%''');
    WhereStarted := True;
  end;  // edtServiceID.
  SubString := Trim(edtBatchID.Text);
  if (Length(SubString) > 0) then begin
    if WhereStarted then begin
      slCondition.Add('AND tFinalisedTransaction.BatchID = '+ SubString);
    end
    else begin
      slCondition.Add('WHERE tFinalisedTransaction.BatchID = '+ SubString);
      WhereStarted := True;
    end;
  end;  // edtBatchID.
  SubString := Trim(edtBillingPeriod.Text);
  if (Length(SubString) > 0) then begin
    if WhereStarted then begin
      slCondition.Add('AND tBatch.BillingPeriod = '''+ SubString +'''');
    end
    else begin
      slCondition.Add('WHERE tBatch.BillingPeriod = '''+ SubString +'''');
      WhereStarted := True;
    end;
  end;  // edtBillingPeriod.
  if rbServiceCode.Checked then begin
    if WhereStarted then begin
      slCondition.Add('AND tFinalisedTransaction.ServiceTypeID = '+
        slServTypeID[cbServCode.ItemIndex]);
    end
    else begin
      slCondition.Add('WHERE tFinalisedTransaction.ServiceTypeID = '+
        slServTypeID[cbServCode.ItemIndex]);
      WhereStarted := True;
    end;
  end;  // ServiceCode.
  if WhereStarted then begin
    if rbExpense.Checked then begin
      slCondition.Add('AND tFinalisedTransaction.RevenueFlag = 0');
    end
    else begin
      slCondition.Add('AND tFinalisedTransaction.RevenueFlag = 1');
    end;
  end
  else begin
    if rbExpense.Checked then begin
      slCondition.Add('WHERE tFinalisedTransaction.RevenueFlag = 0');
    end
    else begin
      slCondition.Add('WHERE tFinalisedTransaction.RevenueFlag = 1');
    end;
  end;
  // Update the final Query.
  slQuery.Clear;
  slQuery.AddStrings(slQuery1);
  if WhereStarted then slQuery.AddStrings(slCondition);
  slCondition.Clear;
  WhereStarted := False;

  SubString := Trim(edtServiceDescription.Text);
  if (Length(SubString) > 0) then begin
    if WhereStarted then begin
      slCondition.Add('AND tServiceType.Description LIKE '''+ SubString +'%''');
    end
    else begin
      slCondition.Add('WHERE tServiceType.Description LIKE '''+ SubString +'%''');
      WhereStarted := True;
    end;
  end;  // ServiceDescription.
  if WhereStarted then begin
    slCondition.Add('AND tTransactionGroup.Description '+
      'NOT LIKE ''ADMIN'') AS FT');
  end
  else begin
    slCondition.Add('WHERE tTransactionGroup.Description '+
      'NOT LIKE ''ADMIN'') AS FT');
    WhereStarted := True;
  end;
  // Add the second constantly applied SQL statements (ref: FormCreate() above).
  slQuery.AddStrings(slQuery2);
  // Add the current filter conditions.
  if WhereStarted then slQuery.AddStrings(slCondition);
  slQuery.Add(sOrder);
  MainForm.CompletedFilter := slQuery;
}
  Hide;
end;  // btnOkayClick.

procedure TFilterForm.cbServCodeChange(Sender: TObject);
begin
  edtServDesc.Text := slServDesc[cbServCode.ItemIndex];
  edtSupplier.Text := slSupplier[cbServCode.ItemIndex];
end;  // cbServCodeChange.

end.
