unit Manual;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Mask, ToolEdit, CurrEdit, RxLookup, Db, DBTables,
  DBCtrls;

type
  TManualForm = class(TForm)
    lblServiceType: TLabel;
    lblServiceID: TLabel;
    edtExGST: TCurrencyEdit;
    edtGST: TCurrencyEdit;
    edtIncGST: TCurrencyEdit;
    lblExGST: TLabel;
    lblGST: TLabel;
    lblIncGST: TLabel;
    lblFrom: TLabel;
    lblTo: TLabel;
    btnCancel: TButton;
    btnSave: TButton;
    qManual: TQuery;
    dsManual: TDataSource;
    qManualService: TQuery;
    dsManualService: TDataSource;
    cbServiceType: TComboBox;
    cbServiceID: TComboBox;
    edtRateDescription: TEdit;
    lblRateDescription: TLabel;
    qry: TQuery;
    lblVar01_Reference: TLabel;
    edtVar01_Reference: TEdit;
    edtTo: TDateEdit;
    edtFrom: TDateEdit;
    lblTransactionGroup: TLabel;
    cbTransactionGroup: TComboBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnCancelClick(Sender: TObject);
    procedure InitiateSQL(const slQuery: TStringList; var Successful: boolean);
    procedure InitiateSQLServiceID(nItem: integer);
    procedure cbServiceTypeChange(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure edtExGSTExit(Sender: TObject);
    procedure edtGSTExit(Sender: TObject);
    procedure edtIncGSTExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    nWindow: Integer;  // Array element in MainForm's (Public) array aWinForm.
    sBatchID: string;
    sManualRentID : string;
    sManualCallID : string;
    sManualOtherID : string;
  protected
    // These procedures give external access to the form variables.
    procedure SetWindowNum(Value: Integer);
    procedure SetBatchID(Value: integer);
    function GetBatchID: integer;
    procedure SetExGST(Value: string);
    function GetExGST: string;
    procedure SetGST(Value: string);
    function GetGST: string;
    procedure SetIncGST(Value: string);
    function GetIncGST: string;
    procedure SetFromDate(Value: TDateTime);
    function GetFromDate: TDateTime;
    procedure SetToDate(Value: TDateTime);
    function GetToDate: TDateTime;
  published
    // Used to access nWindow.
    property ActiveWindowNum: Integer
             read nWindow write SetWindowNum default -1;
    // Used to access window parameters for remote access.
    property BatchID: integer read GetBatchID write SetBatchID;
    property ExGSTAmount: string read GetExGST write SetExGST;
    property GSTAmount: string read GetGST write SetGST;
    property IncGSTAmount: string read GetIncGST write SetIncGST;
    property FromDate: TDateTime read GetFromDate write SetFromDate;
    property ToDate: TDateTime read GetToDate write SetToDate;
  public
    { Public declarations }
  end;

var
  ManualForm: TManualForm;

implementation

{$R *.DFM}

uses
  DataMod, DateFunctions, GenFns, Main;

var
  aServiceTypeID : array of integer;

procedure TManualForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Indicate that this form is closed.
  if (nWindow <> -1) then begin
    // It has been set by the calling program.
    aWinForm[nWindow] := False;   // Value of nWindow set by calling program.
  end;
end;    // FormClose.

procedure TManualForm.SetWindowNum(Value: Integer);
// This procedure allows the calling procedure to identify this form.
begin
  if (nWindow <> Value) then begin
    if (Value > -1) and (Value < WINNUM) then begin
      nWindow := Value;
    end
    else begin
      nWindow := -1;
    end;
  end;  // if nWindow <> Value then.
end;  // SetWindowNum.

procedure TManualForm.SetBatchID(Value: integer);
begin
  sBatchID := IntToStr(Value);
end;    // SetBatchID.

function TManualForm.GetBatchID: integer;
begin
  Result := StrToInt(sBatchID);
end;    // GetBatchID.

procedure TManualForm.SetExGST(Value: string);
begin
  edtExGST.Text := Value;
end;    // SetExGST.

function TManualForm.GetExGST: string;
begin
  Result := edtExGST.Text;
end;    // GetExGST.

procedure TManualForm.SetGST(Value: string);
begin
  edtGST.Text := Value;
end;    // SetGST.

function TManualForm.GetGST: string;
begin
  Result := edtGST.Text;
end;    // GetGST.

procedure TManualForm.SetIncGST(Value: string);
begin
  edtIncGST.Text := Value;
end;    // SetIncGST.

function TManualForm.GetIncGST: string;
begin
  Result := edtIncGST.Text;
end;    // GetIncGST.

procedure TManualForm.SetFromDate(Value: TDateTime);
begin
  edtFrom.Date := Value;
end;    // SetFromDate.

function TManualForm.GetFromDate: TDateTime;
begin
  Result := edtFrom.Date;
end;    // GetFromDate.

procedure TManualForm.SetToDate(Value: TDateTime);
begin
  edtTo.Date := Value;
end;    // SetToDate.

function TManualForm.GetToDate: TDateTime;
begin
  Result := edtTo.Date;
end;    // GetToDate.

procedure TManualForm.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TManualForm.InitiateSQLServiceID(nItem: integer);
// Called by: cbServiceType.OnChange.
// When a ServiceType is selected, the options available to select from
// in cbServiceID are adjusted accordingly.
begin
  qManualService.Close;
  cbServiceID.Items.Clear;
  if not (qManual.Bof and qManual.Eof) and (nItem > -1) then begin
    qManualService.Close;
    qManualService.SQL.Clear;
    qManualService.SQL.Add('SELECT tServiceType.ID AS ServiceTypeID, '+
      'tServiceID.ServiceID, tServiceID.ID');
    qManualService.SQL.Add('FROM tServiceType');
    qManualService.SQL.Add(' LEFT JOIN tServiceID '+
      'ON tServiceType.ID = tServiceID.ServiceTypeID');
    qManualService.SQL.Add('WHERE tServiceID.ServiceTypeID = '+
      IntToStr(aServiceTypeID[nItem]));
    qManualService.SQL.Add('AND tServiceID.Active = 1');
    qManualService.SQL.Add('ORDER BY tServiceID.ServiceID');
    qManualService.Open;
    if not qManualService.Eof then begin
      while not qManualService.Eof do begin
        cbServiceID.Items.Add(qManualService['ServiceID']);
        qManualService.Next;
      end;
      cbServiceID.ItemIndex := 0;
    end;
  end;
end;  // InitiateSQLServiceID.

procedure TManualForm.InitiateSQL(const slQuery: TStringList; var Successful: boolean);
// Called from Main when establishing this Form.
// The SQL statement provided defines the range of valid ServiceTypes for
// the selected batch type.
var
  nArray : integer;
begin
  if (slQuery.Count > 0) then begin
    qManual.Close;
    qManual.SQL := slQuery;
    qManual.Open;
    if qManual.Eof or VarIsNull(qManual['ServiceType']) then begin
      MessageDlg('Unknown System or Supplier. Process ended.', mtWarning, [mbOk], 0);
      qManualService.Close;
      Close;
      Successful := False;
    end
    else begin
      cbServiceType.Items.Clear;
      nArray := 0;
      while not qManual.Eof do begin
        cbServiceType.Items.Add(qManual['ServiceType']);
        SetLength(aServiceTypeID,nArray+1);
        aServiceTypeID[nArray] := qManual.FieldByName('ServiceTypeID').AsInteger;
        Inc(nArray);
        qManual.Next;
      end;
      cbServiceType.ItemIndex := 0;
      InitiateSQLServiceID(cbServiceType.ItemIndex);
      Successful := True;
    end;
  end;
end;  // InitiateSQL.

procedure TManualForm.edtExGSTExit(Sender: TObject);
begin
  if (edtExGST.Value <> 0) and (edtGST.Value <> 0) and
    (edtIncGST.Value = 0) then
    edtIncGST.Value := edtExGST.Value + edtGST.Value;
end;

procedure TManualForm.edtGSTExit(Sender: TObject);
begin
  if (edtExGST.Value <> 0) and (edtGST.Value <> 0)
    and (edtIncGST.Value = 0) then
    edtIncGST.Value := edtExGST.Value + edtGST.Value
  else if (edtExGST.Value = 0) and (edtIncGST.Value <> 0)
    and (edtIncGST.Value <> 0) then
    edtExGST.Value := edtIncGST.Value - edtGST.Value;
end;

procedure TManualForm.edtIncGSTExit(Sender: TObject);
begin
  if (edtExGST.Value = 0) and (edtIncGST.Value <> 0) 
    and (edtIncGST.Value <> 0) then
    edtExGST.Value := edtIncGST.Value - edtGST.Value;
end;

procedure TManualForm.cbServiceTypeChange(Sender: TObject);
begin
  InitiateSQLServiceID(cbServiceType.ItemIndex);
end;

procedure TManualForm.btnSaveClick(Sender: TObject);
var
  slInsertManual : TStringList;
  sManualID : string;
  sRateDesc : string;
  lAddRateDesc : boolean;
  sReference : string;
  lAddReference : boolean;
  sTemp : string;
  sGSTflag : string;
begin
  if (Length(cbServiceID.Text)=0) or (Length(cbServiceType.Text)=0) then begin
    MessageDlg('ServiceType and ServiceID MUST be entered.', mtWarning,
      [mbOk], 0);
  end                     
  else begin
    sReference := UpperCase(cbTransactionGroup.Text);
    if (sReference = 'RENT') then       sManualID := sManualRentID
    else if (sReference = 'CALL') then  sManualID := sManualCallID
    else if (sReference = 'OTHER') then sManualID := sManualOtherID
    else                                sManualID := sManualOtherID;
    // Include values.
    sTemp := 'ServiceID,GSTFlag,AmountExGST,GSTAmount,AmountIncGST,FromDate,ToDate';
    sRateDesc := CleanString(Trim(edtRateDescription.Text));
    lAddRateDesc := (Length(sRateDesc) > 0);
    if lAddRateDesc then sTemp := sTemp + ',RateDescription';
    sReference := CleanString(Trim(edtVar01_Reference.Text));
    lAddReference := (Length(sReference) > 0);
    if lAddReference then sTemp := sTemp + ',Var01';
    slInsertManual := TStringList.Create;
    slInsertManual.Add('INSERT INTO tDataCollection');
    slInsertManual.Add('(BatchID,TransactionTypeID,'+ sTemp +')');
    // Include values.
    if (edtGST.Value = 0) then sGSTflag := 'N' else sGSTflag := 'Y';
    sTemp := ''''+ cbServiceID.Text +''','''+ sGSTflag +''','+
      CurrToStr(edtExGST.Value) +','+ CurrToStr(edtGST.Value) +','+
      CurrToStr(edtIncGST.Value) +',';
    if (Copy(Trim(edtFrom.Text),1,1) = '/') then
      sTemp := sTemp + 'NULL,'
    else
      sTemp := sTemp + ''''+ FormattedDateString(edtFrom.Date) +''',';
    if (Copy(Trim(edtTo.Text),1,1) = '/') then
      sTemp := sTemp + 'NULL'
    else
      sTemp := sTemp + ''''+ FormattedDateString(edtTo.Date) +'''';
    if lAddRateDesc then sTemp := sTemp + ','''+ sRateDesc +'''';
    if lAddReference then sTemp := sTemp + ','''+ sReference +'''';
    slInsertManual.Add('VALUES ('+sBatchID +','+ sManualID +','+sTemp+')');
    with qry do begin
      SQL := slInsertManual;
      ExecSQL;
      Close;
    end;  // qry.
    slInsertManual.Free;
    ModalResult := mrOk;
  end;
end;

procedure TManualForm.FormCreate(Sender: TObject);
begin
  cbTransactionGroup.Items.Add('Call');
  cbTransactionGroup.Items.Add('Other');
  cbTransactionGroup.Items.Add('Rent');
  cbTransactionGroup.Text := 'Other';              // Default.
  with qry do begin
    SQL.Clear;
    SQL.Add('SELECT tTransactionType.id, tTransactionGroup.Description AS "GroupName"');
    SQL.Add('FROM tTransactionType');
    SQL.Add('LEFT JOIN tTransactionGroup '+
      'ON tTransactionType.TransactionGroupID = tTransactionGroup.ID');
    SQL.Add('WHERE tTransactionType.Description LIKE ''Manual%''');
    SQL.Add('AND tTransactionType.Active = 1');
    Open;
    sManualRentID := 'NULL';
    sManualCallID := 'NULL';
    sManualOtherID := 'NULL';
    if not (Bof and Eof) then begin
      while not Eof do begin
        if (FieldValues['GroupName'] = 'RENT') then
          sManualRentID := IntToStr(FieldValues['id'])
        else if (FieldValues['GroupName'] = 'CALL') then
          sManualCallID := IntToStr(FieldValues['id'])
        else if (FieldValues['GroupName'] = 'OTHER') then
          sManualOtherID := IntToStr(FieldValues['id']);
        Next;
      end;
    end;
    Close;
    SQL.Clear;
  end;  // qry.
end;

end.
