unit ReleaseTransactions;

interface

uses SysUtils, dbtables, bde, classes;

  procedure ReleaseTransaction(
    const nThisBatch: LongInt;
    const lRevenueFlag: boolean);
  procedure SaveProcessedTransaction(
    const nThisBatch: LongInt;
    const lRevenueFlag: boolean;
    var lSuccessful: Boolean);


implementation

uses
  CodeFile, DataMod, DateFunctions, GenFns, Main, MyMessage, Progress;

var
  sBatch : string;
  qryTT : TQuery;                   // tTransaction.
  qryStringList : TStringList;


procedure SaveProcessedTransaction(
  const nThisBatch: LongInt;
  const lRevenueFlag: boolean;
  var lSuccessful: Boolean);
var                                 // processed in a single SELECT statement.
  sRevenueFlag : string;
  MessageForm : TMessagesForm;
begin
  sBatch := Trim(IntToStr(nThisBatch));
  if lRevenueFlag then sRevenueFlag := '1' else sRevenueFlag := '0';
  qryTT := TQuery.Create(MainForm);
  qryTT.DatabaseName := 'dbPPdata';
  qryStringList := TStringList.Create;

  qryStringList.Add('INSERT INTO tProcessedTransaction');
  qryStringList.Add('(BatchID, ServiceID, CentreID, ActivityID, ElementID, '+
    'SubledgerID, LocationID, PersonID, TxnDate, OriginID,');
  qryStringList.Add(' DestinationID, Duration, CallCode, RateDescriptionID, '+
    'Peak, DialledNumberID, GSTFlag, AmountExGST, GSTAmount, AmountIncGST,');
  qryStringList.Add(' FromDate, ToDate, CustomerID, CustomerCentreID, '+
    'CustomerActivityID, CustomerElementID, CustomerSubledgerID, RevenueFlag,');
  qryStringList.Add(' TransactionTypeID, TransactionRef, StatusID, ErrorID, '+
    'DataRefID, Var01, Var02, Var03, Var04, Var05, Var06, Var07,');
  qryStringList.Add(' Var08, Var09, Var10, Var11, Var12, Var13, Var14, Var15, '+
    'ServiceTypeID, DepartmentID, GroupID, PayType, SupplierServiceCodeID)');
  qryStringList.Add('SELECT tTransaction.BatchID, tTransaction.ServiceID, '+
    'tTransaction.CentreID, tTransaction.ActivityID, tTransaction.ElementID,');
  qryStringList.Add(' tTransaction.SubledgerID, tTransaction.LocationID, '+
    'tTransaction.PersonID, tTransaction.TxnDate, tTransaction.OriginID,');
  qryStringList.Add(' tTransaction.DestinationID, tTransaction.Duration, '+
    'tTransaction.CallCode, tTransaction.RateDescriptionID, tTransaction.Peak,');
  qryStringList.Add(' tTransaction.DialledNumberID, tTransaction.GSTFlag, '+
    'tTransaction.AmountExGST, tTransaction.GSTAmount, tTransaction.AmountIncGST,');
  qryStringList.Add(' tTransaction.FromDate, tTransaction.ToDate, '+
    'tTransaction.CustomerID, tTransaction.CustomerCentreID, '+
    'tTransaction.CustomerActivityID,');
  qryStringList.Add(' tTransaction.CustomerElementID, '+
    'tTransaction.CustomerSubledgerID, tTransaction.RevenueFlag, '+
    'tTransaction.TransactionTypeID,');
  qryStringList.Add(' tTransaction.TransactionRef, tTransaction.StatusID, '+
    'tTransaction.ErrorID, tTransaction.DataRefID, tTransaction.Var01,');
  qryStringList.Add(' tTransaction.Var02, tTransaction.Var03, '+
    'tTransaction.Var04, tTransaction.Var05, tTransaction.Var06, '+
    'tTransaction.Var07,');
  qryStringList.Add(' tTransaction.Var08, tTransaction.Var09, '+
    'tTransaction.Var10, tTransaction.Var11, tTransaction.Var12, '+
    'tTransaction.Var13,');
  qryStringList.Add(' tTransaction.Var14, tTransaction.Var15, '+
//    'tServiceID.ServiceTypeID, tCentre.DepartmentID, tDepartment.GroupID, '+
    'tTransaction.ServiceTypeID, tCentre.DepartmentID, tDepartment.GroupID, '+
    'tServiceID.PayType, tTransaction.SupplierServiceCodeID');
  qryStringList.Add('FROM ((tTransaction');
  qryStringList.Add('LEFT JOIN tServiceID ON tTransaction.ServiceID = tServiceID.ID)');
//  qryStringList.Add('LEFT JOIN tCentre ON tTransaction.CentreID = tCentre.ID)');
  qryStringList.Add('LEFT JOIN tCentre ON tTransaction.CustomerCentreID = tCentre.ID)');
  qryStringList.Add('LEFT JOIN tDepartment ON tCentre.DepartmentID = tDepartment.ID');
  qryStringList.Add('WHERE tTransaction.BatchID = '+sBatch);
  qryStringList.Add('AND tTransaction.RevenueFlag = '+sRevenueFlag);
//  qryStringList.Add('ORDER BY tTransaction.ServiceID');

  MessageForm := TMessagesForm.Create(MainForm);
  if lRevenueFlag then
    MessageForm.Msg.Caption := 'Releasing Revenue transactions ...'
  else
    MessageForm.Msg.Caption := 'Releasing Expenditure transactions ...';
  MessageForm.Show;
  MessageForm.Refresh;

  lSuccessful := True;
  try
    with qryTT do begin
      Close;
      SQL := qryStringList;
      ExecSQL;
    end;  // qryTT.
  except
    lSuccessful := False;
  end;
  MessageForm.Hide;
  MessageForm.Close;
  MessageForm.Free;
  qryTT.Free;
  qryStringList.Free;
end;  // procedure SaveProcessedTransaction().


procedure ReleaseTransaction(
  const nThisBatch: LongInt;
  const lRevenueFlag: boolean);
var
  qry : TQuery;
  nBatchID : LongInt;
  lRevenue : boolean;
  sRevenue : string;
  lTransferred : boolean;
begin
  nBatchID := nThisBatch;
  lRevenue := lRevenueFlag;
  if lRevenue then sRevenue := '1' else sRevenue := '0';
  lTransferred := false;

//
// MainForm.StartTime;
//
  SaveProcessedTransaction(nBatchID, lRevenue, lTransferred);
//
// MainForm.EndTime;
//
  if lTransferred then begin
    if lRevenue then begin
      CodeToFile('tTransaction', nBatchid);
    end;  // lRevenue.
    // Remove transactions from the tTransaction table.
//
// MainForm.StartTime;
//
    DeleteTransactions('tTransaction', nBatchID, 'RevenueFlag', 'B',
      '=', sRevenue, lTransferred);
    if lRevenue then begin
      UpdateBatchStatus(nBatchID, MainForm.BATCH_STATUS_RELEASED);
      // Last activity: Save BatchEndDate for the processing of this BatchID.
      qry := TQuery.Create(MainForm);
      with qry do begin
        DatabaseName := 'dbPPdata';
        SQL.Clear;
        SQL.Add('UPDATE tBatch');
        SQL.Add('SET BatchEndDate = '''+StandardDateTimeString(Date)+'''');
        SQL.Add('WHERE ID ='+IntToStr(nBatchID));
        ExecSQL;
        Close;
        SQL.Clear;
      end;  // qry.
    end
    else begin
      UpdateBatchStatus(nBatchID, MainForm.BATCH_STATUS_PAYEXP);
    end;  // if lRevenue then begin..else.
//
// MainForm.EndTime;
// 
  end
  else begin
    // Undo any entries transferred to the tProcessedTransaction table.
    DeleteTransactions('tProcessedTransaction', nBatchID, 'RevenueFlag', 'B',
      '=', sRevenue, lTransferred);
  end;  // if lTransferred then begin.
end;  // procedure ReleaseTransaction().

end.
