unit Security;

interface
                                
uses
  Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  db, dbTables;

type
  TSecurityType = record
    ServiceID  : string[17];
    TxnDate    : string[8];
    TxnTime    : string[5];
    FromDate   : string[8];
    ToDate     : string[8];
  end;

  procedure SecurityInvoice(nBatch: LongInt;const dStartDate, dEndDate: TDateTime;
    const BatchDescription : string);
  procedure GetSecurityCharges(
    const nBatchNumber : LongInt;
    const dStartDate, dEndDate : TDateTime;
    const BatchDescription : string;
    var lReturnCode:Boolean);


implementation

uses
  Math,
  Main, DataMod, GenFns, DateFunctions, FileFunctions,
  CriticalTests, MyMessage, Progress;

var
  // Declare static variables as frequently reset and reused.
  nCount: Integer;                    // Current status for frmProgress.
  nFactor: Real;                      // Weighting for nCount in progress.
  frmProgress: TProgressForm;         // frmProgress report to the operator.

procedure GetSecurityCharges(
           const nBatchNumber : LongInt;
           const dStartDate, dEndDate : TDateTime;
           const BatchDescription : string;
           var lReturnCode: Boolean );
var
  sFromDate : string;
  sToDate : string;
  sBatch : string;
  sProcessDate : string;
  sProcessTime : string;

  sListValues : String;
  sListFields : String;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  qryDC : TQuery;
  qrySecurity : TQuery;
  nListValues : integer;
  sPolledID : string;
  sDialUpID : string;
  sSystemID : string;
  nTotalCount : integer;
  nBlock : integer;
  Factor : Real;
  sGSTRate : string;
  MessageForm : TMessagesForm;
begin
  sBatch := IntToStr(nBatchNumber);
  sFromDate := FormattedDateString(dStartDate);
  sToDate := FormattedDateString(dEndDate);
  Str(MAINFORM.GST_RATE:6:4, sGSTRate);

  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  qryStringList.Capacity := 200;

  // Establish the query to be used to save the data read.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';
  qrySecurity := TQuery.Create(MainForm);
  qrySecurity.DatabaseName := 'dbPPdata';

  lReturnCode := True;
  nFactor := 100.0;
  nCount := 0;

  frmProgress := TProgressForm.Create(MainForm);
  frmProgress.btnCancel.Visible := False;
  frmProgress.Show;

  frmProgress.Caption := 'Security Service Batch: ' + sBatch;
  frmProgress.Msg.Caption := 'Reading ...';
  frmProgress.ProgressBar1.Position := 0;
  frmProgress.Msg.Refresh;
  Factor := 0.1000;        // Count to 100 every 1000 records.
  nTotalCount := 1;
  nBlock := 0;

  sPolledID := '0';                                 // Define transaction types.
  sDialUpID := '0';
  qryDC.SQL.Add('SELECT ID');
  qryDC.SQL.Add('FROM tTransactionType');
  qryDC.SQL.Add('WHERE Description LIKE ''Security Polled''');
  qryDC.Open;
  if not qryDC.Eof then
    sPolledID := IntToStr(qryDC['id']);
  qryDC.Close;
  qryDC.SQL.strings[qryDC.SQL.Count-1] :=
    'WHERE Description LIKE ''Security Dial-Up''';
  qryDC.Open;
  if not qryDC.Eof then
    sDialUpID := IntToStr(qryDC['id']);
  qryDC.Close;
  qryDC.SQL.Clear;
  sSystemID := '0';                                 // Define System.
  qryDC.SQL.Add('SELECT ID');
  qryDC.SQL.Add('FROM tSystem');
  qryDC.SQL.Add('WHERE System LIKE ''Security''');
  qryDC.Open;
  if not qryDC.Eof then
    sSystemID := IntToStr(qryDC['id']);
  qryDC.Close;
  qryDC.SQL.Clear;

  qryStringList.Add('SELECT tServiceID.ServiceID, tServiceID.Description, '+
    'tServiceID.ServiceTypeID, tServiceType.Code,');
  qryStringList.Add(' tServiceType.Description AS ServiceTypeDesc, '+
    'tLocation.Description AS Location,');
  qryStringList.Add(' ''TransactionTypeID'' = CASE');
  qryStringList.Add('  WHEN tServiceType.Description LIKE ''% DIAL%'' THEN '+sDialUpID);
  qryStringList.Add('  WHEN tServiceType.Description LIKE ''% POLLED%'' THEN '+sPolledID);
//  qryStringList.Add('  WHEN tServiceType.Code = ''SDU'' THEN '+sDialUpID);
//  qryStringList.Add('  WHEN tServiceType.Code = ''SD1'' THEN '+sDialUpID);//VRT.
//  qryStringList.Add('  WHEN tServiceType.Code = ''SPO'' THEN '+sPolledID);
//  qryStringList.Add('  WHEN tServiceType.Code = ''SP1'' THEN '+sPolledID);//VRT.
  qryStringList.Add('  ELSE 0');
  qryStringList.Add(' END, ''GSTFlag'' = CASE');
  qryStringList.Add('  WHEN tServiceType.Description LIKE ''%VICTRACK%'' THEN ''N''');
//  qryStringList.Add('  WHEN tServiceType.Code = ''SD1'' THEN ''N''');    // VRT.
//  qryStringList.Add('  WHEN tServiceType.Code = ''SP1'' THEN ''N''');    // VRT.
  qryStringList.Add('  ELSE ''Y''');
  qryStringList.Add(' END');
  qryStringList.Add('FROM (tServiceID');
  qryStringList.Add('LEFT JOIN tServiceType '+
    'ON tServiceID.ServiceTypeID = tServiceType.ID)');
  qryStringList.Add('LEFT JOIN tLocation ON tServiceID.LocationID = tLocation.ID');
  qryStringList.Add('WHERE tServiceType.SystemID = '+sSystemID);    // Security.
  qryStringList.Add('AND tServiceID.TerminationDate IS NULL');
{// Full new SQL statement - RJC 050925.
SELECT tServiceID.ServiceID, tServiceID.Description, tServiceID.ServiceTypeID, tServiceType.Code,
 tServiceType.Description AS ServiceTypeDesc, tLocation.Description AS Location,
 'TransactionTypeID' = CASE
  WHEN tServiceType.Description LIKE '% DIAL%' THEN 54
  WHEN tServiceType.Description LIKE '% POLLED%' THEN 53
  ELSE 0
 END, 'GSTFlag' = CASE
  WHEN tServiceType.Description LIKE '%VICTRACK%' THEN 'N'
  ELSE 'Y'
 END
FROM (tServiceID
LEFT JOIN tServiceType ON tServiceID.ServiceTypeID = tServiceType.ID)
LEFT JOIN tLocation ON tServiceID.LocationID = tLocation.ID
WHERE tServiceType.SystemID = 62
AND tServiceID.TerminationDate IS NULL
}
  qrySecurity.SQL := qryStringList;
  qrySecurity.Open;
  qryStringList.Clear;

  //****************************************************************
  //****************************************************************
  //********* START READING BILLS INTO 'tDataCollection' ***********
  //****************************************************************
  if qrySecurity.Eof then begin
    MessageDlg('No entries found.', mtWarning, [mbOk], 0);
    lReturnCode := False;
  end
  else begin
    sProcessDate := ''''+ FormattedDateString(Now) +''',';
    sProcessTime := ''''+ TimeToStr(Now) +''',';

    // NOTE: Charge amounts added below AFTER ALL entries added to tDataCollection.
    while not qrySecurity.Eof do begin
      frmProgress.ProcessWinMessages(MainForm);

      FieldStringList.Clear;
      ValueStringList.Clear;
      sListFields := '(';
      sListValues := 'VALUES (';
      sListFields := sListFields + 'SequenceNo,';
      sListValues := sListValues + ''''+
        Right('0000'+IntToStr(nTotalCount),5)+''',';
      if not VarIsNull(qrySecurity['ServiceID']) then begin
        sListFields := sListFields + 'ServiceID,';
        sListValues := sListValues +''''+
          qrySecurity.FieldByName('ServiceID').AsString +''',';
      end;
      if not VarIsNull(qrySecurity['Code']) then begin
        sListFields := sListFields + 'ServiceType,';
        sListValues := sListValues +''''+
          qrySecurity.FieldByName('Code').AsString+''',';
      end;
      sListFields := sListFields + 'TxnDate,';
      sListValues := sListValues + sProcessDate;
      sListFields := sListFields + 'TxnTime,';
      sListValues := sListValues + sProcessTime;
      if not VarIsNull(qrySecurity['TransactionTypeID']) then begin
        sListFields := sListFields + 'TransactionTypeID,';
        sListValues := sListValues +
          qrySecurity.FieldByName('TransactionTypeID').AsString+',';
      end;
      FieldStringList.Add(sListFields);
      ValueStringList.Add(sListValues);
      sListFields := '';
      sListValues := '';
      sListFields := sListFields + 'GSTFlag,';
      sListValues := sListValues + ''''+
        qrySecurity.FieldByName('GSTFlag').AsString +''',';
      sListFields := sListFields + 'FromDate,';
      sListValues := sListValues +''''+ sFromDate +''',';
      sListFields := sListFields + 'ToDate,';
      sListValues := sListValues +''''+ sToDate +''',';
      if not VarIsNull(qrySecurity['ServiceTypeDesc']) then begin
        sListFields := sListFields + 'Var01,';      // tServiceType.Description.
        sListValues := sListValues +''''+
          CleanString(qrySecurity.FieldByName('ServiceTypeDesc').AsString) +''',';
      end;
      if not VarIsNull(qrySecurity['Description']) then begin
        sListFields := sListFields + 'Var02,';      // ServiceID.Description.
        sListValues := sListValues +''''+
          CleanString(qrySecurity.FieldByName('Description').AsString) +''',';
      end;
      if not VarIsNull(qrySecurity['Location']) then begin
        sListFields := sListFields + 'Var03,';      // tLocation.Description.
        sListValues := sListValues +''''+
          CleanString(qrySecurity.FieldByName('Location').AsString) +''',';
      end;
      sListFields := sListFields + 'BatchID)';           // BatchID.
      sListValues := sListValues + sBatch +')';

      // Now make the SQL statement to save to the database table.
      FieldStringList.Add(sListFields);
      ValueStringList.Add(sListValues);
      qryStringList.Clear;
      qryStringList.Add('INSERT INTO tDataCollection');
      for nListValues := 0 to FieldStringList.Count-1 do begin
        qryStringList.Add(FieldStringList[nListValues]);
      end;
      for nListValues := 0 to ValueStringList.Count-1 do begin
        qryStringList.Add(ValueStringList[nListValues]);
      end;
      // Update and execute the query.
      qryDC.Close;
      qryDC.SQL := qryStringList;
      qryDC.ExecSQL;

      Inc(nCount);
      Inc(nTotalCount);
      Inc(nBlock);
      if (nBlock=10) then begin        // and not AbortProcess
        frmProgress.ProgressBar1.Position := Trunc(nCount * Factor);
        nBlock := 0;
        if ((nCount * Factor)>=100.00) then begin
          // restart the progress bar.
          nCount := 0;
        end;  // if ((nCount * Factor)>100.00) then.
        frmProgress.ProgressBar1.Position := Trunc(nCount * Factor);
        frmProgress.Msg.Caption := 'Processing (transaction: '+
          IntToStr(nTotalCount)+')...';
      end;  // if (nBlock=10) then.
      frmProgress.ProcessWinMessages(MainForm);
      qrySecurity.Next;
    end;  // while not qrySecurity.Eof do begin
    if lReturnCode then begin
      frmProgress.Hide;
      MessageForm := TMessagesForm.Create(MainForm);
      MessageForm.Caption := 'Calculating Security Charges ...';
      MessageForm.Msg.Caption := 'Updating with applied rates ...';
      MessageForm.Show;
      frmProgress.ProcessWinMessages(MainForm);
      qryStringList.Clear;
      qryStringList.Add('UPDATE tDataCollection');
      qryStringList.Add('SET AmountExGST = ChargeRate.AmtExGST,');
      qryStringList.Add(' GSTAmount = ChargeRate.GSTAmt,');
      qryStringList.Add(' AmountIncGST = ChargeRate.AmtExGST + ChargeRate.GSTAmt');
      qryStringList.Add('FROM (SELECT GSTrate.ID, GSTrate.AmtExGST,');
      qryStringList.Add('(GSTrate.AmtExGST * GSTrate.GSTFactor) AS GSTAmt');
      qryStringList.Add('FROM (SELECT tDataCollection.ID, '+
        '(CASE WHEN tDataCollection.GSTFlag LIKE ''Y'' '+
        'THEN '+sGSTRate+' ELSE 0 END) AS GSTFactor, tRate.Cost AS AmtExGST');
      qryStringList.Add('FROM (tDataCollection');
      qryStringList.Add('LEFT JOIN tServiceID '+
        'ON tDataCollection.ServiceID = tServiceID.ServiceID)');
      qryStringList.Add('LEFT JOIN tRate '+
        'ON tServiceID.ServiceTypeID = tRate.ServiceTypeID');
      qryStringList.Add('WHERE tDataCollection.BatchID = '+ sBatch);
      qryStringList.Add('AND tRate.Cost IS NOT NULL) AS GSTrate) AS ChargeRate');
      qryStringList.Add('WHERE tDataCollection.ID = ChargeRate.ID');
      qryDC.Close;
      qryDC.SQL := qryStringList;
      qryDC.ExecSQL;

      // Now update the values stored in the tBatch table.
      qryStringList.Clear;
      qryStringList.Add('UPDATE tBatch');
      qryStringList.Add('SET GSTAmount = BatchRate.TotalGST,');
      qryStringList.Add(' TotalPayable = BatchRate.TotalIncGST');
      qryStringList.Add('FROM (SELECT Sum(tDataCollection.AmountIncGST) '+
        'AS TotalIncGST,');
      qryStringList.Add(' Sum(tDataCollection.GSTAmount) AS TotalGST');
      qryStringList.Add('FROM tDataCollection');
      qryStringList.Add('WHERE tDataCollection.BatchID = '+ sBatch +
        ') AS BatchRate');
      qryStringList.Add('WHERE tBatch.ID = '+ sBatch);
      qryDC.Close;
      qryDC.SQL := qryStringList;
      qryDC.ExecSQL;

      UpdateBatchStatus(nBatchNumber, MainForm.BATCH_STATUS_IMPORT);
      MessageForm.Close;
      MessageForm.Free;
    end;  // if lReturnCode then begin.
  end;  // Quetzal faults found.
  // Done with the input file; close it.
  FieldStringList.Free;
  ValueStringList.Free;
  frmProgress.Close;
  frmProgress.Free;
  qryStringList.Free;
  qryDC.Close;
  qryDC.Free;
  qrySecurity.Close;
  qrySecurity.Free;
end;  // GetSecurityCharges.


procedure SecurityInvoice(nBatch: LongInt;const dStartDate, dEndDate: TDateTime;
  const BatchDescription : string);
var
  lContinue : Boolean;
  EmptyList : TStringList;
begin
  EmptyList := TStringList.Create;         // Removes all entries for the batch.
  lContinue := True;
  DeleteFromDC( nBatch, EmptyList, lContinue);
  EmptyList.Free;
  GetSecurityCharges( nBatch, dStartDate, dEndDate, BatchDescription, lContinue);
end;  // SecurityInvoice.

end.  // Security.

