unit Telstra;
// Revision History.
{
RJC 090504: Calls - DialledNumber shifted 3 Chars left, and Origin 2 Chars left
                    a/c errors in field for SMS calls.
}


interface
                                
uses
  Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  db, dbTables;

type
  TServiceDetails = record
    Custno   : string;
    ServCode : string;
    ServiceID: string;
    RentFrom : TDateTime;
    RentTill : TDateTime;
    CallFrom : TDateTime;
    CallTill : TDateTime;
  end;

  TCallHeaderType = record
    SequenceNo      : string[7];
    ServiceID       : string[17];
    TxnDate         : string[8];
    TxnTime         : string[5];
    Origin          : string[20];
    Duration        : string[9];
    CallCode        : string[2];
    RateDescription : string[83];  // Was 32: RJC-080706.
    DialledNumber   : string[17];  // Temporary variable only.
    ServiceNotes    : string[80];
    AmountExGST     : string[13];
    GSTAmount       : string[13];
    AmountIncGST    : string[13];
    ToDate          : string[8];
  end;

  procedure TelstraInvoice( nBatch: LongInt; sDirectory: string );
  procedure GetPhoneAccountBills( const FName, sSupplierAccount,
    sSupplierInvoice:String; D:TDatabase; nBatchNumber:LongInt;
    var lReturnCode:Boolean);
  procedure GetMobileAccountBills(const FName, sSupplierAccount,
    sSupplierInvoice:String; D:TDatabase;
    nBatchNumber : LongInt;
    var lReturnCode:Boolean);
  procedure ImportFile( const FName: String; D: TDatabase; nBatch: LongInt;
                        var lReturnCode: Boolean);
  procedure BillingInput( stInFiles: Tstrings; nBatch: LongInt );


implementation

uses
  Math,
  Main, DataMod, GenFns, DateFunctions, FileFunctions,
  CriticalTests, Progress;

var
  // Declare static variables as frequently reset and reused.
  nCount: Integer;                    // Current status for frmProgress.
  nFactor: Real;                      // Weighting for nCount in progress.
  frmProgress: TProgressForm;         // frmProgress report to the operator.
  nSupplierID: integer;              // Set in BillingInput().
  sSupplierID: string;               // Set in BillingInput().
  qryTelstra : TQuery;               // Set in BillingInput().
  sRentCodeID : string;              // Set in BillingInput().
  sCallCodeID : string;              // Set in BillingInput().
  sOtherCodeID : string;             // Set in BillingInput().
  sDiscountCodeID : string;          // Set in BillingInput().
  sSupplierRentCodeID : string;      // Set in BillingInput().
  sSupplierCallCodeID : string;      // Set in BillingInput().
  sSupplierOtherCodeID : string;     // Set in BillingInput().
  sSupplierDiscountCodeID : string;  // Set in BillingInput().

  sLastCallCode : string;
  sLastRentCode : string;
  sLastCallTransactionTypeID : string;
  sLastRentTransactionTypeID : string;
  sLastOtherTransactionTypeID : string;
  sLastDiscountTransactionTypeID : string;
  sLastSupplierCallCodeID : string;
  sLastSupplierRentCodeID : string;
  sLastSupplierOtherCodeID : string;
  sLastSupplierDiscountCodeID : string;
  
procedure GetPhoneAccountBills
          (
           const FName, sSupplierAccount, sSupplierInvoice: String;
           D: TDatabase;
           nBatchNumber : LongInt;
           var lReturnCode: Boolean
           );
const
  PROGRESS_MAX_C: Integer = 100;       // Maximum progress is 100%.
var
  nFileSize: Integer;                  // File size in bytes.
  lFirstDF : Boolean;
  lFirstDE : Boolean;
  sPartFileName : string;
  sPartResultSet : string;
  EntryCode : string;
  nBackSlashAt : Integer;
  TelstraFile: TextFile;               // Used for accessing the data.
  CurrentFile: String;                 // Currently open file.
  FileIteration: Integer;              // Current file iteration being processed
  T: string;                           // Line of data read from TextFile.
  SubString: String;                   // A (temp) substring of T.
  vSubString : variant;                // - ditto, used in the Locate() function.
  curValue: Currency;                  // A (temp) currency conversion value.
  nSoFar: Integer;                     // Status - num of records processed.
  dBilling: TDateTime;                 // Billing date.
  dStartBillingPeriod: TDateTime;      // Start of the payment billing period.
  dEndBillingPeriod: TDateTime;        // End of the payment billing period.
  nBatch: LongInt;
  LastEntryIs_1 : boolean;
  SaveLastEntry_1 : boolean;           // Set to save when last DC line is LastEntry_1.
  CallHeader : TCallHeaderType;
  sLastDetailCode : string[2];         // To detect a header line AFTER details.

  DBfromDate : string;                 // Section DB - save read dates.
  DBtoDate : string;

  aResultSet : Array of TServiceDetails;
  nResultSet : Integer;
  nMaxResults : Integer;
  nStartSearchAt : Integer;
  lFound : Boolean;
  dMinDate : TDateTime;
  nCommaPosition : integer;
  sListValues : String;
  sListFields : String;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  slAddFieldNames : TStringList;         // Calls to AddToTable() only.
  slAddFieldValues : TStringList;        // Calls to AddToTable() only.
  qryDC : TQuery;
  nListValues : integer;
  nDummy : Real;
  sInputLineCode : string;

  nDataCollectionID : LongInt;
begin
  sLastDetailCode := '  ';
  nMaxResults := 0;
  nBatch := nBatchNumber;

  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  slAddFieldNames := TStringList.Create;         // Calls to AddToTable() only.
  slAddFieldValues := TStringList.Create;        // Calls to AddToTable() only.

  // Establish the query to be used to save the data read.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';

  // Make sure dates are defined.
  qryDC.SQL.Add('SELECT tBatch.BillingPeriod');
  qryDC.SQL.Add('FROM tBatch');
  qryDC.SQL.Add('WHERE tBatch.ID = '+IntToStr(nBatch));
  qryDC.Open;
  SubString := qryDC.FieldByName('BillingPeriod').AsString;
  dStartBillingPeriod := StrDMYtoDate('01/'+Copy(SubString,6,2)+'/'+
    Copy(SubString,1,4));
  dEndBillingPeriod := LastDayInMonth(dStartBillingPeriod);
  qryDC.Close;
  qryDC.SQL.Clear;

  LastEntryIs_1 := False;
  SaveLastEntry_1 := False;
  lFirstDF := True;
  lReturnCode := False;
  CurrentFile := Fname;
  FileIteration := 0;               // Original file ext'n, then 001,002, etc.
  // Set the earliest a valid date is likely to be.
  dMinDate := EncodeDate(1980,1,1);
  // Set a dummy date for initialisation.
  dBilling := dMinDate;
  if FileExists(CurrentFile) then begin
    // Read the file size to determine the approximate number of records.
    nFileSize := GetFileSize(CurrentFile);
    // Now we can calculate the factor for the progress bar to be displayed.
    // From prev. data: 30,930 lines in a file of 12,135,424 bytes.
    // Estimate the number of data lines in the file.
    if (nFileSize > 0) then begin
      nFactor := 100.0 / ((30930.0 / 12135424.0) * nFileSize);
    end
    else begin
      nFactor := 100.0;
    end;
    nCount := 0;
    sPartFileName := CurrentFile;
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;
    frmProgress.Caption := 'Telstra Fixed Services Batch: '+IntToStr(nBatch);
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;
    // Open the input (text) billing file for reading.
    AssignFile( TelstraFile, CurrentFile );
    Reset( TelstraFile );
    try
      Readln( TelstraFile, T );
      // Determine the billing date only once.
      if FileIteration = 0 then begin
        // First, extract the billing date (6-char date on the first line).
        SubString := Copy(T,28,8);
        dBilling := EncodeDate(StrToInt(Copy(SubString,1,4)),
          StrToInt(Copy(SubString,5,2)),
          StrToInt(Copy(SubString,7,2)));
      end;
//      D.StartTransaction;
      // Continue to search through the remaining text of the file
      // for occurrences of the search string.  On each find, process
      // the resultant string.
      //****************************************************************
      //********* START READING BILLS INTO 'tDataCollection' ***********
      //****************************************************************
      try
        // The following assumes that:
        //   DS section precedes the DE section;
        //   Sections are contiguous;
        //   Only the DS and HU (was:DE) sections are of interest here.
        nResultSet := -1;
        repeat
          frmProgress.ProcessWinMessages(MainForm);

          // Read only relevant text into the database.
          sInputLineCode := UpperCase(Copy(T,1,2));
          // BILL INFORMATION *********************************************
          if (sInputLineCode='BI') then begin
            // Do nothing - electronic copy of the printed bill, always
            // first in the input file.
          end

          // CALL DETAILS *************************************************
          else if (sInputLineCode='DC') or SaveLastEntry_1 then begin
            EntryCode := Copy(T,441,1);
            if SaveLastEntry_1 or ((EntryCode='3') or
              ((EntryCode='1') and (LastEntryIs_1 or
              (LowerCase(Trim(Copy(T,406,10)))='uses')))) then begin
              // This line is a detail call & not a header/summary - save it.
              LastEntryIs_1 := LastEntryIs_1 and ((EntryCode='1') or SaveLastEntry_1);
              FieldStringList.Clear;
              ValueStringList.Clear;
              sListFields := '(';
              sListValues := 'VALUES (';
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.SequenceNo)
              else
                SubString := Trim(Copy(T,352,7));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues + '''' + SubString +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.ServiceID)
              else
                SubString := Trim(Copy(T,45,17));
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ AccountFormat(SubString) +''',';
              // Read the code common for each detail and summary - used below
              // when the summary (type "1") is read.
              sLastDetailCode := Copy(T,67,2);                    // Entry code.
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.TxnDate)
              else
                SubString := Copy(T,197,8);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.TxnTime)
              else
                SubString := Copy(T,205,8);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnTime,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.Origin)
              else
                SubString := Trim(Copy(T,147,20));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Origin,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.Duration)
              else
                SubString := Trim(Copy(T,70,9));
              SubString := ConvertTimeToSecs(CleanString(SubString));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Duration,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.CallCode)
              else
                SubString := Trim(Copy(T,268,2));
              SubString := CleanString(SubString);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastCallTransactionTypeID :=
                      qryTelstra.FieldByName('TransactionType').AsString;
                    sLastSupplierCallCodeID :=
                      qryTelstra.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastCallTransactionTypeID := sCallCodeID;
                    sLastSupplierCallCodeID := sSupplierCallCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastCallTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryTelstra do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryTelstra.
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastCallTransactionTypeID+','+
                  sLastSupplierCallCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sCallCodeID+','+
                  sSupplierCallCodeID+',';
              end;  // CallCode.
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.RateDescription)
              else
                SubString := Trim(Copy(T,213,32));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Peak,';
                if (Pos('N Peak',SubString)>0) then
                  sListValues := sListValues +'1,'
                else
                  sListValues := sListValues +'0,';
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end
              else if LastEntryIs_1 then begin
                SubString := CleanString(Trim(CallHeader.ServiceNotes));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.DialledNumber)
              else
                SubString := Trim(Copy(T,89,17));
              if (Length(SubString) = 0) then begin
                if LastEntryIs_1 then begin
                  // Summary of calls made - assume these are Local/Untimed.
                  // Do nothing - ServicesNotes saved as RateDescription above.
                  // - RJC 070517.
//                  sListFields := sListFields + 'DialledNumber,';
//                  sListValues := sListValues +''''+
//                    CallHeader.ServiceNotes +''',';
                end
                else if (EntryCode='3') then begin
                  // Summary of calls made - assume these are Local/Untimed.
                  sListFields := sListFields + 'DialledNumber,';
                  sListValues := sListValues +''''+
                    CleanString(Trim(Copy(T,375,10))) + ' Local Calls'',';
                end
                else if (EntryCode='1') then begin
                  // Summary of calls made - assume these are Local/Untimed.
                  sListFields := sListFields + 'DialledNumber,';
                  sListValues := sListValues +''''+
                    CleanString(Trim(Copy(T,399,6))) +
                    ' uses of Network Features'',';
                end;
              end
              else begin
                sListFields := sListFields + 'DialledNumber,';
                sListValues := sListValues +''''+
                CleanString(SubString) +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';

              if LastEntryIs_1 then
                SubString := Trim(CallHeader.AmountExGST)
              else
                SubString := Trim(Copy(T,312,13));
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.GSTAmount)
              else
                SubString := Trim(Copy(T,325,13));
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTAmount,GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + SubString + ',''N'',';
              end
              else begin
                sListValues := sListValues + SubString + ',''Y'',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.AmountIncGST)
              else
                SubString := Trim(Copy(T,338,13));
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + SubString +',';
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.ToDate)
              else
                SubString := Trim(Copy(T,13,8));
//                SubString := Trim(Copy(T,14,8)); // 20050119 RJC.
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + IntToStr(nBatch) +')';

              // Now make the SQL statement to save to the database table.
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              for nListValues := 0 to FieldStringList.Count-1 do begin
                qryStringList.Add(FieldStringList[nListValues]);
              end;
              for nListValues := 0 to ValueStringList.Count-1 do begin
                qryStringList.Add(ValueStringList[nListValues]);
              end;
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;

              if LastEntryIs_1 and not SaveLastEntry_1 then begin
                // Save the values on this summary line in case the summary-only
                // entry is provided.   (Copied from following else-if section.)
                sLastDetailCode := Copy(T,67,2);
                with CallHeader do begin
                  ToDate := Copy(T,13,8);
//                  ToDate := Copy(T,14,8);  // 20060119 RJC.
                  ServiceID := Copy(T,45,17);
                  Duration := Copy(T,70,9);
                  SubString := CleanString(Copy(T,89,17));  // Temporary variable.
                  DialledNumber := SubString;
                  Origin := CleanString(Copy(T,147,20));
                  TxnDate := Copy(T,197,8);
                  TxnTime := Copy(T,205,8);
// RateDescription := CleanString(Copy(T,213,32)); RJC-080706.
                  RateDescription := Trim(CleanString(Copy(T,213,32)) +' '+
                    CleanString(Copy(T,360,50)));
                  CallCode := Copy(T,268,2);
                  AmountExGST := Copy(T,312,13);
                  GSTAmount := Copy(T,325,13);
                  AmountIncGST := Copy(T,338,13);
                  SequenceNo := Copy(T,352,7);
//                  if (Length(Trim(SubString))=0) then  // empty DialledNumber.
                    // Save extra messages.
//                    ServiceNotes  := Trim(Copy(T,388,6)) + ' Local Calls';
                    ServiceNotes  := Trim(Copy(T,353,80));
//                  else
//                    ServiceNotes  := ''; // when empty DialledNumber.
                end;  // with CallHeader do.
              end;  // if LastEntryIs_1. (Copied from section immed. following.)
            end
            else if (EntryCode='1') then begin
              // Save the values on this line in case the summary-only entry
              // is provided (usually only once each invoice).
              //
              // Test to ensure the summary line is not following the details by
              // comparing with the values stored in the previous entry/detail.
              if (sLastDetailCode = Copy(T,67,2)) then begin
                // Summary does follow detail.
                LastEntryIs_1 := False;
              end
              else begin
                // Summary does not follow detail, save it in case no details.
                LastEntryIs_1 := True;
                sLastDetailCode := Copy(T,67,2);
              end;
              with CallHeader do begin
                SequenceNo := Copy(T,352,7);
                ServiceID := Copy(T,45,17);
                TxnDate := Copy(T,197,8);
                TxnTime := Copy(T,205,8);
                Origin := CleanString(Copy(T,147,20));
                Duration := Copy(T,70,9);
                CallCode := Copy(T,268,2);
                RateDescription := CleanString(Copy(T,213,32)) +' '+
                  CleanString(Copy(T,360,50));
                SubString := CleanString(Copy(T,89,17));  // Temporary variable.
                DialledNumber := SubString;
                if (Length(Trim(SubString))=0) then begin // empty DialledNumber.
                  // Save extra messages.
                  ServiceNotes  := Trim(Copy(T,388,6)) + ' Local Calls';
                end
                else begin
                  ServiceNotes  := ''; // when empty DialledNumber.
                end;
                AmountExGST := Copy(T,312,13);
                GSTAmount := Copy(T,325,13);
                AmountIncGST := Copy(T,338,13);
                ToDate := Copy(T,13,8);
//                ToDate := Copy(T,14,8);  // 20060119 RJC.
              end;  // with CallHeader do.
            end  // else if (EntryCode='1') then.

            // Anything else goes here.
            else begin
              // Catch all other instances and reset checking variables.
              LastEntryIs_1 := False;
              sLastDetailCode := Copy(T,65,2);
            end;  // else if (EntryCode='1') then.
          end  // 'DC'.



          // LIST OF EQUIPMENT ********************************************
          else if (sInputLineCode='DE') then begin
            // This line has the full service ID and all dates-commit to memory.
            SubString := Trim(Copy(T,45,17));
            Inc( nResultSet );
            SetLength( aResultSet, nResultSet + 1);
            aResultSet[ nResultSet ].Custno := SubString;
            // Save the billing period dates.
            SubString := Trim(Copy(T,253,8));
            if (SubString = '') or (Length(SubString) <> 8) then begin
              dStartBillingPeriod := dBilling;
            end
            else begin
              dStartBillingPeriod := EncodeDate(StrToInt(Copy(SubString,1,4)),
                StrToInt(Copy(SubString,5,2)),
                StrToInt(Copy(SubString,7,2)));
            end;  // if (SubString = '') or (Length(SubString) <> 8) then.
            SubString := Trim(Copy(T,261,8));
            if (SubString = '') or (Length(SubString) <> 8) then begin
              dEndBillingPeriod := dBilling;
            end
            else begin
              dEndBillingPeriod := EncodeDate(StrToInt(Copy(SubString,1,4)),
                StrToInt(Copy(SubString,5,2)),
                StrToInt(Copy(SubString,7,2)));
            end;  // if (SubString = '') or (Length(SubString) <> 8) then.
            // Read the code:
            SubString := UpperCase(Trim(Copy(T,331,2)));
            with aResultSet[ nResultSet ] do begin
              // Rent.
              if (RentFrom<dMinDate) or (dStartBillingPeriod<RentFrom) then
                RentFrom := dStartBillingPeriod;
              if (RentTill < dMinDate) or (RentTill < dEndBillingPeriod) then
                RentTill := dEndBillingPeriod;
              // Code.
              if (Length(SubString) > 0) then
                ServCode := SubString;
            end;  // with aResultSet[ nResultSet ] do.
            lFirstDF := True;
          end  // 'DE'.


          // RENT DETAILS. ***************************************************
          else if (sInputLineCode='DF') then begin
            // This line is a detail rental - save it.
            // but the first thing is to correct the phone numbers.
            SubString := Trim(Copy(T,45,17));
            if lFirstDF then begin
              // Return to the start of the results - check the phone numbers.
              lFirstDF := False;
              nMaxResults := Length( aResultSet ) - 1;
              nResultSet := 0;
              sPartResultSet := Trim(Copy(aResultSet[nResultSet].Custno,1,10));
              if (aResultSet[ nResultSet ].Custno <> SubString) and
                (sPartResultSet <> SubString) then begin
                lFound := False;
                for nResultSet := 0 to nMaxResults do begin
                  sPartResultSet :=
                    Trim(Copy(aResultSet[nResultSet].Custno,1,10));
                  if (aResultSet[ nResultSet ].Custno = SubString) or
                    (sPartResultSet = SubString) then begin
                    lFound := True;
                    break;
                  end;
                end; // for nResultSet := 0 to nMaxResults do.
                // Check that the array bounds have not been exceeded.
                if ( nResultSet > nMaxResults ) then begin
                  // Return to the beginning of the array.
                  nResultSet := 0;
                end;  // if ( nResultSet > nMaxResults ) then.
              end
              else begin
                lFound := True;
              end;  // if (aResultSet[ nResultSet ].Custno <> SubString) and..
            end  // if lFirstDF then.
            else begin
              lFound := True;
              Inc( nResultSet );
              if (nResultSet > nMaxResults) then begin
                nResultSet := nMaxResults;
              end;  // if (nResultSet > nMaxResults) then.
              sPartResultSet := Trim(Copy(aResultSet[nResultSet].Custno,1,10));
              if (aResultSet[ nResultSet ].Custno <> SubString) and
                (sPartResultSet <> SubString) then begin
                // Must search through the array until it is found.
                lFound := False;
                if (nResultSet > nMaxResults) then begin
                  nStartSearchAt := nMaxResults;
                end
                else begin
                  nStartSearchAt := nResultSet;
                end;  // if (nResultSet > nMaxResults) then..else.
                for nResultSet := 0 to nMaxResults do begin
                  sPartResultSet :=
                    Trim(Copy(aResultSet[nResultSet].Custno,1,10));
                  if (aResultSet[ nResultSet ].Custno <> SubString) and
                    (sPartResultSet <> SubString) then begin
                    lFound := True;
                    break;
                  end;  // if aResultSet[nResultSet].Custno=SubString then.
                end;  // for nResultSet := 1 to Length( aResultSet ) do.
                if not lFound then begin
                  nResultSet := nStartSearchAt;
                end;  // if not lFound then.
              end;  // if (aResultSet[nResultSet].Custno<>SubString) then.
            end;  // if lFirstDF then.
            // Now make the SQL statement to store the data in the table.
            sListFields := '(';
            sListValues := 'VALUES (';
            SubString := Trim(Copy(T,33,9));
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'SequenceNo,';
              sListValues := sListValues +''''+ SubString +''',';
            end;  // SequenceNo.
            SubString := AccountFormat(aResultSet[ nResultSet ].custno);
            sListFields := sListFields + 'ServiceID,';
            sListValues := sListValues +''''+ SubString +''',';
            SubString := CleanString(aResultSet[ nResultSet ].ServCode);
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'CallCode,';
              sListValues := sListValues +''''+ SubString +''',';
              if (sLastRentCode <> SubString) then begin
                sLastCallCode := SubString;
                vSubString := SubString;
                with qryTelstra do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastRentTransactionTypeID := qryTelstra['TransactionType'];
                  sLastSupplierRentCodeID := qryTelstra['SupplierCodeID'];
                end
                else begin
                  sLastRentTransactionTypeID := sRentCodeID;
                  sLastSupplierRentCodeID := sSupplierRentCodeID;
                  slAddFieldNames.Clear;
                  slAddFieldValues.Clear;
                  slAddFieldNames.Add('Code');
                  slAddFieldValues.Add(''''+sLastCallCode+'''');
                  slAddFieldNames.Add('AmountCategory');
                  slAddFieldValues.Add('''RENT''');
                  slAddFieldNames.Add('Description');
                  slAddFieldValues.Add('''RENT''');
                  slAddFieldNames.Add('TransactionTypeID');
                  slAddFieldValues.Add(sLastRentTransactionTypeID);
                  slAddFieldNames.Add('SupplierID');
                  slAddFieldValues.Add(sSupplierID);
                  slAddFieldNames.Add('Active');
                  slAddFieldValues.Add('1');
                  AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                  with qryTelstra do begin
                    Close;
                    Open;
                    Locate('Code',vSubString,[loCaseInsensitive]);
                  end;  // qryOptus.
                end;  // if (not) lFound.
              end;  // if (sLastCallCode <> SubString) then begin.
              sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
              sListValues := sListValues + sLastRentTransactionTypeID+','+
                  sLastSupplierRentCodeID+',';
            end
            else begin
              sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
              sListValues := sListValues + sRentCodeID+','+
                  sSupplierRentCodeID+',';
            end;  // RentCode.
            SubString := Trim(Copy(T,98,23));
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'RateDescription,';
              sListValues := sListValues +''''+ CleanString(SubString) +''',';
            end;  // RateDescription.
            sListFields := sListFields + 'AmountExGST,';
            sListValues := sListValues + Copy(T,149,13) + ',';
            curValue := curFormat(StrToFloat(Copy(T,162,13)));
            sListFields := sListFields + 'GSTAmount,';
            sListValues := sListValues + Copy(T,162,13) +',';
            sListFields := sListFields + 'GSTFlag,';
            if (curValue = 0.00) then
              sListValues := sListValues + '''N'','
            else
              sListValues := sListValues + '''Y'',';
            sListFields := sListFields + 'AmountIncGST,';
            sListValues := sListValues + Copy(T,175,13) +',';
            if lFound then begin
              sListFields := sListFields + 'FromDate,';
              sListValues := sListValues +''''+
                FormattedDateString(aResultSet[ nResultSet ].RentFrom) +''',';
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues +''''+
                FormattedDateString(aResultSet[ nResultSet ].RentTill) +''',';
            end
            else begin
              sListFields := sListFields + 'FromDate,';
              sListValues := sListValues +''''+ YMD2DMY(Copy(T,13,8)) +''',';
// sListValues := sListValues +''''+ YMD2DMY(Copy(T,14,8)) +''',';//20060119 RJC.
            end;  // if lFound then.
            sListFields := sListFields + 'BatchID)';
            sListValues := sListValues + IntToStr(nBatch) +')';
            // Now save to the database table.
            qryStringList.Clear;
            qryStringList.Add('INSERT INTO tDataCollection');
            qryStringList.Add(sListFields);
            qryStringList.Add(sListValues);
            // Update and execute the query.
            qryDC.Close;
            qryDC.SQL := qryStringList;
            qryDC.ExecSQL;
          end  // if (sInputLineCode='DF') then.


          // OTHER DETAILS (miscellaneous).
          else if (sInputLineCode='DO') then begin
            // This line is a detailed Other (Miscellaneous) charge - save it.
            // Now make the SQL statement to store the data in the table.
            sListFields := '(';
            sListValues := 'VALUES (';
            SubString := Trim(Copy(T,33,9));
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'SequenceNo,';
              sListValues := sListValues +''''+ SubString +''',';
            end;
            sListFields := sListFields + 'ServiceID,';
            sListValues := sListValues +''''+
              AccountFormat(Trim(Copy(T,45,17))) +''',';
            SubString := Trim(Copy(T,82,70));
            if (Length(Trim(Copy(T,153,15))) > 0) then begin
              if (Length(SubString) > 0) then begin
                SubString := SubString + ' (' + Trim(Copy(T,153,15)) + ')';
              end
              else begin
                SubString := Trim(Copy(T,153,15));
              end;  // if (Length(SubString) > 0) then.
            end;  // if (Length(Trim(Copy(T,153,15))) > 0) then.
            if (Length(Trim(Copy(T,268,90))) > 0) then begin
              if (Length(SubString) > 0) then begin
                SubString := SubString + ' - ';
              end;  // if (Length(SubString) > 0) then.
              SubString := SubString + SingleSpaces(Trim(Copy(T,268,90)));
              SubString := SubString;
            end;  // if (Length(Trim(Copy(T,268,120))) > 0) then.
            if (Length(Trim(Copy(T,358,30))) > 0) then begin
              if (Length(SubString) > 0) then begin
                SubString := SubString + ' - ';
              end;  // if (Length(SubString) > 0) then.
              SubString := SubString + SingleSpaces(Trim(Copy(T,358,30)));
            end;  // if (Length(Trim(Copy(T,268,120))) > 0) then.
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'RateDescription,';
              sListValues := sListValues +''''+ CleanString(SubString) +''',';
            end;
            sListFields := sListFields + 'AmountExGST,';
            sListValues := sListValues + Copy(T,414,13) +',';
            sListFields := sListFields + 'GSTAmount,';
            sListValues := sListValues + Copy(T,427,13) +',';
            curValue := curFormat(StrToFloat(Copy(T,427,13)));
            sListFields := sListFields + 'GSTFlag,';
            if (curValue = 0.00) then begin
              sListValues := sListValues + '''N'',';
            end
            else begin
              sListValues := sListValues + '''Y'',';
            end;
            sListFields := sListFields + 'AmountIncGST,';
            sListValues := sListValues + Copy(T,440,13) +',';
            sListFields := sListFields + 'ToDate,';
            sListValues := sListValues +''''+ YMD2DMY(Copy(T,13,8)) +''',';
// sListValues := sListValues +''''+ YMD2DMY(Copy(T,14,8)) +''',';// 20060119 RJC.
            if (Length(sOtherCodeID)>0) then begin
              // No codes provided - use defsult codes.
              sListFields := sListFields + 'CallCode,';
              sListValues := sListValues + sOtherCodeID +',';
              vSubString := sOtherCodeID;
              with qryTelstra do
                lFound := Locate('Code',vSubString,[loCaseInsensitive]);
              if lFound then begin
                sLastOtherTransactionTypeID := qryTelstra['TransactionType'];
                sLastSupplierOtherCodeID := qryTelstra['SupplierCodeID'];
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastOtherTransactionTypeID+','+
                  sLastSupplierOtherCodeID+',';
              end;  // if lFound then begin.
            end;  // OtherCode.
            sListFields := sListFields + 'BatchID)';
            sListValues := sListValues + IntToStr(nBatch) +')';
            // Now save to the database table.
            qryStringList.Clear;
            qryStringList.Add('INSERT INTO tDataCollection');
            qryStringList.Add(sListFields);
            qryStringList.Add(sListValues);
            // Update and execute the query.
            qryDC.Close;
            qryDC.SQL := qryStringList;
            qryDC.ExecSQL;
          end  // if (sInputLineCode='DO') then.


          // BILLING ALL (summary).
          else if (sInputLineCode='DA') then begin
            // This line may be a detailed discount - save it.
            if (UpperCase(Trim(Copy(T,65,15)))='BILL ROUNDING') then begin
              // Now make the SQL statement to store the data in the table.
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,33,9));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues + '''ROUNDING'',';
              SubString := Trim(Copy(T,65,80));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              sListFields := sListFields + 'FromDate,';
              SubString := Trim(Copy(T,159,8));
              if (Length(SubString) > 0) then begin
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end
              else begin
                sListValues := sListValues +''''+
                  FormattedDateString(dStartBillingPeriod) +''',';
              end;
              sListFields := sListFields + 'ToDate,';
              SubString := Trim(Copy(T,167,8));
              if (Length(SubString) > 0) then begin
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end
              else begin
                sListValues := sListValues +''''+
                  FormattedDateString(dEndBillingPeriod) +''',';
              end;
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + Copy(T,220,13) +',';
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + Copy(T,233,13) +',';
              curValue := curFormat(StrToFloat(Copy(T,246,13)));
              sListFields := sListFields + 'GSTAmount,';
              sListValues := sListValues + Copy(T,246,13) +',';
              sListFields := sListFields + 'GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + '''N'',';
              end
              else begin
                sListValues := sListValues + '''Y'',';
              end;
              if (Length(sOtherCodeID)>0) then begin
                // Code not provided - use generic ones.
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues + sOtherCodeID +',';
                vSubString := sOtherCodeID;
                with qryTelstra do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastOtherTransactionTypeID := qryTelstra['TransactionType'];
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sLastOtherTransactionTypeID+',';
                end;  // if lFound then begin.
              end;  // OtherCode.
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + IntToStr(nBatch) +')';

              // Now save to the database table.
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              qryStringList.Add(sListFields);
              qryStringList.Add(sListValues);
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
            end;  // BILL ROUNDING.
          end  // if (sInputLineCode='DA') then.


          // BILLING DETAILS (account discounts).
          else if (sInputLineCode='DB') then begin
            // This line is a detailed discount - save it only if the
            // total amount is not zero.
            curValue := curFormat(StrToFloat(Copy(T,300,13)));
            if (curValue = 0.00) then begin
              SubString := Trim(Copy(T,175,8));
              if (Length(DBfromDate) = 0) and (Length(SubString)=8) then
                DBfromDate := ''''+YMD2DMY(SubString)+''',';
              SubString := Trim(Copy(T,183,8));
              if (Length(DBtoDate) = 0) and (Length(SubString)=8) then
                DBtoDate := ''''+YMD2DMY(SubString)+''',';
            end
            else begin
              // Now make the SQL statement to store the data in the table.
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,33,9));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+
                AccountFormat(Trim(Copy(T,45,17))) +''',';
              SubString := Trim(Copy(T,65,80));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+
                  CleanString(SubString) +''',';
              end;
              SubString := Trim(Copy(T,175,8));
              if (Length(SubString) = 8) then begin
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
                if (Length(DBfromDate) = 0) then
                  DBfromDate := ''''+YMD2DMY(SubString)+''',';
              end
              else if (Length(DBfromDate) > 0) then begin
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues + DBfromDate;
              end;  // FromDate.
              SubString := Trim(Copy(T,183,8));
              if (Length(SubString) = 8) then begin
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
                if (Length(DBtoDate) = 0) then
                  DBtoDate := ''''+YMD2DMY(SubString)+''',';
              end
              else if (Length(DBtoDate) > 0) then begin
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues + DBtoDate;
              end;  // ToDate.
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + Copy(T,223,11) +',';
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + Copy(T,300,13) +',';
              sListFields := sListFields + 'GSTAmount,';
              sListValues := sListValues + Copy(T,313,13) +',';
              curValue := curFormat(StrToFloat(Copy(T,313,13)));
              sListFields := sListFields + 'GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + '''N'',';
              end
              else begin
                sListValues := sListValues + '''Y'',';
              end;  // if (curValue = 0.00) then.
              if (Length(sOtherCodeID)>0) then begin
                // No code provided - use the generic ones.
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues + sDiscountCodeID +',';
                vSubString := sDiscountCodeID;
                with qryTelstra do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastDiscountTransactionTypeID := qryTelstra['TransactionType'];
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sLastDiscountTransactionTypeID+',';
                end;  // lFound.
              end;  // OtherCode.

              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + IntToStr(nBatch) +')';

              // Now save to the database table.
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              qryStringList.Add(sListFields);
              qryStringList.Add(sListValues);
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
            end;  // if (curValue <> 0.00) then..else.
          end;  // if (sInputLineCode='DB') then.


          Inc(nCount);
          // Report progress to the user.
          nDummy := nFactor * nCount;
          nSoFar := Round( nDummy );
          // Do not know how big the file is - make sure don't exceed 100%.
          if (nSoFar > PROGRESS_MAX_C) then begin
            frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
          end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;
          if SaveLastEntry_1 then begin
            // Reset parameters, then continue with the last read data.
            SaveLastEntry_1 := False;
            LastEntryIs_1   := False;
          end
          else begin
            // Read the next line of text file data.
            Readln( TelstraFile, T );
            // Test if the last entry for 'DC' must be saved.
            if LastEntryIs_1 and (Length(T) > 0) and not Eof(TelstraFile) then begin
              if (UpperCase(Copy(T,1,2)) = 'DC') then begin
                SubString := Trim(CleanString(Copy(T,213,32)) +' '+
                    CleanString(Copy(T,360,50)));
                if ((Pos('Local ',SubString)>0) and (Pos('(NDD1)',SubString)=0)) then begin
                  SubString := CallHeader.RateDescription;
                  // Reset LastEntryIs_1 if consecutive headings for Local calls.
                  LastEntryIs_1 :=
                    ((Pos('Local ',SubString)=0) or (Pos('(NDD1)',SubString)>0));
                end;
              end;
            end;  // LastEntryIs_1...
            // Test if the last entry for 'DC' must be saved.
            SaveLastEntry_1 := LastEntryIs_1 and (Length(T) > 0) and
              not Eof(TelstraFile) and (UpperCase(Copy(T,1,2)) <> 'DC');
          end;
        until (T='') and Eof( TelstraFile );
        lReturnCode := True;
      except                           // D.StartTransaction.
        lReturnCode := False;
//        D.Rollback;  // On failure, undo the changes - not useful anyway.
        raise;         // Raise the exception to avoid call to CommitUpdates.
      end;  // D.StartTransaction.
      //****************************************************************
      //*********** END READ BILLS INTO 'tDataCollection' **************
      //****************************************************************
      // Now need to process the last line if CR-LF not at end.
      //  BUT - nothing done !
//      D.Commit;
    finally                            // TelstraFile (try).
      // Done with the input file; close it.
      CloseFile( TelstraFile );
      FieldStringList.Free;
      ValueStringList.Free;
      qryStringList.Free;
      qryDC.Close;
      qryDC.Free;
    end;                               // TelstraFile (try).
    // Select the next file if exists - extension 001, 002, etc.
    Inc(FileIteration);
    CurrentFile := Copy(CurrentFile,1,Pos('.',CurrentFile));
    CurrentFile := CurrentFile + Right(('00'+IntToStr(FileIteration)),3);
  end;  // if FileExists(CurrentFile) then begin.
end;  // procedure GetPhoneAccountBills.

procedure GetMobileAccountBills
          (
          const FName, sSupplierAccount, sSupplierInvoice: String;
          D: TDatabase;
          nBatchNumber : LongInt;
          var lReturnCode: Boolean
          );
const
  PROGRESS_MAX_C: Integer = 100; // Maximum progress is 100%.
var
  nFileSize: Integer;                  // File size in bytes.
  sPartFileName : string;
  sPartResultSet : string;
  EntryCode : string;
  nBackSlashAt : Integer;
  TelstraFile: TextFile;               // Used for accessing the data.
  CurrentFile: String;                 // Currently open file.
  FileIteration: Integer;              // Current file iteration being processed
  T: string;                           // Line of data read from TextFile.
  SubString: String;                   // A (temp) substring of T.
  vSubString : variant;                // - ditto, used in the Locate() function.
  curValue: Currency;                  // A (temp) currency conversion value.
  nSoFar: Integer;                     // Status - num of records processed.
  dBilling: TDateTime;                 // Billing date.
//  dStartBillingPeriod: TDateTime;      // Start of the payment billing period.
//  dEndBillingPeriod: TDateTime;        // End of the payment billing period.
  sBatch : string;
  LastEntryIs_1 : boolean;
  CallHeader : TCallHeaderType;
  sLastDetailCode : string[2];         // To detect a header line AFTER details.

  DBfromDate : string;                 // Section DB - save read dates.
  DBtoDate : string;

  aResultSet : Array of TServiceDetails;
  nResultSet : Integer;
  nMaxResults : Integer;
  nStartSearchAt : Integer;
  lFound : Boolean;
  dMinDate : TDateTime;
  nCommaPosition : integer;
  sListValues : String;
  sListFields : String;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  slAddFieldNames : TStringList;         // Calls to AddToTable() only.
  slAddFieldValues : TStringList;        // Calls to AddToTable() only.
  qryDC : TQuery;
  nListValues : integer;
  nDummy : Real;
  sInputLineCode : string;
  lSaveToTable : boolean;
  sRateDescription : string;
  sRateDescEnhancement : string;
  sLastService : string;
  sLastServiceID : string;

  nDataCollectionID : LongInt;
//  sLastCallCode : string;
//  sLastTransactionTypeID : string;
begin
  sLastDetailCode := '  ';
  sRateDescEnhancement := '';
  nMaxResults := 0;
  sBatch := IntToStr(nBatchNumber);

  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  slAddFieldNames := TStringList.Create;         // Calls to AddToTable() only.
  slAddFieldValues := TStringList.Create;        // Calls to AddToTable() only.

  // Establish the query to be used to save the data read.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';

  lReturnCode := False;
  CurrentFile := Fname;
  FileIteration := 0;            // Original file extension, then 001,002, etc.
  dMinDate := EncodeDate(1980,1,1);
  while FileExists(CurrentFile) do begin
    // Read the file size to determine the approximate number of records.
    nFileSize := GetFileSize(CurrentFile);
    // Now we can calculate the factor for the progress bar to be displayed.
    // From prev. data: 8876 lines in a file of 1,585,557 bytes.
    // Estimate the number of data lines in the file.
    if (nFileSize > 0) then begin
      nFactor := 100.0 / 8876.0 * (1585557.0/nFileSize);
    end
    else begin
      nFactor := 100.0;
    end;
    nCount := 0;
    sPartFileName := CurrentFile;
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;
    frmProgress.Caption := 'Telstra Mobile Services Batch: '+sBatch;
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;

    // Open the input (text) billing file for reading.
    AssignFile( TelstraFile, CurrentFile );
    Reset( TelstraFile );
    try
      Readln( TelstraFile, T );
      // Calculate the billing date only once.
//      D.StartTransaction;
      // Continue to search through the remaining text of the file
      // for occurrences of the search string.  On each find, process
      // the resultant string.
      //****************************************************************
      //********* START READING BILLS **********************************
      //****************************************************************
      try
        repeat
          frmProgress.ProcessWinMessages(MainForm);
          lSaveToTable := True;
          // Read only relevant text into the database.
          // If T is shorter than indicated, Copy() returns an empty string.
          sInputLineCode := UpperCase(Copy(T, 82, 2));

          // CALL DETAILS.
          if (sInputLineCode='2D') then begin
            sInputLineCode := UpperCase(Trim(Copy(T, 82, 5)));
            if (Pos(' ',sInputLineCode) > 0) then begin
              lSaveToTable := False;
            end
            else if (Length(Trim(Copy(T, 87, 5))) = 0) then begin
              // Column heading, skip on.
              if (Copy(sInputLineCode,5,1) = 'D') then begin
                // Heading - Read the transaction type description.
                sRateDescription := CleanString(Trim(Copy(T,92,30)));
              end;  // ..and ignore any totals etc.
              lSaveToTable := False;
            end
            else if (Trim(Copy(T, 87, 5)) = 'SEQ') then begin
              // Column heading, skip on.
              lSaveToTable := False;
            end
            else begin
              // This line is a detail call & not a header/summary - save it.
              FieldStringList.Clear;
              ValueStringList.Clear;
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,87,5));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues + '''' + SubString +''',';
              end;
              SubString := Trim(Copy(T,61,17));
              if not (SubString = sLastService) then begin
                sLastService := SubString;
                sLastServiceID := AccountFormat(sLastService);
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ sLastServiceID +''',';
              // Read the code common for each detail and summary - used below
              // when the summary (type "1") is read.
              SubString := Copy(T,93,8);
              if (Length(SubString) > 0) then begin
                // Only 2-digit year, add 2000.
                SubString := Copy(SubString,1,6) +'20'+ Copy(SubString,7,2);
                sListFields := sListFields + 'TxnDate,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              SubString := Copy(T,103,5);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnTime,';
                sListValues := sListValues +''''+ SubString +':00'',';
              end;
              SubString := Trim(Copy(T,108,13)); // Prev.110 Amended RJC 090504.
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Origin,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              SubString := Trim(Copy(T,145,9));
              if (Length(SubString) > 0) then begin
                if (Pos(':',SubString) > 0) then begin
                  SubString := ConvertTimeToSecs(CleanString(SubString));
                  sListFields := sListFields + 'Duration,';
                  sListValues := sListValues +''''+ SubString +''',';
                  sRateDescEnhancement := '';
                end
                else begin
                  sRateDescEnhancement := ' '+CleanString(SubString);
                end;
              end
              else begin
                sRateDescEnhancement := '';
              end;
              SubString := CleanString(Copy(sInputLineCode,1,4));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastCallTransactionTypeID :=
                      qryTelstra.FieldByName('TransactionType').AsString;
                    sLastSupplierCallCodeID :=
                      qryTelstra.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastCallTransactionTypeID := sCallCodeID;
                    sLastSupplierCallCodeID := sSupplierCallCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastCallTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    qryTelstra.Close;
                    qryTelstra.Open;
                    with qryTelstra do
                      Locate('Code',vSubString,[loCaseInsensitive]);
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastCallTransactionTypeID+','+
                  sLastSupplierCallCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sCallCodeID+','+
                  sSupplierCallCodeID+',';
              end;  // CallCode.
              SubString := sRateDescription;
              if (Length(Trim(Copy(T,141,4))) > 0) then
                SubString := SubString +' - '+ Trim(Copy(T,141,4));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Peak,';
                if (Right(SubString,2) = ' P') then
                  sListValues := sListValues +'1,'
                else
                  sListValues := sListValues +'0,';
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ SubString+sRateDescEnhancement+''',';
              end;
              SubString := Trim(Copy(T,121,17)); // Prev.124 Amended RJC 090504.
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'DialledNumber,';
                sListValues := sListValues +''''+
                CleanString(SubString) +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';

              SubString := Trim(Copy(T,166,11));
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              SubString := Trim(Copy(T,178,11));
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTAmount,GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + SubString + ',''N'',';
              end
              else begin
                sListValues := sListValues + SubString + ',''Y'',';
              end;
              SubString := Trim(Copy(T,189,11));
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + SubString +',';
              SubString := '20'+ Trim(Copy(T,55,6));
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
            end;  // ..sInputLineCode contents.
          end  // if (sInputLineCode='2D') then.


          else if (sInputLineCode='2S') or (sInputLineCode='1A') or
            (sInputLineCode='2A') then begin
            // This line is a summary subtotal - check it for Discounts.
            sInputLineCode := UpperCase(Trim(Copy(T, 82, 5)));
            if (sInputLineCode = '2S02D') or
              (sInputLineCode = '2S02V')then begin
              // This is a discount or rounding item, or a Blackberry item.
              FieldStringList.Clear;
              ValueStringList.Clear;
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,61,17));
              if not (SubString = sLastService) then begin
                sLastService := SubString;
                sLastServiceID := AccountFormat(sLastService);
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ sLastServiceID +''',';
              SubString := CleanString(Copy(sInputLineCode,1,4));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastDiscountTransactionTypeID :=
                      qryTelstra.FieldByName('TransactionType').AsString;
                    sLastSupplierDiscountCodeID :=
                      qryTelstra.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastDiscountTransactionTypeID := sDiscountCodeID;
                    sLastSupplierDiscountCodeID := sSupplierDiscountCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''OTHER''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''OTHER''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastDiscountTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryTelstra do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryTelstra.
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastDiscountTransactionTypeID+','+
                  sLastSupplierDiscountCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sDiscountCodeID+','+
                  sSupplierDiscountCodeID+',';
              end;  // CallCode.
              sRateDescription := CleanString(Trim(Copy(T,92,30)));
              SubString := sRateDescription;
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';

              SubString := Trim(Copy(T,166,11));
              if (Length(SubString) = 0) then SubString := '0';
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              SubString := Trim(Copy(T,178,11));
              if (Length(SubString) = 0) then SubString := '0';
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTAmount,GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + SubString + ',''N'',';
              end
              else begin
                sListValues := sListValues + SubString + ',''Y'',';
              end;
              SubString := Trim(Copy(T,189,11));
              if (Length(SubString) = 0) then SubString := '0';
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + SubString +',';
              SubString := '20'+ Trim(Copy(T,55,6));
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
            end  // 2so2d or 2so2v.

            else if ((sInputLineCode = '2S10V') and
              ((Pos('DISCOUNT', UpperCase(Trim(Copy(T, 90, 35)))) > 0) or
              (Pos('REBATE', UpperCase(Trim(Copy(T, 90, 35)))) > 0) or
              (Pos('INCLUDED', UpperCase(Trim(Copy(T, 90, 35)))) > 0) or
              (Pos('ALLOWANCE', UpperCase(Trim(Copy(T, 90, 35)))) > 0))) or
              (sInputLineCode = '1A02V') or ((sInputLineCode = '2A02V') and
              (Pos('ROUND', UpperCase(Trim(Copy(T, 90, 35))))>0)) then begin
              // This is a discount item.
              FieldStringList.Clear;
              ValueStringList.Clear;
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,61,17));
              if not (SubString = sLastService) then begin
                sLastService := SubString;
                sLastServiceID := AccountFormat(sLastService);
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ sLastServiceID +''',';
              SubString := CleanString(Copy(sInputLineCode,1,4));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastDiscountTransactionTypeID :=
                      qryTelstra.FieldByName('TransactionType').AsString;
                    sLastSupplierDiscountCodeID :=
                      qryTelstra.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastDiscountTransactionTypeID := sDiscountCodeID;
                    sLastSupplierDiscountCodeID := sSupplierDiscountCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''OTHER''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''OTHER''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastDiscountTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryTelstra do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryTelstra.
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastDiscountTransactionTypeID+','+
                  sLastSupplierDiscountCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sDiscountCodeID+','+
                  sSupplierDiscountCodeID+',';
              end;  // CallCode.
              sRateDescription := CleanString(Trim(Copy(T,92,30)));
              SubString := sRateDescription;
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';

              SubString := Trim(Copy(T,166,11));
              if (Length(SubString)=0) and
                (Length(Trim(Copy(T,178,11)+Copy(T,189,11)))=0) then begin
                lSaveToTable := False;               // No discount - ignore.
              end
              else begin
                if (Length(SubString) = 0) then SubString := '0';
                sListFields := sListFields + 'AmountExGST,';
                sListValues := sListValues + SubString +',';
                SubString := Trim(Copy(T,178,11));
                curValue := curFormat(StrToFloat(SubString));
                if (Length(SubString) = 0) then SubString := '0';
                sListFields := sListFields + 'GSTAmount,GSTFlag,';
                if (curValue = 0.00) then begin
                  sListValues := sListValues + SubString + ',''N'',';
                end
                else begin
                  sListValues := sListValues + SubString + ',''Y'',';
                end;
                SubString := Trim(Copy(T,189,11));
                if (Length(SubString) = 0) then SubString := '0';
                sListFields := sListFields + 'AmountIncGST,';
                sListValues := sListValues + SubString +',';
                SubString := '20'+ Trim(Copy(T,55,6));
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;  // if (Length(SubString)=0) then begin..else.
            end  // if (sInputLineCode = '2S10V') then begin.

            else begin
              lSaveToTable := False;
            end;  // ..else begin.
          end  // ..else if (sInputLineCode='2S') then begin.

          else begin
            lSaveToTable := False;
          end;  // ..else begin.
          if lSaveToTable then begin
            // Now make the SQL statement to save to the database table.
            sListFields := sListFields + 'BatchID)';
            sListValues := sListValues + sBatch +')';
            FieldStringList.Add(sListFields);
            ValueStringList.Add(sListValues);
            qryStringList.Clear;
            qryStringList.Add('INSERT INTO tDataCollection');
            for nListValues := 0 to FieldStringList.Count-1 do begin
              qryStringList.Add(FieldStringList[nListValues]);
            end;
            for nListValues := 0 to ValueStringList.Count-1 do begin
              qryStringList.Add(ValueStringList[nListValues]);
            end;
            // Update and execute the query.
            qryDC.Close;
            qryDC.SQL := qryStringList;
            qryDC.ExecSQL;
          end;  // if lSaveToTable then begin.
          Inc(nCount);
          // Report progress to the user.
          nSoFar := Round(nCount * nFactor);
          // Do not know how big the file is - make sure don't exceed 100%.
          if (nSoFar > PROGRESS_MAX_C) then begin
            frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
          end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;
          // Read the next line of text file data.
          Readln( TelstraFile, T );
        until Eof( TelstraFile );
        lReturnCode := True;
      except    // D.StartTransaction.
        lReturnCode := False;
//        D.Rollback;       // On failure, undo the changes - not useful anyway.
        raise;            // Raise the exception to avoid call to CommitUpdates.
      end;    // D.StartTransaction.
      // Now need to process the last line if CR-LF not at end.
      //  BUT - nothing done !
//      D.Commit;
      //****************************************************************
      //*********** END READING BILLS **********************************
      //****************************************************************
    finally    // TelstraFile (try).
      // Done with the input file; close it.
      CloseFile( TelstraFile );
    end;    // TelstraFile (try).
    // Select the next file if exists - extension 001, 002, etc.
    Inc(FileIteration);
    CurrentFile := Copy(CurrentFile,1,Pos('.',CurrentFile));
    CurrentFile := CurrentFile + Right(('00'+IntToStr(FileIteration)),3);
  end;  // if FileExists(CurrentFile) then begin.
end;  // procedure GetMobileAccountBills.


procedure ImportFile
          (const FName: String;
           D: TDatabase;
           nBatch: LongInt;
           var lReturnCode: Boolean
           );
// This function determines the type of the input file (FName) and calls
// the appropriate procedure to read the information.
var
  TelstraFile: TextFile;               // Used for accessing the data.
  T: string;                           // Line of data read from TextFile.
  sSupplierAccount: string;            // Telstra's account number.
  sAccountType: String;                // Unique string for each type of account
  sSupplierInvoice: String;            // Telstra Bill Number.
  nBatchID : LongInt;
begin
  lReturnCode := False;
  // Open the input (text) billing file for reading.
  AssignFile( TelstraFile, Fname );
  Reset( TelstraFile );
  nCount := 0;
  // Assume that the results table has already been emptied.

  // Get the first occurence of the search string.
  Readln( TelstraFile, T );
  if Eof( TelstraFile ) then begin  // To be sure the file isn't empty too.
    CloseFile( TelstraFile );
  end
  else begin
    // Determine the Sequence Number first ********** Start TEMPORARY ********
    nBatchID := nBatch;
    // The required data is in the first line, so close the input file.
    // First, determine the account number.
    sSupplierAccount := Trim(Copy(T,10,17));
    sAccountType := UpperCase(Copy(T,1,8));
    // Next, determine if the Bill is for standard or mobile accounts
    // and transfer control.
    if (sAccountType='HDR:FCAB') then begin
      // This is a standard account.
      Readln( TelstraFile, T );
      // Search for the the first line of the "printed" Bill.
      while not Eof( TelstraFile ) and (UpperCase(Copy(T,1,2))<>'BI') do begin
        Readln( TelstraFile, T );
      end;  // while not do.
      if Eof( TelstraFile ) then begin
        CloseFile( TelstraFile );
      end
      else begin
        sSupplierInvoice := Copy(T,22,10);
        CloseFile( TelstraFile );
        GetPhoneAccountBills( Fname, sSupplierAccount, sSupplierInvoice,
          D, nBatchID, lReturnCode );
      end;
    end
    else if (sAccountType='HDR:RACE') then begin
      // This is a Mobile phone account.
      Readln( TelstraFile, T );
      // Search for the a line of the "printed" Bill - always at start of file.
      while not Eof( TelstraFile ) and (UpperCase(Copy(T,1,3))<>'RBM') do begin
        Readln( TelstraFile, T );
      end;  // while not do.
      if Eof( TelstraFile ) then begin
        CloseFile( TelstraFile );
      end
      else begin
        sSupplierInvoice := Trim(Copy(T,37,17));
        CloseFile( TelstraFile );
        GetMobileAccountBills( Fname, sSupplierAccount, sSupplierInvoice,
          D, nBatchID, lReturnCode );
      end;
    end;  // if (sAccountType='HDR:FCAB') then..else.
  end;   //  if Eof( TelstraFile ) then..else.
end;  // procedure ImportFile().

procedure BillingInput( stInFiles: Tstrings; nBatch: LongInt );
// This program processes the Telstra billing reports supplied on disk
// and places the data into the Temp.dbf database for unverified bill
// information.
//
// The parameter passed contains the names of the input files, which are
// passed on to the ImportFile() procedure.
//
var
  lContinue : Boolean;
  sPartFileName : String;
  nBackSlashAt: Integer;
begin
  // initialise the Static table elements defined as Private for this form.
  qryTelstra := TQuery.Create(MainForm);
  qryTelstra.DatabaseName := 'dbPPdata';
  qryTelstra.SQL.Add('SELECT id');
  qryTelstra.SQL.Add('FROM tSupplier');
  qryTelstra.SQL.Add('WHERE SupplierName LIKE ''TELSTRA%''');
  qryTelstra.Open;
  if qryTelstra.Eof then begin
    MessageDlg('TELSTRA not found in the tSupplier table. Processing stopped.',
      mtWarning, [mbOk], 0);
  end
  else begin
    nSupplierID := qryTelstra['id'];
    sSupplierID := IntToStr(nSupplierID);
    qryTelstra.Close;
    qryTelstra.SQL.Clear;
    qryTelstra.SQL.Add('SELECT Code, CAST(TransactionTypeID AS VARCHAR(20)) '+
      'AS TransactionType, CAST(ID AS VARCHAR(20)) AS SupplierCodeID');
    qryTelstra.SQL.Add('FROM tSupplierServiceCode');
    qryTelstra.SQL.Add('WHERE SupplierID = '+sSupplierID);
    qryTelstra.SQL.Add('AND Active = 1');
    qryTelstra.SQL.Add('AND Description LIKE ''RENT''');
    qryTelstra.Open;
    if qryTelstra.Eof then sRentCodeID := 'NULL'
    else begin
      sRentCodeID := Trim(qryTelstra['TransactionType']);
      sSupplierRentCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''CALLS''';
    qryTelstra.Open;
    if qryTelstra.Eof then sCallCodeID := 'NULL'
    else begin
      sCallCodeID := Trim(qryTelstra['TransactionType']);
      sSupplierCallCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''Unknown%''';
    qryTelstra.Open;
    if qryTelstra.Eof then sOtherCodeID := 'NULL'
    else begin
      sOtherCodeID := qryTelstra['TransactionType'];
      sSupplierOtherCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''MISCELLANEOUS''';
    qryTelstra.Open;
    if qryTelstra.Eof then sDiscountCodeID := 'NULL'
    else begin
      sDiscountCodeID := Trim(qryTelstra['TransactionType']);
      sSupplierDiscountCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Delete(qryTelstra.SQL.Count-1);
    qryTelstra.Open;

    // Initialise the progress bar to display to the operator.
    frmProgress := TProgressForm.Create(MainForm);
    frmProgress.btnCancel.Visible := False;
    frmProgress.Show;

    sPartFileName := stInFiles.Strings[0];
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;  // while Pos('\',sPartFileName)>0 do begin.
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;
    // Now move the text data into the Table.
//
// MainForm.StartTime;
//
    ImportFile( stInFiles.Strings[0], DM.dbPPdata, nBatch, lContinue);

//
// MainForm.EndTime;
// 
    frmProgress.Close;
    if lContinue then begin
      UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_IMPORT);

      // **********************************************************************
      // **********************************************************************
      // Test for critical errors that prevent progress to the next stage.
      lContinue := False;
      CriticalErrorTest( nBatch, lContinue );
      if lContinue then begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Passed all tests for Critical Errors.', mtInformation,
          [mbOk], 0);
      end
      else begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Critical errors found.'#13#10#13#10'These must be resolved ' +
          'before this invoice can be progressed.',
          mtWarning, [mbOk], 0);
      end;
    end;  // if lContinue then begin.
  end;  // if qryTelstra.Eof then begin..else.
end;  // BillingInput.


procedure TelstraInvoice( nBatch: LongInt; sDirectory: string );
// This function prompts the user for the name of the Telstra supplied
// text file and then calls the BillingInput function (which reads the
// file into a table and controls its processing).
var
  lContinue : Boolean;
  stFiles : TStringList;
  OpenDlg: TOpenDialog;
begin
  // Create the stringlist which contains the names of selected filenames.
  stFiles := TStringList.Create;
  try
    // Now, we must establish which files are to be processed.
    // Select the source directory.
    OpenDlg := TOpenDialog.Create(MainForm);
    OpenDlg.Filter := 'Telstra Billing file (*.dat)|*.DAT|All files (*.*)|*.*';
    OpenDlg.FilterIndex := 1;
    // Use the program directory as the default directory.
    if (Length(sDirectory) > 0) then OpenDlg.InitialDir := sDirectory
    else OpenDlg.InitialDir := ExtractFilePath(ParamStr(0));
    OpenDlg.Title := 'Select The Telstra Account Data File';
    OpenDlg.Options := [ofFileMustExist, ofNoChangeDir];
    // Read and save the user's choices before continuing.
    lContinue := OpenDlg.Execute;
    stFiles.Text := OpenDlg.Files.Text;
    OpenDlg.Free;
    if lContinue then begin
      lContinue := stFiles.Count > 0;
      // Now call for processing of the input files.
      if lContinue then begin
        BillingInput(stFiles, nBatch);
      end;  // if lContinue then.
    end;  // if lContinue then.
  finally
    stFiles.Free;
  end;
end;    // procedure TelstraInvoice.


end.
