unit TelstraFunctions;

interface

uses
  Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  db, dbTables;

  procedure SaveTelstraSummaryCharges(nBatchNumber : LongInt);
  procedure ApplyTelstraDataRates(const nBatch: integer;
                                        LumpedCosts, IncludedRate: boolean);

implementation

uses
  Math,
  Main,
  DataMod,
  GenFns,
  DateFunctions,
//  FileFunctions,
//  CriticalTests,
  Progress,
  MyMessage;
//  ConstantValues;



procedure SaveTelstraSummaryCharges(nBatchNumber : LongInt);
// Save those items in DS not matched with individual call details.
const
  PROGRESS_MAX_C: Integer = 100;        // Maximum progress is 100%.
var
  nCount: Integer;                    // Current status for frmProgress.
  nFactor: Real;                      // Weighting for nCount in progress.
  nBatchID: LongInt;
  sBatchID: string;
  frmProgress: TProgressForm;         // frmProgress report to the operator.
  qryDCCount  : TQuery;
  qryDCUpdate : TQuery;
  qryDSCount  : TQuery;
  qryDS       : TQuery;
  qryDC       : TQuery;
  nEntries : integer;
  nDummy      : Real;
  nSoFar      : integer;
  nDCSavedCalls : integer;
  nDSTotalCalls : integer;
  nDSCalls      : integer;
  nDCCalls      : integer;
  nStep         : integer;
  curDCTally   : currency;
  sServiceID : string;
  sCallCode  : string;
begin
  nBatchID := nBatchNumber;
  sBatchID := IntToStr(nBatchID);
  qryDCCount := TQuery.Create(MainForm);
  qryDCCount.DatabaseName := 'dbPPdata';
  with qryDCCount do begin
    SQL.Add('SELECT COUNT(*) AS NumberOfCalls');
    SQL.Add('FROM tDataCollection');
    SQL.Add('WHERE BatchID = '+sBatchID);
    SQL.Add('AND ServiceID LIKE ''PHONENUMBER''');  // Dummy value - "bookmark".
    SQL.Add('AND CallCode LIKE ''CALLCODEVALUE'''); // Dummy value - "bookmark".
  end;  // qryDCCount.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';
  with qryDC do begin
    SQL.Add('SELECT ServiceID, CallCode, AmountExGST');
    SQL.Add('FROM tDataCollection');
    SQL.Add('WHERE BatchID = '+sBatchID);
    SQL.Add('AND ServiceID LIKE ''PHONENUMBER''');  // Dummy value - "bookmark".
    SQL.Add('AND CallCode LIKE ''CALLCODEVALUE'''); // Dummy value - "bookmark".
    SQL.Add('ORDER BY ID');
  end;  // qryDC.
  qryDSCount := TQuery.Create(MainForm);
  qryDSCount.DatabaseName := 'dbPPdata';
  // Count the number of DS entries to be processed.
  with qryDSCount do begin
    SQL.Add('SELECT COUNT(*) AS NumberDSCalls');
    SQL.Add('FROM tDataCollect_Temp');
    SQL.Add('WHERE BatchID = '+sBatchID);
    Open;
  end;  // qryDSCount.
  frmProgress := TProgressForm.Create(MainForm);
  frmProgress.btnCancel.Visible := False;
  frmProgress.Show;
  nEntries := qryDSCount.FieldByName('NumberDSCalls').AsInteger;
  if (nEntries > 0) then begin
    nFactor := 100.0 / nEntries;
  end
  else begin
    nFactor := 100.0;
  end;
  nCount := 0;
  nSoFar := 0;
  frmProgress.Msg.Caption := 'Processing DS Summary list ...';
  frmProgress.Msg.Refresh;
  frmProgress.ProgressBar1.Position := nSoFar;
  frmProgress.ProcessWinMessages(MainForm);
  // Now reset the qryDSCount SQL statement for processing below.
  with qryDSCount do begin
    Close;
    SQL.Clear;
    SQL.Add('SELECT SUM(NumCalls) AS NumberDSCalls');
    SQL.Add('FROM tDataCollect_Temp');
    SQL.Add('WHERE BatchID = '+sBatchID);
    SQL.Add('AND ServiceID LIKE ''PHONENUMBER''');  // Dummy value = "bookmark".
    SQL.Add('AND CallCode LIKE ''CALLCODEVALUE'''); // Dummy value = "bookmark".
  end;  // qryDSCount.
  //
  qryDCUpdate := TQuery.Create(MainForm);
  qryDCUpdate.DatabaseName := 'dbPPdata';
  with qryDCUpdate do begin
    SQL.Add('UPDATE tDataCollect_Temp');          // 0.
    SQL.Add('SET Active = 1');                    // 1.
    SQL.Add('WHERE ID = 12345');                  // 2.
  end;  // qryDCUpdate.
  // Now open the Summaries and we are ready to see which ones to save.
  qryDS := TQuery.Create(MainForm);
  qryDS.DatabaseName := 'dbPPdata';
  with qryDS do begin
    Close;
    SQL.Clear;
    SQL.Add('SELECT *');
    SQL.Add('FROM tDataCollect_Temp');
    SQL.Add('WHERE BatchID = '+sBatchID);
    SQL.Add('ORDER BY ID');
    Open;
  end;  // qryDS.
  // And here we go.
  while not qryDS.Eof do begin
    // Get the number of details read into the tDataCollection table.
    sServiceID := qryDS.FieldByName('ServiceID').AsString;
    sCallCode := qryDS.FieldByName('CallCode').AsString;

// ****************************************************************************
// *****************  DEBUG  **************************************************
// ****************************************************************************
//    if sServiceID = '03-53681240' then begin
//    frmProgress.ProcessWinMessages(MainForm);
//    end;
// ****************************************************************************
// *****************  DEBUG  **************************************************
// ****************************************************************************

    qryDCCount.Close;
    qryDCCount.SQL.Strings[3] := 'AND ServiceID LIKE '''+sServiceID+'''';
    qryDCCount.SQL.Strings[4] := 'AND CallCode LIKE '''+sCallCode+'''';
    qryDCCount.Open;
    if qryDCCount.Eof then
      nDCSavedCalls := 0
    else
      nDCSavedCalls := qryDCCount.FieldByName('NumberOfCalls').AsInteger;
    // Get the number of calls in the Summary list of tDataCollext_Temp table.
    qryDSCount.Close;
    qryDSCount.SQL.Strings[3] := 'AND ServiceID LIKE '''+sServiceID+'''';
    qryDSCount.SQL.Strings[4] := 'AND CallCode LIKE '''+sCallCode+'''';
    qryDSCount.Open;
    if qryDSCount.Eof then
      nDSTotalCalls := 0
    else
      nDSTotalCalls := qryDSCount.FieldByName('NumberDSCalls').AsInteger;
    // Get the number of calls in this summary item ServiceID & CallCode.
    // Test if there is a mismatch for each entry.
    if (nDCSavedCalls = nDSTotalCalls) then begin
      // Calls and summary match for this serviceID & Type. Skip over.
      while not qryDS.Eof and
        (sServiceID = qryDS.FieldByName('ServiceID').AsString) and
        (sCallCode  = qryDS.FieldByName('CallCode').AsString) do begin
        // Skip to the next entry.
        qryDS.Next;
        Inc(nCount);
      end;  // while not qryDS.eof.
      // Now we skip in qryDS next, so back up one position.
      if not qryDS.Eof and
        ((sServiceID <> qryDS.FieldByName('ServiceID').AsString) or
         (sCallCode  <> qryDS.FieldByName('CallCode').AsString)) then begin
        // Oops - we have passed out of this ServiceID and CallCode. Go back.
        qryDS.Prior;
        Dec(nCount);
      end;
    end
    else begin
      // Load up the relevant DC data.
      qryDC.Close;
      qryDC.SQL.Strings[3] := 'AND ServiceID LIKE '''+sServiceID+'''';
      qryDC.SQL.Strings[4] := 'AND CallCode LIKE '''+sCallCode+'''';
      qryDC.Open;
      // Now process the matching of DS items with details in DC.
      while not qryDS.Eof and
        (sServiceID = qryDS.FieldByName('ServiceID').AsString) and
        (sCallCode  = qryDS.FieldByName('CallCode').AsString) do begin
        // Test the current DS entry.
        nDSCalls := qryDS.FieldByName('NumCalls').AsInteger;
        if (nDSCalls > nDCSavedCalls) then begin
          // Clearly these have not been saved.
          // Mark this DS entry for inclusion in tDataCollection (Active=1).
          qryDCUpdate.SQL.Strings[2] := 'WHERE ID ='+
                                 IntToStr(qryDS.FieldByName('ID').AsInteger);
          qryDCUpdate.ExecSQL;
        end
        else if ((nDSTotalCalls - nDSCalls) = nDCSavedCalls) and
                (nDSCalls < nDCSavedCalls) then begin
          // Assume 2 DS entries, first DS has fewer calls but
          // second DS entry matches the number of calls in DC.
          // So, save the first and hope total amount for next is a match.
          //
          // Mark this DS entry for inclusion in tDataCollection (Active=1).
          qryDCUpdate.SQL.Strings[2] := 'WHERE ID ='+
                                 IntToStr(qryDS.FieldByName('ID').AsInteger);
          qryDCUpdate.ExecSQL;
        end
        else begin
          // Add up the value of the calls and check with DS value.
          nDCCalls := 0;
          curDCTally := 0.0;
          for nStep := 1 to nDSCalls do begin
            if not qryDC.Eof then begin
              curDCTally := curDCTally + qryDC.FieldByName('AmountExGST').AsCurrency;
              Dec(nDCCalls);
              qryDC.Next;
            end;  // eof.
          end;
          if ((nDCCalls + nDSCalls) = 0) then begin   // Correct number counted.
            // Completed addition. Compare values.
            if (curDCTally = qryDS.FieldByName('AmountExGST').AsCurrency) then begin
              // The details are for this DS entry. Do not save DS entry.
              // Reset the count of remaining DC calls.
              nDCSavedCalls := nDCSavedCalls + nDCCalls;
            end
            else begin
              // Mark this DS entry for inclusion in tDataCollection (Active=1).
              qryDCUpdate.SQL.Strings[2] := 'WHERE ID ='+
                                 IntToStr(qryDS.FieldByName('ID').AsInteger);
              qryDCUpdate.ExecSQL;
              // Return DC to where we started - ready for the next DS entry.
              qryDC.MoveBy(nDCCalls)
            end;
          end
          else begin
            // Not enough calls in DC.
            // Mark this DS entry for inclusion in tDataCollection (Active=1).
            qryDCUpdate.SQL.Strings[2] := 'WHERE ID ='+
                                           qryDS.FieldByName('ID').AsString;
            qryDCUpdate.ExecSQL;
            // Return DC to where we started - ready for the next DS entry.
            qryDC.MoveBy(nDCCalls)
          end;  // (nDCCalls = nDSCalls).
        end;  // (nDCSavedCalls+nDSCalls=0).
        // Move to the next DS entry for this ServiceID and CallCode.
        qryDS.Next;
        Inc(nCount);
      end;  // while not qryDS.eof (inner loop).
      if not qryDS.Eof and
        ((sServiceID <> qryDS.FieldByName('ServiceID').AsString) or
         (sCallCode  <> qryDS.FieldByName('CallCode').AsString)) then begin
        // Oops - we have passed out of this ServiceID and CallCode. Go back.
        qryDS.Prior;
        Dec(nCount);
      end;
    end;  // if numbers are a mismatch.
    // ************* Report progress to the user *************************
    // ************* Report progress to the user *************************
    Inc(nCount);
    nDummy := nFactor * nCount;
    nSoFar := Round( nDummy );
    // Make sure we don't exceed 100% on the Progress bar.
    if (nSoFar > PROGRESS_MAX_C) then begin
      frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
    end
    else begin
      frmProgress.ProgressBar1.Position := nSoFar;
    end;
    // Update the user's screen.
    frmProgress.ProcessWinMessages(MainForm);
    qryDS.Next;
  end;  // while not qryDS.eof (outer loop).
  // Done. Now add all Active entries into the tDataCollection table.
  qryDCCount.Close;
  qryDCUpdate.Close;
  qryDSCount.Close;
  qryDS.Close;
  with qryDCUpdate do begin
    Unprepare;
    SQL.Clear;
    SQL.Add('INSERT INTO tDataCollection');
    SQL.Add('(SequenceNo, ');
    SQL.Add(' ServiceID, ');
    SQL.Add(' CallCode, ');
    SQL.Add(' TransactionTypeID, ');
    SQL.Add(' RateDescription, ');
    SQL.Add(' AmountExGST, ');
    SQL.Add(' GSTAmount, ');
    SQL.Add(' AmountIncGST, ');
    SQL.Add(' GSTFlag, ');
    SQL.Add(' FromDate, ');
    SQL.Add(' ToDate, ');
    SQL.Add(' SupplierServiceCodeID, ');
    SQL.Add(' BatchID)');
    SQL.Add(' SELECT ');
    SQL.Add('   SequenceNo, ');
    SQL.Add('   ServiceID, ');
    SQL.Add('   CallCode, ');
    SQL.Add('   TransactionTypeID, ');
    SQL.Add('   RateDescription, ');
    SQL.Add('   AmountExGST, ');
    SQL.Add('   GSTAmount, ');
    SQL.Add('   AmountIncGST, ');
    SQL.Add('   GSTFlag, ');
    SQL.Add('   FromDate, ');
    SQL.Add('   ToDate, ');
    SQL.Add('   SupplierServiceCodeID, ');
    SQL.Add('   BatchID');
    SQL.Add(' FROM tDataCollect_Temp');
    SQL.Add(' WHERE Active = 1');
    ExecSQL;
  end;  // qryDCUpdate.
  qryDCCount.Free;
  // DONE - Delete the DS entries in the temporary table.
  with qryDS do begin
    Close;
    SQL.Clear;
    SQL.Add('DELETE');
    SQL.Add('FROM tDataCollect_Temp');
    SQL.Add('WHERE BatchID = '+sBatchID);
    ExecSQL;
  end;  // qryDS.
  frmProgress.Close;
  frmProgress.Free;
  qryDS.Free;
  qryDSCount.Free;
  qryDCUpdate.Free;
end;  // SaveTelstraSummaryCharges.


procedure ApplyTelstraDataRates(const nBatch: integer;
  LumpedCosts, IncludedRate: boolean);
var
  MessageForm : TMessagesForm;
  sBatchID : string;
  sServiceTypeID : string;
  sSystemID : string;
  sFromDate : string;
  sToDate : string;
  curAmountExGST : currency;
  curGSTAmount : currency;
  curExcessAmtExGST : currency;
  curTotalGSTAmount : currency;
  sNewServiceID : string;
  nIncludedUse : integer;
  sIncludedUse : string;
  sTransactionTypeID : string;
  nTransactionTypeID : integer;
  slDC : TStringList;
  qryDC : TQuery;
  sVRTID : string;
  nkByte : integer;
  skByte : string;
  nExcessKByte : integer;
  nGByte : real;
  sGByte : string;
  nUsage : real;
  sGSTRate : string;
  sRateCost : string;
  sServiceType : string;
  fPartCost : real;
//  nSequenceNoLength : integer;
//  nSequenceNoLast : Integer;
  nServiceCount : integer;
begin
  sBatchID := IntToStr(nBatch);
  MessageForm := TMessagesForm.Create(nil);
  MessageForm.Caption := 'Importing Batch:  '+sBatchID;
  MessageForm.Msg.Caption := 'Please wait. Creating revenue transactions...';
  slDC := TStringList.Create;
  qryDC := TQuery.Create(nil);
  qryDC.DatabaseName := 'dbPPdata';
  Str(MAINFORM.GST_RATE:6:4, sGSTRate);

  sVRTID := '0';                                   // Define VRT CustomerID.
  with slDC do begin
    Add('SELECT ID');
    Add('FROM tCustomer');
    Add('WHERE ShipTo LIKE ''VRT''');
    Add('AND Active = 1');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  if not qryDC.Eof then
    sVRTID := IntToStr(qryDC['id']);
  qryDC.Close;
  qryDC.SQL.Clear;

  // Read invoice data usage.                      // Determine total data used.
  with slDC do begin
    Clear;
    Add('SELECT Sum(CAST(Var01 AS integer)) AS kByte');
    Add('FROM tDataCollection');
    Add('WHERE BatchID = '+sBatchID);
    Add('AND RevenueFlag = 0');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  nkByte := qryDC.FieldByName('kByte').AsInteger;
  qryDC.Close;
  Str(nkByte:11,skByte);
  nGByte := nkByte / 1024.0 / 1024.0;
  Str(nGByte:11:4,sGByte);

  // Get the ServiceTypeID linked to this invoice.
  with slDC do begin
    Clear;
    Add('SELECT tServiceType.ID AS "ServiceTypeID", tSystem.ID AS "SystemID"');
    Add('FROM tServiceType');
    Add('INNER JOIN tSystem ON tServiceType.SystemID = tSystem.ID');
    Add('INNER JOIN tBatchType ON tSystem.BatchTypeID = tBatchType.ID');
    Add('INNER JOIN tBatch ON tBatchType.ID = tBatch.BatchTypeID');
    Add('WHERE tBatch.ID = '+sBatchID);
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  sServiceTypeID := qryDC.FieldByName('ServiceTypeID').AsString;
  sSystemID := qryDC.FieldByName('SystemID').AsString;
  qryDC.Close;
  if (Length(sServiceTypeID) = 0) then sServiceTypeID := 'NULL';
  if (Length(sSystemID) = 0) then sSystemID := 'NULL';

  // Now read the transaction type to be used.
  with slDC do begin
    Clear;
    Add('SELECT tTransactionType.ID');
    Add('FROM tTransactionType');
    Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
    Add('AND tTransactionType.Description LIKE ''%Aggregate%''');
    Add('AND tTransactionType.SystemID = '+sSystemID);
    Add('AND tTransactionType.Active = 1');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
  qryDC.Close;
  if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';

  // Now get the data usage included in the fixed cost.
  with slDC do begin
    Clear;
    Add('SELECT CAST(Min(tRate.Length) AS int) AS "Length"');
    Add('FROM tRate');
    Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
    Add('AND tRate.Active = 1');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  nIncludedUse := qryDC.FieldByName('Length').AsInteger;
  sIncludedUse := IntToStr(nIncludedUse);
  qryDC.Close;

  // Look up the appropriate rate from tRate and convert to monthly rates.
  with slDC do begin
    Clear;
    Add('SELECT tRate.Cost');
    Add('FROM tRate');
    Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
    Add('AND tRate.Length = '+sIncludedUse);
    Add('AND tRate.Active = 1');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  curAmountExGST := qryDC.FieldByName('Cost').AsCurrency / 12;
  Str(qryDC.FieldByName('Cost').AsCurrency:11:4,sRateCost);
  qryDC.Close;
  curGSTAmount := curAmountExGST * MAINFORM.GST_RATE;
  curTotalGSTAmount := curGSTAmount;

  // Try to get the dates from the tDataCollection table.
  with slDC do begin
    Clear;
    Add('SELECT DISTINCT tDataCollection.FromDate, tDataCollection.ToDate');
    Add('FROM tDataCollection');
    Add('WHERE tDataCollection.BatchID = '+sBatchID);
    Add('AND tDataCollection.FromDate IS NOT NULL');
    Add('AND tDataCollection.ToDate IS NOT NULL');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  if not (qryDC.Bof and qryDC.Eof) then begin
    sFromDate := ''''+ qryDC.FieldByName('FromDate').AsString +''', ';
    sToDate := ''''+ qryDC.FieldByName('ToDate').AsString +''', ';
  end;
  qryDC.Close;
  // If unsuccessful, use the dates in the tBatch table.
  if (Length(sFromDate) = 0) or (Length(sToDate) = 0) then begin
    // Read the dates from the tBatch table.
    with slDC do begin
      Clear;
      Add('SELECT tBatch.RentStartDate, tBatch.RentEndDate');
      Add('FROM tBatch');
      Add('WHERE tBatch.ID = '+sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    if VarIsNull(qryDC['RentStartDate']) then begin
      sFromDate := 'NULL';
    end
    else begin
      sFromDate := ''''+
      FormattedDateString(qryDC.FieldByName('RentStartDate').AsDateTime)+''', ';
    end;
    if VarIsNull(qryDC['RentEndDate']) then begin
      sToDate := 'NULL';
    end
    else begin
      sToDate := ''''+
        FormattedDateString(qryDC.FieldByName('RentEndDate').AsDateTime)+''', ';
    end;
    qryDC.Close;
  end;  // FromDate or EndDate unknown.

  // Create the Revenue transactions in the tDataCollection table.
  if LumpedCosts then begin                   // add test for "IncludedRate".
    // For lumped sums, all transactions have AmtExGST=0.
    // Charges are lumped into 1 or 2 new transactions. Also,
    // any ServiceID not on the invoice is to be added from the tServiceID
    // table.
    // This applies to SmartBus invoices.

    // First, read the ServiceID to hold the base charges.
    with slDC do begin
      Clear;
      Add('SELECT tParameter.ParmText');
      Add('FROM tParameter');
      Add('WHERE tParameter.Parm LIKE ''BaseDataServiceID''');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    sNewServiceID := qryDC.FieldByName('ParmText').AsString;
    qryDC.Close;
    if (Length(sNewServiceID) = 0) then begin
      MessageDlg('No entry found for Parm=BaseDataServiceID in'+#13#10+
      'the tParameter table.  Data is incomplete.',
        mtError, [mbOk], 0);
    end
    else begin
      // Ready to add the basic (minimum) charge.
      with slDC do begin
        Clear;
        Add('INSERT INTO tDataCollection');
        Add('(BatchID, ServiceID, FromDate, ToDate, TransactionTypeID, '+
            'AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag)');
        Add('VALUES('+ sBatchID+', ');
        Add(''''+sNewServiceID+''', ');
        Add(sFromDate);
        Add(sToDate);
        Add(sTransactionTypeID+', ');
        Add(CurrToStr(curAmountExGST)+', ');
        Add(CurrToStr(curGSTAmount)+', ');
        Add(CurrToStr(curAmountExGST + curGSTAmount)+', ');
        Add('''Y'', ');
        Add('1)');
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.ExecSQL;

      // Now test to see if the total usage exceeds the basic allowance.
      // First, see if the data usage exceeds the included data. Values are in
      // kiloBytes (kB).
      if (nkByte > nIncludedUse) then begin
{
        curExcessAmtExGST := 0;
}
        nExcessKByte := nkByte - nIncludedUse;
        // Read the transaction type to be used.
        with slDC do begin
          Clear;
          Add('SELECT tTransactionType.ID');
          Add('FROM tTransactionType');
          Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
          Add('AND tTransactionType.Description LIKE ''%Excess%''');
          Add('AND tTransactionType.SystemID = '+sSystemID);
          Add('AND tTransactionType.Active = 1');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
        qryDC.Close;
        if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';
        // And determine the appropriate rate (PartCost) for excess usage.
        with slDC do begin
          Clear;
          Add('SELECT tRate.PartCost');
          Add('FROM tRate');
          Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
          Add('AND tRate.Length =');
          Add(' (SELECT Max(tRate.Length)');
          Add(' FROM tRate');
          Add(' WHERE tRate.ServiceTypeID = '+sServiceTypeID);
          Add(' AND tRate.Length <= '+IntToStr(nkByte));
          Add(' AND tRate.Active = 1)');
          Add('AND tRate.Active = 1');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        if (qryDC.Bof and qryDC.Eof) then begin
          curExcessAmtExGST := 0;
        end
        else begin
          // Round up the usage to the nearest MegaByte of data (rounded up).
          nUsage := Trunc((nkByte - nIncludedUse) / 1024.0) + 1;
          curExcessAmtExGST := nUsage * qryDC.FieldByName('PartCost').AsCurrency;
        end;
        qryDC.Close;
        if (curExcessAmtExGST > 0) then begin
          curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;
          curTotalGSTAmount := curTotalGSTAmount + curGSTAmount;
          // First, read the ServiceID to hold the Excess charges.
          with slDC do begin
            Clear;
            Add('SELECT ParmText');
            Add('FROM tParameter');
            Add('WHERE tParameter.Parm LIKE ''ExcessDataServiceID''');
          end;
          qryDC.SQL.Text := slDC.Text;
          qryDC.Open;
          sNewServiceID := qryDC.FieldByName('ParmText').AsString;
          qryDC.Close;
          if (Length(sNewServiceID) = 0) then begin
            MessageDlg('No entry found for Parm=ExcessDataServiceID in'+#13#10+
              'the tParameter table.  Data is incomplete.',
               mtError, [mbOk], 0);
          end
          else begin
            // Ready to add the Excess charges above the included allowance.
            with slDC do begin
              Clear;
              Add('INSERT INTO tDataCollection');
              Add('(BatchID, ServiceID, FromDate, ToDate, TransactionTypeID, '+
                  'AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag)');
              Add('VALUES('+ sBatchID+', ');
              Add(''''+sNewServiceID+''', ');
              Add(sFromDate);
              Add(sToDate);
              Add(sTransactionTypeID+', ');
              Add(CurrToStr(curExcessAmtExGST)+', ');
              Add(CurrToStr(curGSTAmount)+', ');
              Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+', ');
              Add('''Y'', ');
              Add('1)');
            end;
            qryDC.SQL.Text := slDC.Text;
            qryDC.ExecSQL;
          end;  // empty NewServiceID.
        end;  // curAmountExGST > 0.
      end
      else begin
        curExcessAmtExGST := 0;
        nExcessKByte := 0;
      end;  // nkByte > nIncludedUse.
      // Set RevenueFlag=1 for all of the data transactions (with AmountExGST=0).
      // Any other RevenueFlag=NULL will be reset to "0" later.
      with slDC do begin
        Clear;
        Add('UPDATE tDataCollection');
        Add('SET RevenueFlag = 1');
        Add('WHERE BatchID = '+ sBatchID);
        Add('AND AmountExGST = 0');
        Add('AND AmountIncGST = 0');
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.ExecSQL;
      // Get the TransactionTypeID from the tDataCollection table where
      // no charges have been calculated.
      with slDC do begin
        Clear;
        Add('SELECT DISTINCT tDataCollection.TransactionTypeID');
        Add('FROM tDataCollection');
        Add('WHERE tDataCollection.BatchID = '+ sBatchID);
        Add('AND tDataCollection.AmountExGST = 0');
        Add('AND tDataCollection.AmountIncGST = 0');
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.Open;
      nTransactionTypeID := qryDC.FieldByName('TransactionTypeID').AsInteger;
      qryDC.Close;
      // Add those services that did not register any usage on the network.
      AddMissingServiceID(nBatch, nTransactionTypeID);

      // Make sure that all entries for this BatchID do not have
      // RevenueFlag=Null so that revenue Transactions will not be created when
      // transferring to the tTransaction table later using SaveToTransaction().
      with slDC do begin
        Clear;
        Add('UPDATE tDataCollection');
        Add('SET RevenueFlag = 0');
        Add('WHERE RevenueFlag IS NULL');
        Add('AND BatchID = '+sBatchID);
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.ExecSQL;

      // Update the values stored in the tBatch table.
      with slDC do begin
        Clear;
        Add('UPDATE tBatch');
        Add('SET TotalRevenue = '+CurrToStr(curAmountExGST)+',');
//        Add(' PaymentsReceived = '+ IntToStr(nStdkByte)+',');
        Add(' InvoiceNo = '''+ sServiceType +''',');
        Add(' OrderNumber = '''+ IntToStr(nExcesskByte) +''',');
        Add(' RequisitionNumber = '''+ IntToStr(nIncludedUse) +''',');
        Add(' OpeningBalance = '+ CurrToStr(curExcessAmtExGST) +',');
        Add(' TotalCredits = '+ sRateCost +',');
//        Add(' TotalPayable = '+ CurrToStr(curAmountExGST + curExcessAmtExGST) +',');
        Add(' GSTAmount = '+ CurrToStr(curTotalGSTAmount));
        Add('WHERE tBatch.ID = '+ sBatchID);
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.ExecSQL;
    end;  // empty NewServiceID.
  end  // LumpedSum.
  else if IncludedRate then begin
    // A fixed fee for each ServiceID plus excess data for the total account.
    // To start, create the revenue transactions with the fixed monthly fee
    // for each service. Set All item charges to zero - included in rental.
    with slDC do begin
      Clear;
      Add('INSERT INTO tDataCollection');
      Add('(SequenceNo, ServiceID, BatchID,');
      Add(' TxnDate, TxnTime, Origin,');
      Add(' Duration, CallCode, RateDescription,');
      Add(' FromDate, ToDate, TransactionTypeID,');
      Add(' AmountExGST, GSTAmount, GSTFlag, AmountIncGST,');
      Add(' RevenueFlag, Var01, Var04)');
      Add('SELECT tDataCollection.SequenceNo, tDataCollection.ServiceID, '+
          'tDataCollection.BatchID,');
      Add(' tDataCollection.TxnDate, tDataCollection.TxnTime, '+
          'tDataCollection.Origin,');
      Add(' tDataCollection.Duration, tDataCollection.CallCode, '+
          'tDataCollection.RateDescription,');
      Add(' tDataCollection.FromDate, tDataCollection.ToDate, '+
          'tDataCollection.TransactionTypeID,');
      // Set all recorded amounts to zero.
      Add(' (0) AS "AmountExGST", (0) AS "GSTAmount", tDataCollection.GSTFlag,');
      Add(' (0) AS "AmountIncGST", (1) AS "RevenueFlag",'+
      ' tDataCollection.Var01, ('+skByte+') AS "Var04"');
{
      Add(' tDataCollection.AmountExGST, tDataCollection.GSTAmount, '+
          'tDataCollection.GSTFlag,');
      Add(' tDataCollection.AmountIncGST, (1) AS "RevenueFlag",'+
          ' tDataCollection.Var01, ('+sGByte+') AS "Var04"');
}
      Add('FROM tDataCollection');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
      Add('AND tDataCollection.RevenueFlag = 0');
{
      Add('INSERT INTO tDataCollection');
      Add('(SequenceNo, ServiceID, BatchID,');
      Add(' TxnDate, TxnTime, Origin,');
      Add(' Duration, CallCode, RateDescription,');
      Add(' FromDate, ToDate, TransactionTypeID,');
      Add(' AmountExGST, GSTAmount, GSTFlag, AmountIncGST,');
      Add(' RevenueFlag, Var01, Var02, Var03, Var04)');
      Add('SELECT tDataCollection.SequenceNo, tDataCollection.ServiceID, '+
          'tDataCollection.BatchID,');
      Add(' tDataCollection.TxnDate, tDataCollection.TxnTime, '+
          'tDataCollection.Origin,');
      Add(' tDataCollection.Duration, tDataCollection.CallCode, '+
          'tDataCollection.RateDescription,');
      Add(' tDataCollection.FromDate, tDataCollection.ToDate, '+
          'tDataCollection.TransactionTypeID,');
      Add(' DC3.AmountExGST, DC3.GSTAmount, DC3.GSTFlag,');
      Add(' CAST((DC3.AmountExGST + DC3.GSTAmount) AS money) AS '+
          '"AmountIncGST", (1) AS "RevenueFlag",');
      Add(' tDataCollection.Var01, CONVERT(varchar,DC3.Cost,2) AS "Var02",');
      Add(' CONVERT(varchar,DC3.AmountExGST,2) AS "Var03", ('+
          sGByte+') AS "Var04"');
      Add('FROM (SELECT DC2.*,');
      Add(' CAST((CASE WHEN DC2.GSTFlag LIKE ''Y'' THEN (DC2.AmountExGST * '+
          sGSTRate+') ELSE 0 END) AS money) AS "GSTAmount"');
      Add('FROM (SELECT tDataCollection.ID AS "DataCollectionID", '+
          'tServiceID.ServiceTypeID, tGroup.CustomerID, tRate.Cost,');
      Add('  (tRate.Cost / 12) AS "AmountExGST", '+
          '(CASE WHEN tGroup.CustomerID = '+sVRTID+
          ' THEN ''N'' ELSE ''Y'' END) AS "GSTFlag"');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID '+
          'ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('LEFT JOIN tCentre ON tServiceID.CustomerCentreID = tCentre.ID');
      Add('LEFT JOIN tDepartment ON tCentre.DepartmentID = tDepartment.ID');
      Add('LEFT JOIN tGroup ON tDepartment.GroupID = tGroup.ID');
      Add('LEFT JOIN tRate '+
          'ON ((tServiceID.ServiceTypeID = tRate.ServiceTypeID)'+
          ' AND (tRate.Active = 1))');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
      Add('AND tDataCollection.RevenueFlag = 0) AS DC2) AS DC3');
      Add('LEFT JOIN tDataCollection '+
        'ON DC3.DataCollectionID = tDataCollection.ID');
}
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

    // Fix the date fields first.
    sFromDate := Copy(sFromDate, 1, Length(sFromDate) - 2);
    sToDate := Copy(sToDate, 1, Length(sToDate) - 2);
    // Read the transaction type to be used.
    with slDC do begin
      Clear;
      Add('SELECT tTransactionType.ID');
      Add('FROM tTransactionType');
//      Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
      Add('AND tTransactionType.Description LIKE ''%Excess%''');
      Add('AND tTransactionType.SystemID = '+sSystemID);
      Add('AND tTransactionType.Active = 1');
    end;
{          // Not used - refer to read in common area above in this procedure.
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    sTransactionTypeID := qryDC.FieldByName('ID').AsString;
    qryDC.Close;
}
    // Count how many mobile services are being invoiced.
    with slDC do begin
      Clear;
      Add('SELECT COUNT(DISTINCT tDataCollection.ServiceID) AS "TotalCount"');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID '+
          'ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tServiceID.ServiceTypeID = '+sServiceTypeID);
//      Add('AND tDataCollection.RevenueFlag = 0');
//      Add('AND tServiceID.ServiceID IS NOT NULL');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    nServiceCount := qryDC.FieldByName('TotalCount').AsInteger;
    qryDC.Close;

    // Get the rental & data usage included in the fixed cost for each mobile.
    with slDC do begin
      Clear;
      Add('SELECT CAST(tRate.Length AS int) AS "Length", tRate.PartCost,'+
          ' tRate.Cost');
      Add('FROM tRate');
      Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
      Add('AND tRate.Active = 1');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    nIncludedUse := qryDC.FieldByName('Length').AsInteger * nServiceCount;
    sIncludedUse := IntToStr(nIncludedUse);
    fPartCost    := qryDC.FieldByName('PartCost').AsFloat;
    curExcessAmtExGST  := qryDC.FieldByName('Cost').AsCurrency / 12;
    qryDC.Close;
    curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;

    // Save the Rentals.
    with slDC do begin
      Clear;
      Add('INSERT INTO tDataCollection');
      Add('(ServiceID, BatchID, FromDate, ToDate, TransactionTypeID,'+
          ' AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag,'+
          ' Var02, Var03, Var04)');
//          ' Var01, Var02, Var03, Var04)');
//      Add('SELECT DISTINCT tDataCollection.ServiceID,');
      Add('SELECT DC.ServiceID,');
      Add('DC.BatchID,');
      Add(sFromDate+' AS "FromDate",');
      Add(sToDate+' AS "ToDate",');
      Add(sTransactionTypeID+' AS "TransactionTypeID", ');
      Add(CurrToStr(curExcessAmtExGST)+' AS "AmountExGST", ');
      Add(CurrToStr(curGSTAmount)+' AS "GSTAmount",');
      Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+' AS "AmountIncGST",');
      Add('''Y'' AS "GSTflag",');
      Add('(1) AS "RevenueFlag",');
//      Add('CAST(DC.kB AS varchar) AS "Var01",');
      Add('('+ CurrToStr(curExcessAmtExGST * 12) +') AS "Var02",');
      Add('('+ CurrToStr(curExcessAmtExGST) +') AS "Var03",');
      Add('('+ skByte +') AS "Var04"');
      Add('FROM (SELECT DISTINCT tDataCollection.ServiceID, tDataCollection.BatchID');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
      Add('AND tDataCollection.Ratedescription NOT LIKE ''%ROUND%''');
      Add('AND tServiceID.ServiceTypeID = '+sServiceTypeID+') AS DC');
{
      Add('FROM (SELECT DC1.*');
      Add('FROM (SELECT tDataCollection.ServiceID,');
      Add('SUM(CAST(tDataCollection.Var01 AS integer)) AS "kB",');
      Add('tDataCollection.BatchID');
      Add('FROM tDataCollection');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
//      Add('AND tDataCollection.RevenueFlag = 0');
      Add('AND tDataCollection.Ratedescription NOT LIKE ''%ROUND%''');
      Add('GROUP BY tDataCollection.ServiceID, tDataCollection.BatchID) AS DC1');
      Add('LEFT JOIN tServiceID ON DC1.ServiceID = tServiceID.ServiceID');
      Add('WHERE tServiceID.ServiceTypeID = '+sServiceTypeID+') AS DC');
}
      Add('ORDER BY DC.ServiceID');
    end;  
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

{
    // Now determine if there are any excess usage charges to be claimed.
    // Get the ServiceTypeID linked to this invoice.
    with slDC do begin
      Clear;
      Add('SELECT DISTINCT tServiceID.ServiceTypeID');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID '+
          'ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tDataCollection.Ratedescription NOT LIKE ''%ROUND%''');
      Add('AND tServiceID.ServiceTypeID IS NOT NULL');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    sServiceTypeID := qryDC.FieldByName('ServiceTypeID').AsString;
    qryDC.Close;
}
    if (Length(sServiceTypeID) = 0) then sServiceTypeID := 'NULL';

    // Test to see if the total usage exceeds the nett included amount.
    if (nkByte < nIncludedUse) then begin
      // No excess fees are to be applied.
      curExcessAmtExGST := 0;
      nkByte := 0;
    end
    else begin
      // Included data bandwidth exceeded - charge for the excess amount.
      curExcessAmtExGST := (nkByte - nIncludedUse) * fPartCost;
      if (curExcessAmtExGST > 0) then begin
        curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;
        // First, read the ServiceID to hold the Excess charges.
        with slDC do begin
          Clear;
          Add('SELECT ParmText');
          Add('FROM tParameter');
          Add('WHERE tParameter.Parm LIKE ''ExcessDataServiceID''');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        sNewServiceID := qryDC.FieldByName('ParmText').AsString;
        qryDC.Close;
        // Now read the transaction type to be used.
        with slDC do begin
          Clear;
          Add('SELECT tTransactionType.ID');
          Add('FROM tTransactionType');
          Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
          Add('AND tTransactionType.Description LIKE ''%Excess%''');
          Add('AND tTransactionType.Active = 1');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
        qryDC.Close;
        if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';
{
        // Get the number of characters in the sequence number.
        with slDC do begin
          Clear;
          Add('SELECT tDataCollection.SequenceNo');
          Add('FROM tDataCollection');
          Add('WHERE tDataCollection.BatchID = '+sBatchID);
          Add('AND tDataCollection.RevenueFlag = 0');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        nSequenceNoLength := Length(qryDC.FieldByName('SequenceNo').AsString);
        qryDC.Close;
        // Get the last entry in sequence number.
        qryDC.SQL.Strings[0] :=
          'SELECT Max(CAST(tDataCollection.SequenceNo AS INT)) AS "SequenceNo"';
        qryDC.Open;
        nSequenceNoLast := qryDC.FieldByName('SequenceNo').AsInteger;
        qryDC.Close;
}
        // Ready to add the Excess charges above the included allowance.
{
        // Fix the date fields first.
        sFromDate := Copy(sFromDate, 1, Length(sFromDate) - 2);
        sToDate := Copy(sToDate, 1, Length(sToDate) - 2);
}
        with slDC do begin
          Clear;
          Add('INSERT INTO tDataCollection');
          Add('(ServiceID, BatchID, FromDate, ToDate, TransactionTypeID,'+
              ' AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag,'+
              ' Var02)');
          Add('VALUES('''+sNewServiceID+''',');
          Add(sBatchID+',');
          Add(sFromDate+',');
          Add(sToDate+',');
          Add(sTransactionTypeID+',');
          Add(CurrToStr(curExcessAmtExGST)+',');
          Add(CurrToStr(curGSTAmount)+',');
          Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+',');
          Add('''Y'',');
          Add('1, '+CurrToStr(curExcessAmtExGST)+')');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.ExecSQL;
{
        // And now the sequence numbers
...to be continued...
}

      end;  // curExcessAmtExGST.
    end;  // Excess data fees.
    // Make sure that all entries for this BatchID do not have
    // RevenueFlag=Null so that revenue Transactions will not be created when
    // transferring to the tTransaction table later using SaveToTransaction().
    with slDC do begin
      Clear;
      Add('UPDATE tDataCollection');
      Add('SET RevenueFlag = 0');
      Add('WHERE RevenueFlag IS NULL');
      Add('AND BatchID = '+sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

    // Update the values stored in the tBatch table.
    with slDC do begin
      Clear;
      Add('UPDATE tBatch');
      Add('SET TotalRevenue = BatchRate.RevenueAmt,');
      Add(' TotalCredits = '+ FloatToStrF(fPartCost,ffFixed,10,5) +',');
      Add(' OpeningBalance = '+ CurrToStr(curExcessAmtExGST) +',');
      Add(' OrderNumber = '''+ IntToStr(nkByte) +''',');
      Add(' TenderNo = '''+ IntToStr(nServiceCount) +''',');
      Add(' InvoiceNo = '''+ sServiceTypeID +''',');
      Add(' RequisitionNumber = '''+ sIncludedUse +'''');
      Add('FROM (SELECT Sum(tDataCollection.AmountExGST) AS "RevenueAmt"');
      Add('FROM tDataCollection');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tDataCollection.RevenueFlag = 1) AS BatchRate');
      Add('WHERE tBatch.ID = '+ sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;
  end  // IncludedRate.
  else begin
    // The usage for each ServiceID is calculated and the transactions updated.
    with slDC do begin
      Clear;
      Add('INSERT INTO tDataCollection');
      Add('(SequenceNo, ServiceID, BatchID,');
      Add(' TxnDate, TxnTime, Origin,');
      Add(' Duration, CallCode, RateDescription,');
      Add(' FromDate, ToDate, TransactionTypeID,');
      Add(' AmountExGST, GSTAmount, GSTFlag, AmountIncGST,');
      Add(' RevenueFlag, Var01, Var02, Var03, Var04)');
      Add('SELECT tDataCollection.SequenceNo, tDataCollection.ServiceID, '+
          'tDataCollection.BatchID,');
      Add(' tDataCollection.TxnDate, tDataCollection.TxnTime, '+
          'tDataCollection.Origin,');
      Add(' tDataCollection.Duration, tDataCollection.CallCode, '+
          'tDataCollection.RateDescription,');
      Add(' tDataCollection.FromDate, tDataCollection.ToDate, '+
          'tDataCollection.TransactionTypeID,');
      Add(' DC3.AmountExGST, DC3.GSTAmount, DC3.GSTFlag,');
      Add(' CAST((DC3.AmountExGST + DC3.GSTAmount) AS money) AS '+
          '"AmountIncGST", (1) AS "RevenueFlag",');
      Add(' tDataCollection.Var01, CONVERT(varchar,DC3.Cost,2) AS "Var02",');
      Add(' CONVERT(varchar,DC3.AmountExGST,2) AS "Var03", ('+
          sGByte+') AS "Var04"');
      Add('FROM (SELECT DC2.*,');
      Add(' CAST((CASE WHEN DC2.GSTFlag LIKE ''Y'' THEN (DC2.AmountExGST * '+
          sGSTRate+') ELSE 0 END) AS money) AS "GSTAmount"');
      Add('FROM (SELECT DC1.DataCollectionID, DC1.Cost,');
      Add(' (CAST(tDataCollection.Var01 AS Integer)*Cost) AS "AmountExGST",');
      Add(' (CASE WHEN DC1.CustomerID = '+sVRTID+
          ' THEN ''N'' ELSE ''Y'' END) AS "GSTFlag"');
      // Include Incremental values in determining the final charge rate (cost).
      Add('FROM (SELECT DCFullRate.DataCollectionID, '+
          'DCFullRate.ServiceTypeID, DCFullRate.CustomerID,');
      Add(' (DCFullRate.Cost + (CASE WHEN '+sGByte+' > DCFullRate.Usage THEN');
      Add(' (DCFullRate.PartCost * ('+sGByte+' - DCFullRate.Usage)) '+
          'ELSE 0 END)) AS "Cost"');
      // Use Min() rate a/c price is reduced for more usage.
      Add('FROM (SELECT DCRate.*, tRate.Length AS "Usage",');
      Add(' (CASE WHEN tRate.PartCost IS NULL THEN 0 ELSE tRate.PartCost END'+
          ') AS "PartCost"');
      Add('FROM (SELECT DC0.*, Min(tRate.Cost) AS "Cost"');
      Add('FROM (SELECT tDataCollection.ID AS "DataCollectionID",');
      Add(' tServiceID.ServiceTypeID, tGroup.CustomerID');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID '+
          'ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('LEFT JOIN tCentre ON tServiceID.CustomerCentreID = tCentre.ID');
      Add('LEFT JOIN tDepartment ON tCentre.DepartmentID = tDepartment.ID');
      Add('LEFT JOIN tGroup ON tDepartment.GroupID = tGroup.ID');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
      Add('AND tDataCollection.RevenueFlag = 0) AS DC0');
      Add('LEFT JOIN tRate '+
          'ON ((DC0.ServiceTypeID = tRate.ServiceTypeID) AND (tRate.Length <= '+
          sGByte+') AND (tRate.Active = 1))');
      Add('GROUP BY DC0.DataCollectionID, DC0.ServiceTypeID, DC0.CustomerID'+
          ') AS DCRate');
      Add('LEFT JOIN tRate ON ((DCRate.ServiceTypeID = tRate.ServiceTypeID)');
      Add(' AND (tRate.Cost = DCRate.Cost) AND (tRate.Active = 1))');
      Add('WHERE tRate.Cost = DCRate.Cost) AS DCFullRate) AS DC1');
      Add('LEFT JOIN tDataCollection '+
          'ON DC1.DataCollectionID = tDataCollection.ID) AS DC2) AS DC3');
      Add('LEFT JOIN tDataCollection '+
          'ON DC3.DataCollectionID = tDataCollection.ID');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

    // Read a value to include in the update of the tBatch table.
    with slDC do begin
      Clear;
      Add('SELECT CAST(CAST(DC1.Cost AS real) AS varchar) AS "RateCost",');
      Add(' CAST(ServiceTypeID AS varchar) AS "ServiceType"');
      Add('FROM (SELECT Min(tRate.Cost) AS "Cost", DC0.ServiceTypeID');
      Add('FROM (SELECT tServiceID.ServiceTypeID');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tDataCollection.RevenueFlag = 0) AS DC0');
      Add('LEFT JOIN tRate ON ((DC0.ServiceTypeID = tRate.ServiceTypeID)');
      Add(' AND (tRate.Active = 1)');
      Add(' AND (tRate.Length <= '+ sGByte+'))');
      Add('GROUP BY DC0.ServiceTypeID) AS DC1');
      Add('WHERE DC1.Cost <> 0');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    sRateCost := qryDC.FieldByName('RateCost').AsString;
    sServiceType := qryDC.FieldByName('ServiceType').AsString;
    if (Length(sRateCost) = 0) then sRateCost := '0';  // Avoids invalid "Set =".
    qryDC.Close;

    // Make sure that all entries for this BatchID do not have
    // RevenueFlag=Null so that extra revenue Transactions will not be created
    // when transferring to tTransaction later using SaveToTransaction().
    with slDC do begin
      Clear;
      Add('UPDATE tDataCollection');
      Add('SET RevenueFlag = 0');
      Add('WHERE RevenueFlag IS NULL');
      Add('AND BatchID = '+sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

    // Update some more of the values stored in the tBatch table.
    with slDC do begin
      Clear;
      Add('UPDATE tBatch');
      Add('SET TotalRevenue = BatchRate.RevenueAmt,');
      Add(' TotalCredits = '+ sRateCost +',');
      Add(' OrderNumber = '+ sGByte +',');
      Add(' InvoiceNo = '''+ sServiceType +''',');
      Add(' RequisitionNumber = '+ IntToStr(nkByte));
      Add('FROM (SELECT Sum(tDataCollection.AmountExGST) AS "RevenueAmt"');
      Add('FROM tDataCollection');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tDataCollection.RevenueFlag = 1) AS BatchRate');
      Add('WHERE tBatch.ID = '+ sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;
  end;  // LumpedCosts.

  // Done. Do some housework.
  MessageForm.Close;
  MessageForm.Free;
  slDC.Free;
  qryDC.Free;
end;  // ApplyTelstraDataRates.

end.
 