unit Vodafone;

interface

uses
  Messages, Classes, SysUtils, Graphics, Dialogs,
  Db, DbTables;

  procedure GetVodaReferences(var lContinue: boolean);
  procedure GetMobileData( const nBatch: LongInt; const FName : string;
    var lReturnCode: Boolean);
  procedure VodafoneInvoice( const nBatch: LongInt; sDirectory: string );

implementation

uses
  CriticalTests, Math,
//  Main, DataMod, Progress, SaveData, ClrTemp, DateFunctions, FileFunctions,
  Main, DataMod, Progress, DateFunctions, FileFunctions,
  MyMessage, GenFns;

var
  nSupplierID: integer;        // Set in BillingInput().
  sSupplierID: string;         // Set in BillingInput().
  qryVodafone : TQuery;        // Set in BillingInput().
  nAirCreditPercent : real; // Vodafone discount for calls made.
  sRentCodeID : string;        // Set in BillingInput().
  sCallCodeID : string;        // Set in BillingInput().
  sCallAirCodeID : string;     // Discount for calls made.
  sCallUsageCodeID : string;   // Corporate Usage discount.
  sOtherCodeID : string;       // Set in BillingInput().
  sDiscountCodeID : string;    // Set in BillingInput().
  sLastCallCode : string;
  sLastCallTransactionTypeID : string;
  sLastSupplierCallCodeID : string;
  sSupplierRentCodeID : string;
  sSupplierCallCodeID : string;
  sSupplierCallAirCodeID : string;
  sSupplierUsageCallCodeID : string;
  sSupplierOtherCodeID : string;
  sSupplierDiscountCodeID : string;

  curTotalExGST  : currency;
  curTotalGST    : currency;
  curTotalIncGST : currency;
//  DefaultJob : String;
//  sCreditAccount : string;    // Read from Config-acct for all credits.
//  lSaveRentCredit : boolean;  // Save credits on rentals.
//  lSaveCallCredit : boolean;  // Save credits on calls.
//  lSaveCredit : Boolean;      // True if sCreditAccount is valid.

procedure GetVodaReferences(var lContinue: boolean);
begin
  // initialise the Static table elements defined as Private for this form.
  qryVodafone := TQuery.Create(MainForm);
  qryVodafone.DatabaseName := 'dbPPdata';
  qryVodafone.SQL.Add('SELECT ParmNumber FROM tParameter');
  qryVodafone.SQL.Add('WHERE Parm LIKE ''VodafoneAirTimeCreditPercent''');
  qryVodafone.Open;
  if qryVodafone.eof then begin
    nAirCreditPercent := 15;                 // Default = 15%.
    qryVodafone.Close;
    qryVodafone.SQL.Clear;
    qryVodafone.SQL.Add('INSERT INTO tParameter');
    qryVodafone.SQL.Add('(Parm,ParmField,ParmNumber,ParmUserDefined)');
    qryVodafone.SQL.Add('VALUES (''VodafoneAirTimeCreditPercent'','+
      '''ParmNumber'',15,0)');
    qryVodafone.ExecSQL;
  end
  else begin
    nAirCreditPercent := qryVodafone.FieldByName('ParmNumber').AsInteger;
  end;  // if qryLook.eof then..else.
  qryVodafone.Close;
  qryVodafone.SQL.Clear;
  qryVodafone.SQL.Add('SELECT id');
  qryVodafone.SQL.Add('FROM tSupplier');
  qryVodafone.SQL.Add('WHERE SupplierName LIKE ''VODAFONE%''');
  qryVodafone.Open;
  if qryVodafone.Eof then begin
    MessageDlg('VODAFONE not found in tSupplier. Cannot import the Invoice.',
      mtWarning, [mbOk], 0);
    lContinue := False;
  end
  else begin
    lContinue := True;
    nSupplierID := qryVodafone['id'];
    sSupplierID := IntToStr(nSupplierID);
    qryVodafone.Close;
    qryVodafone.SQL.Clear;
    qryVodafone.SQL.Add('SELECT Code, CAST(TransactionTypeID AS VARCHAR(20)) '+
      'AS TransactionType, CAST(ID AS VARCHAR(20)) AS SupplierCodeID');
    qryVodafone.SQL.Add('FROM tSupplierServiceCode');
    qryVodafone.SQL.Add('WHERE SupplierID = '+sSupplierID);
    qryVodafone.SQL.Add('AND Active = 1');
    qryVodafone.SQL.Add('AND Code LIKE ''V_R''');                  // Rent.
    qryVodafone.Open;
    if qryVodafone.Eof then sRentCodeID := '999991'
    else begin
      sRentCodeID := Trim(qryVodafone['TransactionType']);
      sSupplierRentCodeID := Trim(qryVodafone['SupplierCodeID']);
    end;
    qryVodafone.Close;
    qryVodafone.SQL.Strings[qryVodafone.SQL.Count-1] :=
      'AND Code LIKE ''NA''';                                      // Call.
    qryVodafone.Open;
    if qryVodafone.Eof then sCallCodeID := '999992'
    else begin
      sCallCodeID := Trim(qryVodafone['TransactionType']);
      sSupplierCallCodeID := Trim(qryVodafone['SupplierCodeID']);
    end;
    qryVodafone.Close;
    qryVodafone.SQL.Strings[qryVodafone.SQL.Count-1] :=
      'AND Code LIKE ''V_A''';                                     // CallAir.
    qryVodafone.Open;
    if qryVodafone.Eof then sCallAirCodeID := '999993'
    else begin
      sCallAirCodeID := qryVodafone['TransactionType'];
      sSupplierCallAirCodeID := Trim(qryVodafone['SupplierCodeID']);
    end;
    qryVodafone.Close;
    qryVodafone.SQL.Strings[qryVodafone.SQL.Count-1] :=
      'AND Code LIKE ''V_U''';                                     // CallUsage.
    qryVodafone.Open;
    if qryVodafone.Eof then sCallUsageCodeID := '999994'
    else begin
      sCallUsageCodeID := qryVodafone['TransactionType'];
      sSupplierUsageCallCodeID := Trim(qryVodafone['SupplierCodeID']);
    end;
    qryVodafone.Close;
    qryVodafone.SQL.Strings[qryVodafone.SQL.Count-1] :=
      'AND Code LIKE ''V_O''';                                     // Other.
    qryVodafone.Open;
    if qryVodafone.Eof then sOtherCodeID := '999995'
    else begin
      sOtherCodeID := qryVodafone['TransactionType'];
      sSupplierOtherCodeID := Trim(qryVodafone['SupplierCodeID']);
    end;
    qryVodafone.Close;
    qryVodafone.SQL.Strings[qryVodafone.SQL.Count-1] :=
      'AND Code LIKE ''V_D''';                                     // Discount.
    qryVodafone.Open;
    if qryVodafone.Eof then sDiscountCodeID := '999996'
    else begin
      sDiscountCodeID := Trim(qryVodafone['TransactionType']);
      sSupplierDiscountCodeID := Trim(qryVodafone['SupplierCodeID']);
    end;
    qryVodafone.Close;
    qryVodafone.SQL.Delete(qryVodafone.SQL.Count-1);
    qryVodafone.Open;
  end;  // if qryVodafone.Eof then begin.
end;  // procedure GetVodaReferences().

procedure GetMobileData( const nBatch: LongInt; const FName : string;
  var lReturnCode: Boolean );
// Data handled is for calls made from mobile phones.
// This function scans the input file for relevant data and saves it.
//
// Variable sFieldValueList contains the billing codes processed below:
// 260500  Start of details for a service.
// 361005  Charges summary (all types) - use for NSTDCH and USAGE.
// 361505  Rental credit details.
// 363005  Rental details.
// 363505  Call details.
// 231099  Total invoice value.
// 130000  Invoicing dates.
// ONLY the above values will be processed!!!
const
  RECORD_BYTES_C: Integer = 150;       // Typical minimum bytes per record.
  PROGRESS_COUNTER_C: Integer=100;     // Update Progress every 100 records read
  PROGRESS_MAX_C: Integer = 100;       // Maximum progress is 100%.
  LINE_C: integer = 0;                 // Bill entry number.
  CODE_C: integer = 1;                 // Billing code.
  ABBR_C: integer = 2;                 // Abbreviation.
  DESC1_C: integer = 3;                // Description field.
  DESC2_C: integer = 4;                // Description field.
  DESC3_C: integer = 5;                // Description field.
  DESC4_C: integer = 6;                // Description field.
  DESC5_C: integer = 7;                // Description field.
  DESC6_C: integer = 8;                // Peak/Off Peak.
  CODE2_C: integer = 9;                // Rate description.
  CODE3_C: integer = 10;               // Call duration.
  COST2_C: integer = 13;               // Call charge field.
  DESC7_C: integer = 14;               // GST flag.
var
  lContinue : boolean;
  nFileSize: Integer;                  // Actual file size.
  InputTextFile: TextFile;
  InputTextFileBuffer: array[1..4096] of Char;  // 4KB buffer.
  sCaption : String;                   // Used for modifying Progress caption.
  ReadString: string;                  // Read in from the text file.
  ConvertString: TStringList;          // Used to convert SDF format to strings.
  nProgressCounted : integer;          // Count each set of PROGRESS_COUNTER_C.
  Progress: TProgressForm;
  nCount: Integer;
  nSoFar: Integer;
  nFactor: Real;
  vSubString : variant;                // - ditto, used in the Locate() function.
  sFieldValueList : String;  // Lists those billing codes to be handled here.
  lSaveToTable : boolean;
  curAmountCharged : Currency;
  curAmountGST : Currency;
  curTotalCall : currency;
  curActualCall : currency;

  sBillCode: String;
  cAbbr: String;
  sServiceID: string;
  SubString: string;
  sBatch : string;
  sListFields : string;
  sListValues : string;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  slAddFieldNames : TStringList;         // Calls to AddToTable() only.
  slAddFieldValues : TStringList;        // Calls to AddToTable() only.
  nListValues : integer;
  qryDC : TQuery;
  lFound : boolean;
  
//  nCreditRent : Integer;
//  nCreditCall : Integer;
  sRentFrom : string;
  sRentTill : string;
  sCallFrom : string;
  sCallTill : string;
begin
  curTotalCall := 0.00;
  curActualCall := 0.00;
  lReturnCode := False;
  sBatch := IntToStr( nBatch );
//  sSupplierID := 'VODAFONE';
  sServiceID := 'NULL,';
  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  slAddFieldNames := TStringList.Create;         // Calls to AddToTable() only.
  slAddFieldValues := TStringList.Create;        // Calls to AddToTable() only.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';
  
  sRentFrom := '';
  sRentTill := '';
  sCallFrom := '';
  sCallTill := '';

  GetVodaReferences(lContinue);
  
  if lContinue then begin
    nFileSize := GetFileSize(FName);
    if (nFileSize > 0) then
      nFactor := 100.0 * RECORD_BYTES_C / nFileSize
    else
      nFactor := 100.0;

    // Open the input (text) billing file for reading.
    AssignFile( InputTextFile, Fname );
    // Bigger buffer for faster reads.
    System.SetTextBuf(InputTextFile, InputTextFileBuffer);
    Reset( InputTextFile );

    Progress := TProgressForm.Create(MainForm);
    Progress.Caption := 'Reading The Input Text File';
    Progress.btnCancel.Visible := False;
    // Convert nFileSize to x100kBytes (based on 1.0MB=1,048,576B).
    nFileSize := nFileSize div 104858;
    Str( (nFileSize / 10.0):5:1, sCaption );
    Progress.Msg.Caption := 'Reading ('+Trim(sCaption)+'MB) ...';
    Progress.Show;
    Progress.ProcessWinMessages(MainForm);

//    sFieldValueList := ',260500,361005,361505,363005,363505,130000,';
    sFieldValueList := ',260500,361005,361505,363005,363505,130000,231099,';
    ConvertString := TStringList.Create; // Used to convert SDF format to strings.
    try
      // Get the first occurence of the search string.
      Readln( InputTextFile, ReadString );

      // To be sure the file isn't empty too...
      if (Length( ReadString ) > 0) or (not Eof( InputTextFile )) then begin
        Reset( InputTextFile );
        // Initiate the data variables.
        nProgressCounted := 0;
        nCount := 0;
        // Continue to search through the remaining text of the file
        // for occurrences of the search string.  On each each find,
        // process the resultant string.
        repeat
          Progress.ProcessWinMessages(MainForm);
          Readln( InputTextFile, ReadString ); // Read in the next line of data.
          if (Length(ReadString) = 0) then begin
            lSaveToTable := False;
          end
          else begin
            ConvertString.CommaText := ReadString;      // Decode the line read.
            sBillCode := Trim(ConvertString.Strings[CODE_C]);
            if (Length(sBillCode) = 0) then begin
              lSaveToTable := False;
            end
            else begin
              lSaveToTable := (Pos(sBillCode,sFieldValueList)>0);
            end;
          end;

          if lSaveToTable then begin
            // Save the strings into the table.
            FieldStringList.Clear;
            ValueStringList.Clear;
            sListFields := '(';
            sListValues := 'VALUES (';
            cAbbr := Trim(UpperCase(ConvertString.Strings[ABBR_C]));
            SubString := Trim(ConvertString.Strings[LINE_C]);
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'SequenceNo,';
              sListValues := sListValues +''''+ SubString +''',';
            end;
            sListFields := sListFields + 'ServiceID,';
            sListValues := sListValues + sServiceID;

            // Process each billing code separately.

            // Calls (USAGE).
            if (sBillCode = '363505') then begin
              SubString := Trim(ConvertString.Strings[DESC1_C]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryVodafone do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastCallTransactionTypeID :=
                      qryVodafone.FieldByName('TransactionType').AsString;
                    sLastSupplierCallCodeID :=
                      qryVodafone.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastCallTransactionTypeID := sCallCodeID;
                    sLastSupplierCallCodeID := sSupplierCallCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastCallTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryVodafone do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryOptus.
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastCallTransactionTypeID+','+
                  sLastSupplierCallCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sCallCodeID+','+
                  sSupplierCallCodeID+',';
              end;  // CallCode.

              SubString := Trim(ConvertString.Strings[DESC2_C]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;
              SubString := Trim(ConvertString.Strings[DESC3_C]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnTime,';
                sListValues := sListValues +''''+ Copy(SubString,1,2)+
                  ':'+Copy(SubString,3,2)+':'+Copy(SubString,5,2) +''',';
              end;
              SubString := Trim(ConvertString.Strings[CODE3_C]);
              if (Length(SubString) > 0) then begin
                SubString := ConvertTimeToSecs(CleanString(SubString));
                sListFields := sListFields + 'Duration,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';
              SubString := Trim(ConvertString.Strings[DESC5_C]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'DialledNumber,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              SubString := Trim(ConvertString.Strings[CODE2_C]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Peak,';
                if (UpperCase(SubString) = 'PEAK') then
                  sListValues := sListValues +'1,'
                else
                  sListValues := sListValues +'0,';
                if (Length(Trim(ConvertString.Strings[DESC6_C])) > 0) then
                  SubString := SubString +' - ';
                SubString := SubString + Trim(ConvertString.Strings[DESC6_C]);
              end;  // if (Length(SubString) > 0) then begin.
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                if (Length(Trim(ConvertString.Strings[DESC1_C])) > 0) then
                  SubString := Trim(ConvertString.Strings[DESC1_C])+' - '+SubString;
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';
              SubString := Trim(ConvertString.Strings[COST2_C]);
              curAmountCharged := StrToCurr(SubString);
              curAmountGST := curFormat(curAmountCharged*0.1);
              curTotalCall := curSum(curTotalCall, curAmountCharged); // Running total.
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              SubString := Trim(ConvertString.Strings[DESC7_C]);
              sListFields := sListFields + 'GSTFlag,';
              sListValues := sListValues + ''''+SubString+''',';
              sListFields := sListFields + 'AmountIncGST,GSTAmount,';
              if (UpperCase(SubString)='Y') then begin
                sListValues := sListValues +
                  CurrToStr(curSum(curAmountCharged,curAmountGST)) +','+
                  CurrToStr(curAmountGST) +',';
              end
              else begin
                sListValues := sListValues + CurrToStr(curAmountCharged) +',0,';
              end;  // if GSTFlag='Y' then begin.
              sListFields := sListFields + 'FromDate,';
              sListValues := sListValues + sCallFrom;
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues + sCallTill;
            end  // else if (sBillCode = '363505') then.

            // Rental (CREDIT).
            else if (sBillCode = '361505') then begin
              sListFields := sListFields + 'TransactionTypeID, SupplierServiceCodeID,';
              sListValues := sListValues + sDiscountCodeID+','+
                  sSupplierDiscountCodeID+',';
              SubString := Trim(ConvertString.Strings[DESC3_C]);
              curAmountCharged := StrToCurr(SubString);
              curAmountGST := curFormat(curAmountCharged*0.1);
              if (Round(curAmountCharged*100)<>0) then begin
                sListFields := sListFields + 'AmountExGST,';
                sListValues := sListValues + SubString +',';
                SubString := Trim(ConvertString.Strings[DESC4_C]);
                sListFields := sListFields + 'GSTFlag,';
                sListValues := sListValues + ''''+SubString+''',';
                sListFields := sListFields + 'AmountIncGST,GSTAmount,';
                if (UpperCase(SubString)='Y') then begin
                  sListValues := sListValues +
                    CurrToStr(curSum(curAmountCharged,curAmountGST)) +','+
                    CurrToStr(curAmountGST) +',';
                end
                else begin
                  sListValues := sListValues + CurrToStr(curAmountCharged) +',0,';
                end;  // if GSTFlag='Y' then begin.
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+
                  YMD2DMY(ConvertString.Strings[DESC1_C]) +''',';
                FieldStringList.Add(sListFields);
                ValueStringList.Add(sListValues);
                sListFields := '';
                sListValues := '';
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+
                  CleanString(Trim(ConvertString.Strings[DESC2_C])) +''',';
              end
              else begin
                lSaveToTable := False;
              end;  // if (Round(StrToCurr(SubString)*100)<>0) then begin.
            end  // sBillCode = '361505'.

            // **********  Payment required.  **********
            // Rental (STDCHG).
            else if (sBillCode = '363005') then begin
              sListFields := sListFields + 'TransactionTypeID, SupplierServiceCodeID,';
              sListValues := sListValues + sRentCodeID+','+
                  sSupplierRentCodeID+',';
              SubString := Trim(ConvertString.Strings[DESC3_C]);
              curAmountCharged := StrToCurr(SubString);
              curAmountGST := curFormat(curAmountCharged*0.1);
              if (Round(curAmountCharged*100)<>0) then begin
                sListFields := sListFields + 'AmountExGST,';
                sListValues := sListValues + SubString +',';
                SubString := Trim(ConvertString.Strings[DESC4_C]);
                sListFields := sListFields + 'GSTFlag,';
                sListValues := sListValues + ''''+SubString+''',';
                sListFields := sListFields + 'AmountIncGST,GSTAmount,';
                if (UpperCase(SubString)='Y') then begin
                  sListValues := sListValues +
                    CurrToStr(curSum(curAmountCharged,curAmountGST)) +','+
                    CurrToStr(curAmountGST) +',';
                end
                else begin
                  sListValues := sListValues + CurrToStr(curAmountCharged) +',0,';
                end;  // if GSTFlag='Y' then begin.
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+
                  YMD2DMY(ConvertString.Strings[ABBR_C]) +''',';
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+
                  YMD2DMY(ConvertString.Strings[DESC1_C]) +''',';
                FieldStringList.Add(sListFields);
                ValueStringList.Add(sListValues);
                sListFields := '';
                sListValues := '';
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+
                  CleanString(Trim(ConvertString.Strings[DESC2_C])) +''',';
              end
              else begin
                lSaveToTable := False;
              end;  // if (Round(StrToCurr(SubString)*100)<>0) then begin.
            end  // '363005'.

            // **********  New phone number / service.  ***********************
            else if (sBillCode = '260500') then begin
              // Save any discounts (difference between calculated & actual calls).
              //   Airtime credit is 15% of call charges, rest of difference is
              //   Corporate Sales Recurring Usage Credit.
              if (Copy(sServiceID,1,4) = 'NULL') or
                (Length(sServiceID) = 0) then     // first loop - no data.
                lSaveToTable := False
              else begin
                // Use Int() to round down (& NOT up) to the nearest cent.
                curAmountCharged := Int(-nAirCreditPercent*curTotalCall)/100.0000;
                // Deduct discount sum from the total charge.
                curTotalCall := curSum(curAmountCharged, curTotalCall);
                if (Round(curAmountCharged*100)<>0) then begin
                  sListFields := sListFields + 'TransactionTypeID, '+
                    'SupplierServiceCodeID,';
                  sListValues := sListValues + sCallAirCodeID+','+
                    sSupplierCallAirCodeID+',';
                  sListFields := sListFields + 'AmountExGST,GSTFlag,';
                  sListValues := sListValues +
                    CurrToStr(curFormat(curAmountCharged)) +',''Y'',';
                  curAmountGST := curFormat(curAmountCharged*0.1);
                  sListFields := sListFields + 'AmountIncGST,GSTAmount,';
                  sListValues := sListValues +
                    CurrToStr(curSum(curAmountCharged,curAmountGST)) +','+
                    CurrToStr(curAmountGST) +',';
                  sListFields := sListFields + 'FromDate,';
                  sListValues := sListValues + sCallFrom;
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues + sCallTill;
                  FieldStringList.Add(sListFields);
                  ValueStringList.Add(sListValues);
                  sListFields := '';
                  sListValues := '';
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +'''Vodafone airtime credit'',';

                  // Now make the SQL statement to save to the database table.
                  sListFields := sListFields + 'BatchID)';
                  sListValues := sListValues + sBatch +')';
                  FieldStringList.Add(sListFields);
                  ValueStringList.Add(sListValues);
                  qryStringList.Clear;
                  qryStringList.Add('INSERT INTO tDataCollection');
                  for nListValues := 0 to FieldStringList.Count-1 do begin
                     qryStringList.Add(FieldStringList[nListValues]);
                  end;
                  for nListValues := 0 to ValueStringList.Count-1 do begin
                    qryStringList.Add(ValueStringList[nListValues]);
                  end;
                  // Update and execute the query.
                  qryDC.Close;
                  qryDC.SQL := qryStringList;
                  qryDC.ExecSQL;
                end;  // curAmountCharged <> 0.

                // Now save the Usage Credit - makes the numbers add up.
                FieldStringList.Clear;
                ValueStringList.Clear;
                curAmountCharged := curSum((-1*curTotalCall), curActualCall);
                if (Round(curAmountCharged*100)=0) then lSaveToTable := False
                else begin
                  sListFields := '(';
                  sListValues := 'VALUES (';
                  cAbbr := Trim(UpperCase(ConvertString.Strings[ABBR_C]));
                  SubString := Trim(ConvertString.Strings[LINE_C]);
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'SequenceNo,';
                    sListValues := sListValues +''''+ SubString +''',';
                  end;
                  sListFields := sListFields + 'ServiceID,';
                  sListValues := sListValues + sServiceID;
                  sListFields := sListFields + 'TransactionTypeID, '+
                    'SupplierServiceCodeID,';
                  sListValues := sListValues + sCallUsageCodeID+','+
                    sSupplierUsageCallCodeID +',';
                  sListFields := sListFields + 'AmountExGST,GSTFlag,';
                  sListValues := sListValues +
                    CurrToStr(curFormat(curAmountCharged)) +',''Y'',';
                  curAmountGST := curFormat(curAmountCharged*0.1);
                  sListFields := sListFields + 'AmountIncGST,GSTAmount,';
                  sListValues := sListValues +
                    CurrToStr(curSum(curAmountCharged,curAmountGST)) +','+
                    CurrToStr(curAmountGST) +',';
                  sListFields := sListFields + 'FromDate,';
                  sListValues := sListValues + sCallFrom;
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues + sCallTill;
                  FieldStringList.Add(sListFields);
                  ValueStringList.Add(sListValues);
                  sListFields := '';
                  sListValues := '';
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +
                  '''Corporate Sales Recurring Usage Credit'',';
                end;  // curAmountCharged <> 0.
              end;  // test if first time through the loop.
              // General information saved later.
              sServiceID := ''''+
                AccountFormat(Trim(ConvertString.Strings[DESC2_C])) +''',';
              curTotalCall := 0.0000;
            end  // sBillCode = '260500'.

            else if (sBillCode = '130000') then begin
              // General information saved later.
              lSaveToTable := False;
              // First (actually the 2nd line) useful line of file.
              // This appears only once in the file, near the beginning.
              sCallFrom := ''''+YMD2DMY(ConvertString.Strings[DESC2_C]) +''',';
              sCallTill := ''''+YMD2DMY(ConvertString.Strings[DESC3_C]) +''',';
              sRentFrom := ''''+YMD2DMY(ConvertString.Strings[DESC4_C]) +''',';
              sRentTill := ''''+YMD2DMY(ConvertString.Strings[DESC5_C]) +''',';
            end  // sBillCode = '130000'.

            // **********  Total to be read.  **********
            else if (sBillCode = '361005') then begin
              SubString := Trim(ConvertString.Strings[ABBR_C]);
              if (SubString = 'NSTDCH') then begin
                // Non-Standard Charge - only shown here so save it.
                sListFields := sListFields + 'TransactionTypeID, '+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sRentCodeID+','+
                  sSupplierRentCodeID+',';
                SubString := Trim(ConvertString.Strings[DESC2_C]);
                curAmountCharged := StrToCurr(SubString);
                if (Round(curAmountCharged*100)<>0) then begin
                  sListFields := sListFields + 'AmountExGST,GSTFlag,';
                  sListValues := sListValues + SubString +',''N'',';
                  sListFields := sListFields + 'AmountIncGST,GSTAmount,';
                  sListValues := sListValues + SubString +',0,';
                  sListFields := sListFields + 'FromDate,';
                  sListValues := sListValues + sRentFrom;
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues + sRentTill;
                  FieldStringList.Add(sListFields);
                  ValueStringList.Add(sListValues);
                  sListFields := '';
                  sListValues := '';
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+sRentCodeID+' '+
                    CleanString(Trim(ConvertString.Strings[DESC1_C])) +''',';
                end
                else begin
                  lSaveToTable := False;
                end;  // if (Round(StrToCurr(SubString)*100)<>0) then begin.
              end
              else begin
                lSaveToTable := False;
                if (SubString = 'USAGE') then begin
                  // Total calls.  (Used to check the data is correct.
                  SubString := Trim(ConvertString.Strings[DESC2_C]);
                  curActualCall := StrToCurr(SubString);
                end;  // USAGE.
                // Reset the running total.
                curTotalCall := 0.0000;
              end;  // NSTDCH.
            end  // '361005'.

            // **********  Payment required.  **********
            else if (sBillCode = '231099') then begin
              // Totals for invoice - save to correct GST calculations later.
              lSaveToTable := False;
              // This appears only once in the file, near the beginning.
              curTotalExGst  := StrToCurr(ConvertString.Strings[DESC1_C]);
              curTotalGst    := StrToCurr(ConvertString.Strings[DESC2_C]);
              curTotalIncGst := StrToCurr(ConvertString.Strings[DESC3_C]);
            end  // sBillCode = '231099'.

            else begin
              lSaveToTable := False;
            end;
          end;  // if lSaveToTable then.
          if lSaveToTable then begin
            // Now make the SQL statement to save to the database table.
            sListFields := sListFields + 'BatchID)';
            sListValues := sListValues + sBatch +')';
            FieldStringList.Add(sListFields);
            ValueStringList.Add(sListValues);
            qryStringList.Clear;
            qryStringList.Add('INSERT INTO tDataCollection');
            for nListValues := 0 to FieldStringList.Count-1 do begin
              qryStringList.Add(FieldStringList[nListValues]);
            end;
            for nListValues := 0 to ValueStringList.Count-1 do begin
              qryStringList.Add(ValueStringList[nListValues]);
            end;
            // Update and execute the query.
            qryDC.Close;
            qryDC.SQL := qryStringList;
            qryDC.ExecSQL;
          end;  // if lSaveToTable then begin.
          // Keep track of how much work has been done to date.
          Inc(nCount);
          Inc(nProgressCounted);
          if (nProgressCounted > PROGRESS_COUNTER_C) then begin
            // Report progress to the user.
            nSoFar := Round(nCount * nFactor);
            // Do not know how big the file is - make sure don't exceed 100%.
            if (nSoFar > PROGRESS_MAX_C) then
              Progress.ProgressBar1.Position := PROGRESS_MAX_C
            else
              Progress.ProgressBar1.Position := nSoFar;
            nProgressCounted := 0;
          end;  // if (nProgressCounted > PROGRESS_COUNTER_C) then.
          Progress.ProcessWinMessages(MainForm);    // Let windows update all messages.
//          Readln( InputTextFile, ReadString ); // Read in the next line of data.
        until Eof( InputTextFile );

        // Now save any air-time discounts for the last entry:
        // Use Int() to round down (& NOT up) to the nearest cent.
        curAmountCharged := Int(-nAirCreditPercent*curTotalCall)/100.0000;
        // Deduct discount sum from the total charge.
        curTotalCall := curSum(curAmountCharged, curTotalCall);
        if (Round(curAmountCharged*100)<>0) then begin
          FieldStringList.Clear;
          ValueStringList.Clear;
          sListFields := '(';
          sListValues := 'VALUES (';
          sListFields := sListFields + 'ServiceID,';
          sListValues := sListValues + sServiceID;
          sListFields := sListFields + 'TransactionTypeID, SupplierServiceCodeID,';
          sListValues := sListValues + sCallAirCodeID+','+sSupplierCallAirCodeID+',';
          sListFields := sListFields + 'AmountExGST,GSTFlag,';
          sListValues := sListValues +
            CurrToStr(curFormat(curAmountCharged)) +',''Y'',';
          curAmountGST := curFormat(curAmountCharged*0.1);
          sListFields := sListFields + 'AmountIncGST,GSTAmount,';
          sListValues := sListValues +
            CurrToStr(curSum(curAmountCharged,curAmountGST)) +','+
            CurrToStr(curAmountGST) +',';
          sListFields := sListFields + 'FromDate,';
          sListValues := sListValues + sCallFrom;
          sListFields := sListFields + 'ToDate,';
          sListValues := sListValues + sCallTill;
          FieldStringList.Add(sListFields);
          ValueStringList.Add(sListValues);
          sListFields := '';
          sListValues := '';
          sListFields := sListFields + 'RateDescription,';
          sListValues := sListValues +'''Vodafone airtime credit'',';
          // Now make the SQL statement to save to the database table.
          sListFields := sListFields + 'BatchID)';
          sListValues := sListValues + sBatch +')';
          FieldStringList.Add(sListFields);
          ValueStringList.Add(sListValues);
          qryStringList.Clear;
          qryStringList.Add('INSERT INTO tDataCollection');
          for nListValues := 0 to FieldStringList.Count-1 do begin
            qryStringList.Add(FieldStringList[nListValues]);
          end;
          for nListValues := 0 to ValueStringList.Count-1 do begin
            qryStringList.Add(ValueStringList[nListValues]);
          end;
          // Update and execute the query.
          qryDC.Close;
          qryDC.SQL := qryStringList;
          qryDC.ExecSQL;
        end;  // curAmountCharged <> 0.

        // Now save the Usage Credit - makes the numbers add up.
        FieldStringList.Clear;
        ValueStringList.Clear;
        curAmountCharged := curSum((-1*curTotalCall), curActualCall);
        if (Round(curAmountCharged*100)<>0) then begin
          sListFields := '(';
          sListValues := 'VALUES (';
          cAbbr := Trim(UpperCase(ConvertString.Strings[ABBR_C]));
          SubString := Trim(ConvertString.Strings[LINE_C]);
          if (Length(SubString) > 0) then begin
            sListFields := sListFields + 'SequenceNo,';
            sListValues := sListValues +''''+ SubString +''',';
          end;
          sListFields := sListFields + 'ServiceID,';
          sListValues := sListValues + sServiceID;
          sListFields := sListFields + 'TransactionTypeID, SupplierServiceCodeID,';
          sListValues := sListValues + sCallUsageCodeID+','+
                  sSupplierUsageCallCodeID +',';
          sListFields := sListFields + 'AmountExGST,';
          sListValues := sListValues + CurrToStr(curFormat(curAmountCharged)) +',';
          sListFields := sListFields + 'GSTFlag,';
          sListValues := sListValues + '''Y'',';
          curAmountGST := curFormat(curAmountCharged*0.1);
          sListFields := sListFields + 'AmountIncGST,GSTAmount,';
          sListValues := sListValues +
            CurrToStr(curSum(curAmountCharged,curAmountGST)) +','+
            CurrToStr(curAmountGST) +',';
          sListFields := sListFields + 'FromDate,';
          sListValues := sListValues + sCallFrom;
          sListFields := sListFields + 'ToDate,';
          sListValues := sListValues + sCallTill;
          FieldStringList.Add(sListFields);
          ValueStringList.Add(sListValues);
          sListFields := '';
          sListValues := '';
          sListFields := sListFields + 'RateDescription,';
          sListValues := sListValues +
            '''Corporate Sales Recurring Usage Credit'',';

          // Now make the SQL statement to save to the database table.
          sListFields := sListFields + 'BatchID)';
          sListValues := sListValues + sBatch +')';
          FieldStringList.Add(sListFields);
          ValueStringList.Add(sListValues);
          qryStringList.Clear;
          qryStringList.Add('INSERT INTO tDataCollection');
          for nListValues := 0 to FieldStringList.Count-1 do begin
            qryStringList.Add(FieldStringList[nListValues]);
          end;
          for nListValues := 0 to ValueStringList.Count-1 do begin
            qryStringList.Add(ValueStringList[nListValues]);
          end;
          // Update and execute the query.
          qryDC.Close;
          qryDC.SQL := qryStringList;
          qryDC.ExecSQL;
        end;  // curAmountCharged <> 0.

        // Final balancing of the books (usually a/c GST is on total charge but
        // those calls < $0.50 calculate to nil.
        qryDC.Close;
        qryDC.SQL.Clear;
        qryDC.SQL.Add('SELECT SUM(tDataCollection.AmountExGST) AS SumAmountExGST,');
        qryDC.SQL.Add('SUM(tDataCollection.GSTAmount) AS SumGSTAmount,');
        qryDC.SQL.Add('SUM(tDataCollection.AmountIncGST) AS SumAmountIncGST');
        qryDC.SQL.Add('FROM tDataCollection');
        qryDC.SQL.Add('WHERE tDataCollection.BatchID = '+ sBatch);
        qryDC.Open;
        curTotalExGst  := curSum(curTotalExGst,
          qryDC.FieldByName('SumAmountExGST').AsCurrency*(-1));
        curTotalGst    := curSum(curTotalGst,
          qryDC.FieldByName('SumGSTAmount').AsCurrency*(-1));
        curTotalIncGst := curSum(curTotalIncGst,
          qryDC.FieldByName('SumAmountIncGST').AsCurrency*(-1));
        if (curTotalExGst <> 0) or
          (curTotalGst <> 0) or
          (curTotalIncGst <> 0) then begin
          // Save the errors as Rounding.
          FieldStringList.Clear;
          ValueStringList.Clear;
          sListFields := '(';
          sListValues := 'VALUES (';
          sListFields := sListFields + 'ServiceID,';
          sListValues := sListValues + '''Rounding'',';
          sListFields := sListFields + 'TransactionTypeID, SupplierServiceCodeID,';
          sListValues := sListValues + sOtherCodeID+','+
                  sSupplierOtherCodeID+',';
          sListFields := sListFields + 'AmountExGST,';
          sListValues := sListValues + CurrToStr(curTotalExGst) +',';
          sListFields := sListFields + 'GSTFlag,';
          sListValues := sListValues + '''Y'',';
          sListFields := sListFields + 'GSTAmount,';
          sListValues := sListValues + CurrToStr(curTotalGst) +',';
          sListFields := sListFields + 'AmountIncGST,';
          sListValues := sListValues + CurrToStr(curTotalIncGst) +',';
          sListFields := sListFields + 'FromDate,';
          sListValues := sListValues + sCallFrom;
          sListFields := sListFields + 'ToDate,';
          sListValues := sListValues + sCallTill;
          FieldStringList.Add(sListFields);
          ValueStringList.Add(sListValues);
          sListFields := '';
          sListValues := '';
          sListFields := sListFields + 'RateDescription,';
          sListValues := sListValues + '''Rounding'',';

          // Now make the SQL statement to save to the database table.
          sListFields := sListFields + 'BatchID)';
          sListValues := sListValues + sBatch +')';
          FieldStringList.Add(sListFields);
          ValueStringList.Add(sListValues);
          qryStringList.Clear;
          qryStringList.Add('INSERT INTO tDataCollection');
          for nListValues := 0 to FieldStringList.Count-1 do begin
            qryStringList.Add(FieldStringList[nListValues]);
          end;
          for nListValues := 0 to ValueStringList.Count-1 do begin
            qryStringList.Add(ValueStringList[nListValues]);
          end;
          // Update and execute the query.
          qryDC.Close;
          qryDC.SQL := qryStringList;
          qryDC.ExecSQL;
        end;
        lReturnCode := True;
      end;  // if (Length( ReadString ) > 0) or (not Eof( InputTextFile )) then.
    finally
      // Done with the input file and resources, close them.
      CloseFile( InputTextFile );
      Progress.Close;
      ConvertString.Free;
      FieldStringList.Free;
      ValueStringList.Free;
      qryStringList.Free;
      qryDC.Free;
      Progress.Close;
    end;
  end;  // if lContinue then begin
end;  // procedure GetMobileData().


procedure VodafoneInvoice( const nBatch: LongInt; sDirectory: string );
// This function prompts the user for the name of the Vodafone supplied
// text file and then calls the VodaMobile function (which reads the
// file into a table and controls its processing).
var
  OpenDlg : TOpenDialog;
  lContinue : Boolean;
  EmptyList : TStringList;
  sFile : String;
begin
  // First, we must establish which file is the one to be processed.
  OpenDlg := TOpenDialog.Create(MainForm);
  OpenDlg.Filter := 'Vodafone Billing file (*.txt)|*.TXT|All files (*.*)|*.*';
  OpenDlg.FilterIndex := 1;
  // Use the program directory as the default directory.
  if (Length(sDirectory) > 0) then OpenDlg.InitialDir := sDirectory
  else OpenDlg.InitialDir := ExtractFilePath(ParamStr(0));
  OpenDlg.Title := 'Select The Vodafone Input File';
  // Read and save the user's choices before continuing.
  lContinue := OpenDlg.Execute;
  sFile := OpenDlg.FileName;
  OpenDlg.Free;
  if lContinue and (Length(Trim(sFile)) > 0) then begin
    if FileExists(sFile) then begin
      EmptyList := TStringList.Create;     // Removes all entries for the batch.
      DeleteFromDC( nBatch, EmptyList, lContinue);
      EmptyList.Free;
      GetMobileData(nBatch,sFile,lContinue);
      UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_IMPORT);

      // **************************************************************************
      // **************************************************************************
      // Test for critical errors that prevent progress to the next stage.
      lContinue := False;
      CriticalErrorTest( nBatch, lContinue );
      if lContinue then begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Passed all tests for Critical Errors.', mtInformation,
          [mbOk], 0);
      end
      else begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Critical errors found.'#13#10#13#10'These must be resolved ' +
          'before this invoice can be progressed.',
          mtWarning, [mbOk], 0);
      end;
    end
    else begin
      MessageDlg('That file does not exist.', mtWarning, [mbAbort], 0 );
    end;  // if FileExists(sFile) then begin.
  end;  // if (Length(Trim(sFile)) > 0) then begin.
end;  // procedure VodafoneInvoice().


end.
